const upperFirst = require('lodash/upperFirst');
const escape = require('lodash/escape');
const takeRight = require('lodash/takeRight');
const difference = require('lodash/difference');

$(document).ready(function() {
  var directories = window.location.pathname.split('/');
  if (directories[directories.length - 1] === '') {
    directories.pop();
  }
  if (directories[directories.length - 3] === 'webinars' && directories[directories.length - 1] === 'play') {
    Chat.initialize('public');
  }
});

var Chat = {

  initialize: function(side) {
    Chat.side = side;
    if (side == 'public') {
      Chat.webinarId = document.getElementById("tracking-data").dataset.id;
    } else {
      Chat.webinarId = $('.webinar-id-div').html();
    }
    const firebaseAccessToken = document.querySelector('meta[property="firebaseAccessToken"]').content;
    Chat.authPromise = firebase.auth().signInWithCustomToken(firebaseAccessToken);

    Chat.authPromise.then(function() {
      Chat.webinarReference = firebase.database().ref(Chat.webinarId);
      if (side == 'public') {
        Chat.webinarReference.on('value', WebinarPlay.firebaseUpdate.bind(WebinarPlay), (e) => {
          console.log(e);
        });
      }
      $('textarea').on('keydown', Chat.growTextArea);
      $('.chat-modal-background').click(Chat.closeModal);
      Common.clickOrEnter({
        click: '.send-button',
        enter: 'textarea',
        action: Chat.clickPostButton
      });
    });
  },

  // render chat messages ---

  updateChatSection: function(obj) {

    // don't update if user hasn't joined chat yet
    if (Chat.side == 'public' && !WebinarPlay.userCookie) {
      return;
    }

    // determine which messages need to be added
    var messages = obj.messages || {};
    var firstLoad = !Chat.messages;
    var newKeys;
    if (firstLoad) {
      Chat.messages = messages;
      newKeys = takeRight(Object.keys(Chat.messages), 50);
    } else {
      newKeys = difference(Object.keys(messages), Object.keys(Chat.messages));
      newKeys.forEach(function(key) {
        Chat.messages[key] = messages[key];
      });
    }

    newKeys.forEach(function(key) {
      // create the html element
      var message = messages[key];
      var $newElement = $('<div/>');
      if (message.image) {
        $newElement.html(Chat.bubbleHtml(messages, key) + Chat.initialsContainerHtml(message));
        $newElement.addClass('teachable');
      } else {
        $newElement.html(Chat.initialsContainerHtml(message) + Chat.bubbleHtml(messages, key) + Chat.replyHtml(messages, key));
      }
      $newElement.addClass('chat-message');
      if (Chat.side == 'admin') {
        if (message['user-blocked']) {
          $newElement.addClass('user-blocked');
        }
        if (message['message-blocked']) {
          $newElement.addClass('message-blocked');
        }
        if (message['replyPrivate']) {
          $newElement.addClass('private');
        }
      }
      $newElement[0].dataset.id = key;
      $newElement[0].dataset.user = message.user;
      $newElement[0].dataset.name = message.name;

      // append the element in all the right places
      if (Chat.side == 'public') {
        if ((!message['message-blocked'] && !message['replyPrivate']) || (message['replyPrivate'] && Chat.messageByThisUser(message)) || (message['message-blocked'] && Chat.messageByThisUser(message))) {
          Chat.appendAndMaybeScroll($('.chat-messages'), $newElement);
        }
        if (Chat.messageByThisUser(message) && message.replyText) {
          Chat.appendAndMaybeScroll($('.reply-chat-messages'), $newElement.clone());
          if (!$('.chat-sidebar ul li:nth-of-type(3)').hasClass('active') && !firstLoad) {
            $('.chat-sidebar ul li:nth-of-type(3) img.notification').show();
          }
        }
      } else {
        Chat.appendAndMaybeScroll($('.chat-messages.all'), $newElement);
        if (message['message-blocked']) {
          Chat.appendAndMaybeScroll($('.chat-messages.blocked'), $newElement.clone());
        } else {
          var foundQuestionWord = false;
          var questionWords = ['?', 'are there', 'who', 'what', 'when', 'why', 'where', 'can i'];
          for (var i = 0; i < questionWords.length; i++) {
            if (message.text.indexOf(questionWords[i]) > -1) {
              foundQuestionWord = true;
              Chat.appendAndMaybeScroll($('.chat-messages.questions'), $newElement.clone());
              break;
            }
          }
          if (foundQuestionWord == false) {
            if (message['text'].length >= 120) {
              Chat.appendAndMaybeScroll($('.chat-messages.comments'), $newElement.clone());
            } else {
              Chat.appendAndMaybeScroll($('.chat-messages.hello'), $newElement.clone());
            }
          }
        }
      }
    });
  },

  messageByThisUser: function(message) {
    if (WebinarPlay.userCookie) {
      return WebinarPlay.userCookie['id'] == message['user'];
    } else {
      return false;
    }
  },

  initialsContainerHtml: function(message, reply) {
    var properImage = (reply ? message.replyImage : message.image);
    var styleString = (properImage ? ("background-image: url('" + properImage + "')") : "background-color:" + message.color);
    return '<div class="initials-container"><div class="initials" style="' + styleString + ';">' + Chat.getInitials(message, reply) + '</div>' + '<div class="triangle"></div></div>';
  },

  getInitials: function(message, reply) {
    if (message.image || reply) {
      return '<img class="teachable" src="' + window.Images.teachableSymbol + '" />';
    } else {
      var words = message.name.split(" ");
      return upperFirst(words[0].split('')[0]) + (words[1] ? upperFirst(words[1].split('')[0]) : '') + (Chat.side == 'admin' ? '<img class="user-blocked" src="' + window.Images.blockSymbol + '" />' : '');
    }
  },

  bubbleHtml: function(messages, key) {
    var startingDivTag = (Chat.side == 'admin' ? '<div class="bubble" onclick="Chat.openReplyModal(this, false);">' : '<div class="bubble">');
    var result = startingDivTag + '<span>' + messages[key].name + ':</span> ' + messages[key].text + '</div>';
    if (Chat.side == 'admin' && messages[key].color) {
      result = (result + '<div class="buttons"><div class="reply-button" onclick="Chat.openReplyModal(this, false);"></div><div class="private-button" onclick="Chat.openReplyModal(this, true);"></div>' + (messages[key]['user-blocked'] ? '' : '<div class="block-button" onclick="Chat.clickBlock(this);"></div>') + '</div>');
    }
    if (messages[key].image) {
      result = Chat.replaceURLWithHTMLLinks(result);
    }
    return result;
  },

  replaceURLWithHTMLLinks: function(text) {
    var exp = /((([A-Za-z]{3,9}:(?:\/\/)?)(?:[\-;:&=\+\$,\w]+@)?[A-Za-z0-9\.\-]+|(?:www\.|[\-;:&=\+\$,\w]+@)[A-Za-z0-9\.\-]+)((?:\/[\+~%\/\.\w\-_]*)?\??(?:[\-\+=&;%@\.\w_]*)#?(?:[\.\!\/\\\w]*))?)/;
    return text.replace(exp, "<a href='$1' target='_blank' rel='noopener noreferrer'>$1</a>");
  },

  replyHtml: function(messages, key) {
    var message = messages[key];
    if (message.replyText) {
      result = '<div class="teachable chat-message"><img class="reply-arrow" src="' + window.Images.replyArrowBig + '" /><div class="bubble reply-bubble"><span>' + message.replyName + ':</span> ' + message.replyText + '</div>';
      result = Chat.replaceURLWithHTMLLinks(result);
      result += (Chat.initialsContainerHtml(message, true) + '</div>');
      return result;
    } else {
      return '';
    }
  },

  // post a new message ---

  growTextArea: function(e) {
    if (e.keyCode == 13) {
      if (e.target.value === '') {
        e.preventDefault();
      }
    } else {
      Common.autoGrow(e.target, 64);
    }
  },

  clickPostButton: function() {
    var text = escape($('textarea').val());
    if (text) {
      var userBlocked = false;
      if (Chat.side == 'public') {
        var user = WebinarPlay.userCookie['id'];
        Chat.webinarReference.child('users').child(user).once("value", function(snap) {
          userBlocked = snap.val().blocked;
        });
      }
      var options = {
        text: text,
        name: (Chat.side == 'public' ? WebinarPlay.userCookie.name : Chat.employeeName()),
        user: (Chat.side == 'public' ? WebinarPlay.userCookie.id : Chat.employeeUserId())
      };
      options['user-blocked'] = userBlocked;
      options['message-blocked'] = Chat.messageContainsCompetitors(text) || userBlocked;
      options['date-created'] = Math.floor(Date.now() / 1000);
      if (Chat.side == 'public') {
        options.color = WebinarPlay.userCookie.color;
      }
      if (Chat.side == 'admin') {
        options.image = $('.image-div').html();
      }
      Chat.webinarReference.child('messages').push().set(options, function() {
        Chat.resetMessageArea();
      });
    }
  },

  messageContainsCompetitors: function(text) {
    var competitors = ['thinkific', 'kajabi', 'podia'];
    var lowerCaseText = text.toLowerCase();
    for (var i = 0; i < competitors.length; i++) {
      if (lowerCaseText.indexOf(competitors[i]) > -1 ) {
        return true;
      }
     }
     return false;
  },

  resetMessageArea: function() {
    var $textarea = $('textarea');
    $textarea.val('');
    $textarea.css('height', 64);
  },

  appendAndMaybeScroll: function($bigElement, $newElement) {
    var atBottom = Common.elementIsScrolledToBottom($bigElement[0]);
    $bigElement.append($newElement);
    if ($bigElement.children(".chat-message").length > 1000) {
      $bigElement.children(".chat-message").first().remove();
    }
    if (atBottom) {
      Common.scrollOverflowElementToBottom($bigElement[0]);
    }
  },

  // admin message actions ---

  clickBlock: function(t) {
    var user = t.parentElement.parentElement.dataset.user;
    var name = t.parentElement.parentElement.dataset.name;
    var $background = $('.chat-modal-background');
    $background.show();
    var $modal = $("<div />");
    $modal.addClass("confirm-block-modal");
    $modal.html("<p>Are you sure you want to block " + name + "?</p>");
    var $yes = $("<a />");
    $yes.addClass('modal-button');
    $yes.html("Block");
    $yes.click(Chat.confirmBlock.bind(this, user))
    var $no = $("<a />");
    $no.addClass('modal-button');
    $no.html("Cancel");
    $no.click(Chat.closeModal);
    $modal.append($yes);
    $modal.append($no);
    $('body').append($modal);
  },

  confirmBlock: function(user) {
    Chat.webinarReference.child('users').child(user).update({ "blocked": true });
    var messagesRef = Chat.webinarReference.child('messages');
    messagesRef.orderByChild('user').equalTo(user).once("value", function(snap) {
      if (snap) {
        var messageIds = Object.keys(snap.val());
        messageIds.forEach(function(id) {
          Chat.webinarReference.child('messages').child(id).update({ "user-blocked": true });
          $("div[data-user='" + user + "']").addClass('user-blocked');
          $("div[data-user='" + user + "'] .buttons .block-button").hide();
        });
      }
    });
    Chat.closeModal();
  },

  openReplyModal: function(t, isPrivate) {
    var id = t.parentElement.dataset.id || t.parentElement.parentElement.dataset.id;
    var $background = $('.chat-modal-background');
    $background.show();
    var $modal = $("<div />");
    $modal.addClass("reply-modal");
    if (isPrivate) {
      $modal.addClass("private");
    }
    $modal.addClass("chat-messages");
    $modal.html("<p>Replying " + (isPrivate ? "privately " : "") + "to:</p>");
    var $message = $($("div[data-id='" + id + "']")[0].cloneNode(true));
    $message.children().prop('onclick', null);
    $modal.append($message);
    $modal.append('<textarea />');
    var $buttonDiv = $('<div />');
    $buttonDiv.addClass("text-center");
    var $yes = $("<a />");
    $yes.addClass('modal-button');
    $yes.html("Reply");
    var $no = $("<a />");
    $no.addClass('modal-button');
    $no.html("Cancel");
    $no.click(Chat.closeModal);
    $buttonDiv.append($yes);
    $buttonDiv.append($no);
    $modal.append($buttonDiv);
    $('body').append($modal);
    $('.reply-modal textarea')[0].focus();
    Common.clickOrEnter({
      click: '.reply-modal a:first-of-type',
      enter: '.reply-modal textarea',
      action: Chat.sendReply.bind(this, id, isPrivate)
    });
  },

  sendReply: function(id, isPrivate) {
    var $message = $("div[data-id='" + id + "']");
    var id = $message.data().id;
    var messagesRef = Chat.webinarReference.child('messages');
    messagesRef.child(id).once("value", function(snap) {
      var obj = snap.val();
      obj.replyText = $('.reply-modal textarea').val();
      obj.replyName = Chat.employeeName();
      obj.replyUser = Chat.employeeUserId();
      obj.replyImage = $('.image-div').html();
      obj.replyPrivate = isPrivate;
      Chat.webinarReference.child('messages').push().set(obj);
    });
    Chat.closeModal();
  },

  closeModal: function() {
    var $background = $('.chat-modal-background');
    $background.hide();
    var $blockModal = $('.confirm-block-modal');
    $blockModal.remove();
    var $replyModal = $('.reply-modal');
    $replyModal.remove();
  },

  // helper methods

  employeeUserId: function() {
    return 'teachable-' + $('.id-div').html();
  },

  employeeName: function() {
    return $('.name-div').html() + ', Teachable';
  }
}

module.exports = Chat;
