// ensure fetch sends csrf token
const makeHeaders = (headers: HeadersInit = {}) => {
  const metaEl = document.querySelector('meta[name=csrf-token]') as HTMLMetaElement;
  if (!metaEl) {
    return headers
  }
  const token = metaEl.content
  if (headers instanceof Headers) {
    headers.set('X-CSRF-Token', token)
    headers.set('X-Requested-With', 'XMLHttpRequest')

    return headers
  } else if (Array.isArray(headers)) {
    return headers.concat([
      ['X-CSRF-Token', token],
      ['X-Requested-With', 'XMLHttpRequest']
    ])
  }
  return {
    ...headers,
    'X-CSRF-Token': token,
    'X-Requested-With': 'XMLHttpRequest'
  }
}

const originalFetch = fetch;
window.fetch = (url: RequestInfo, init: RequestInit = {}) => {
  const request = typeof url === 'string' ? new Request(url) : url;
  const fixedInit = new URL(request.url).host === window.location.host ? {
    ...init,
    headers: makeHeaders(init.headers)
  } : init

  return originalFetch(url, fixedInit)
}
