$(document).ready(function() {
  Common.initialize();
});

var Common = {

  params: {},

  initialize: function() {
    $.fn.matchHeight._maintainScroll = true;
    Common.installSentryRaven();
    Common.localStorageDisabled = Common.isLocalStorageDisabled();
    Common.sessionStorageDisabled = Common.isSessionStorageDisabled();
    Common.storeURLParams();
    Common.addPreservedParamsToLinks();
    Common.launchSmoothScroll();
    Common.displayNotification();
    $('.match-height').matchHeight();
    $('[data-toggle="tooltip"]').tooltip();
  },

  checkEmailValidation: function(emailInput){
    var email = emailInput.value
    var suggestionResponse = window.EmailValidator.checkForSuggestion(email, "Are you sure you didn't mean %{suggested_email}?")
    if (suggestionResponse.suggestion) {
      return {
        suggestedEmail: suggestionResponse.suggestedEmail,
        error: suggestionResponse.error
      }
    }
  },

  checkIfOptedIntoEmails: function(){
    if (Common.localStorageDisabled) {
      return false;
    }

    var optedInChoice = localStorage.getItem('optedIntoMarketingEmails');
    return optedInChoice === 'true';
  },

  checkIfNeedsMarketingEmailOptIn: function(){
    if (Common.checkIfOptedIntoEmails() || Common.sessionStorageDisabled) {
      return false;
    }

    var requiresOptIn = sessionStorage.getItem('requiresMarketingEmailOptIn');
    return requiresOptIn === 'true';
  },

  displayOptInInput: function(inputElement) {
    $(inputElement).removeClass("hidden")
  },

  verifyIfMarketingEmailOptInNeeded: function(optInElement) {
    if (Common.checkIfNeedsMarketingEmailOptIn()) {
      Common.displayOptInInput(optInElement);
    } else {
      $.ajax({
        url: "/api/current_ip",
        success: function(response) {
          if (!Common.sessionStorageDisabled) {
            sessionStorage.setItem("requiresMarketingEmailOptIn", response.eu)
          }
          if (response.eu && !Common.checkIfOptedIntoEmails()) {
            Common.displayOptInInput(optInElement);
          }
        }
      });
    }
  },

  updateMarketingEmailOptInChoice: function(choice, email){
    localStorage.setItem('optedIntoMarketingEmails', choice)
    var iterableObj = {
      eventName: 'Opted Into Marketing Materials',
      email: email,
      instanceId: 1,
    };
    Iterable.sendEvent(iterableObj)
  },

  displayNotification: function() {
    if (Common.params.message_id) {
      $('.message-' + Common.params.message_id).removeClass('hidden');
    }
  },

  installSentryRaven: function() {
    if ($('#environment').data().env === "production") {
      Raven.config('https://8c77653f3051455f9feb5598c26590cc@sentry.io/1255863', {
        environment: 'production', // Change to 'development' test errors without polluting
      }).install();
    }
  },

  addPreservedParamsToLinks: function() {
    var preservedParamsObject = this.preservedParamsObject();
    if (Object.keys(preservedParamsObject).length > 0) {
      var $links = $('a[href^="/"], a[href*="teachable.com"]').not('a[href^="mailto:"]');
      $links.each(function() {
        var $thisLink = $(this);
        var existingParamsObject = $thisLink[0].search.substring(1).split('&').reduce((memo, paramPair) => {
          const [key, value] = paramPair.split('=');
          memo[key] = value;
          return memo;
        }, {});
        var mergedObject = { ...preservedParamsObject, ...existingParamsObject };
        var newHref = $thisLink.attr('href').replace(/\?.*/, '') + '?' + Tools.getStringFromObject(mergedObject, '&');
        $thisLink.attr('href', newHref);
      });
    }
  },

  preservedParamsObject: function () {
    whitelistedKeys = [
      "plan_id",
      "upgrade_src",
      "i_template_id",
      "i_campaign_id",
      "utm_source",
      "utm_medium",
      "utm_campaign",
      "utm_content",
      "utm_sales_page_cta",
      "mt_tier",
      "mt_interval",
      "mt_campaign_group",
      "coupon_code",
      "mbsy",
      "mbsy_source",
      "utm_keyword",
      "utm_adgroup",
      "cx"
    ]

    var result = {};
    for (var key in this.params) {
      if (whitelistedKeys.indexOf(key) > -1) {
        result[key] = this.params[key];
      }
    }
    return result;
  },

  autoGrow: function(element, minHeight) {
    element.style.height = minHeight;
    element.style.height = (element.scrollHeight > minHeight ? element.scrollHeight : minHeight) + "px";
  },

  clickOrEnter: function(object) {
    $(object['enter']).keypress(function(e) {
      if (e.keyCode == 13) {
        var this_object = object['enter'];
        object.action(e, this_object);
      }
    });
    $(object['click']).click(function(e) {
      var this_object = object['click'];
      object.action(e, this_object);
    });
  },

  elementIsScrolledToBottom: function(element) {
    return (element.scrollHeight - element.scrollTop) === element.offsetHeight;
  },

  getParamsToPassToProduct: function(includeAmbassador) {
    var result = [];
    var exclude = ["signup", "login", "headline", "subtitle", "mbsy", "campaignid", "mbsy_source"];
    var paramKeys = Object.keys(Common.params);
    paramKeys.forEach(function(paramKey) {
      if (exclude.indexOf(paramKey) == -1) {
        result.push(paramKey + "=" + Common.params[paramKey]);
      }
    });
    var queryString = result.join('&');
    return queryString ? ('&' + queryString) : "";
  },

  getRecaptchaSiteKey: function() {
    return $("meta[name='recaptcha-site-key']").attr("value");
  },

  getProperSSOEndPoint: function() {
    return $('meta[name="teachable-api-urls"]').data("baseUrl")
  },

  getProperMarketingApiEndpoint: function() {
    return $('meta[name="teachable-api-urls"]').data("marketingApiUrl")
  },

  getProperReferralsEndpoint: function() {
    return $('meta[name="teachable-api-urls"]').data("referralsUrl")
  },

  getSegmentEventObject: function(id) {
    var object = {};
    object.event = $.trim($('#' + id).attr('value'));
    return object;
  },

  getUrlDirectories: function() {
    var directories = window.location.pathname.split('/');
    if (directories[directories.length - 1] === '') {
      directories.pop();
    }
    return directories;
  },

  getVideoId: function(videoContainerId) {
    var elementToGet = '#' + videoContainerId + ' ' + '> iframe';
    var src = $(elementToGet).attr('src').split('/');
    return $.trim(src[src.length - 2]);
  },

  hitRealEndpoints: function() {
    return (["teachable.com", "teachable-cms.herokuapp.com", "teachable-cms-staging.herokuapp.com"].indexOf(document.domain) >= 0);
  },

  launchSmoothScroll: function() {
    $('a[href^="#"]').each(function(index) {
      if (($(this).attr('href').length > 1) && (String($(this).data('scroll')) != 'false')) {
        $(this).smoothScroll();
      }
    });
  },

  scrollOverflowElementToBottom: function(element) {
    element.scrollTop = element.scrollHeight;
  },

  storeURLParams: function() {
    var urlParamString = window.location.search.substring(1);
    if (urlParamString) {
      var urlParams = urlParamString.split('&');
      urlParams.forEach(function(param) {
        var paramKeyValuePair = param.split('=');
        // store all params in a nonpersistent Javascript object
        Common.params[paramKeyValuePair[0]] = paramKeyValuePair[1];
      });
    }

    // fill text of email fields with email param or local storage
    var email = this.getEmailFromLocalStorageOrParams();

    if (email) {
      email = decodeURIComponent(email);
      $('[type="email"]').val(email);
      $('input[data-input-type="email"]').val(email);
      $("#modal_email_field").val(email);
    }
  },

  getEmailFromLocalStorageOrParams: function() {
    var email = Common.params.email;
    if (Common.localStorageDisabled) {
      return email;
    }

    var formEmailExpiration = localStorage.getItem('formEmailExpiration');
    var formEmail = localStorage.getItem('formEmail');
    var localStorageEmail = localStorage.getItem('email');
    if (formEmailExpiration && parseInt(formEmailExpiration) < new Date().getTime()) {
      localStorage.removeItem('formEmail');
      localStorage.removeItem('formEmailExpiration');
      formEmail = '';
    }

    if ($('.sign-up-form, .single-page-checkout')[0] && formEmail) {
      email = formEmail;
    } else if (localStorageEmail) {
      email = localStorageEmail
    }

    return email;
  },

  isLocalStorageDisabled: function() {
    try {
      localStorage.setItem('localStorageTest', 'true');
      localStorage.removeItem('localStorageTest');
      return false;
    } catch (e) {
      Tracking.irisEmit('local_storage_disabled', { target: 'segment' });
      return true;
    }
  },

  isSessionStorageDisabled: function() {
    try {
      sessionStorage.setItem('sessionStorageTest', 'true');
      sessionStorage.removeItem('sessionStorageTest');
      return false;
    } catch (e) {
      Tracking.irisEmit('session_storage_disabled', { target: 'segment' });
      return true;
    }
  },

  paramsObjectToSearchString: function(obj) {
    var result = [];
    var keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
      result.push(keys[i] + "=" + obj[keys[i]]);
    }
    return "?" + result.join("&");
  },

  playWistiaVideo: function(videoId) { // this function is from the Wistia api
    window._wq = window._wq || [];
    _wq.push({ id: videoId, onReady: function(video) {
    video.play();
    }});
  },

  redirectWithPreservedParams: function(redirectUrl) {
    var preservedParamsObject = Common.preservedParamsObject();
    if (Object.keys(preservedParamsObject).length > 0) {
      var separator = redirectUrl.indexOf('?') > -1 ? '&' : '?';
      window.location.href = redirectUrl + separator + Tools.getStringFromObject(preservedParamsObject, '&');
    } else {
      window.location.href = redirectUrl
    };
  },

  validateEmail: function(email) {
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
  },

  parseCookies: function() {
    return Object.fromEntries(document.cookie.split('; ').map(s => s.split('=')))
  }
}

module.exports = Common;
