const { sendEmailSubmitConversion } = require('./conversion-events');

$(document).ready(function() {
  ContentDownloadLandingPage.initialize();
});

var ContentDownloadLandingPage = {

  IRIS_EVENT_NAME: 'marketing_submitted_content_download_request',

  $container: null,
  $emailInput: null,
  $identificationSelect: null,

  initialize: function() {
    this.$container = $('.content-download-landing-page');

    this.$emailInput = this.getEl('#email-input');
    this.$identificationSelect = this.getEl('#identification-select');

    this.$emailInput.on('keydown', this.clearErrors.bind(this));
    this.$identificationSelect.on('change', this.clearErrors.bind(this));
    this.getEl('form').on('submit', this.onSubmit.bind(this));
  },

  getEl: function(selector) { return this.$container.find(selector); },

  formData: function() {
    return {
      email: this.$emailInput.val(),
      identification: this.$identificationSelect.val(),
    }
  },

  validate: function() {
    var isValid = true;
    var formData = this.formData();

    var isValidEmail = Common.validateEmail(formData.email);
    if (!isValidEmail) {
      this.$emailInput.addClass('error');
      isValid = false;
    }

    var isValidIdentification = !!formData.identification;
    console.log("valid id:", !!formData.identification);

    if (!isValidIdentification) {
      this.$identificationSelect.addClass('error');
      isValid = false;
    }

    return isValid;
  },

  clearErrors: function() {
    this.$emailInput.removeClass('error');
    this.$identificationSelect.removeClass('error');
  },

  onSubmit: function(e) {
    e.preventDefault()
    if (!this.validate()) return

    var formData = this.formData();
    localStorage.setItem('email', formData.email);
    this.emitIrisData();
    sendEmailSubmitConversion();

    // we don't get back a promise here, so we hope for the
    // best with a 500ms delay
    setTimeout(this.redirect, 500);
  },

  emitIrisData: function() {
    var formData = this.formData();

    var contentInfo = {
      cta: {
        text: this.getEl('button[type="submit"]')[0].innerText,
        url: this.getEl(".redemption-link")[0].innerText
      },
      title: this.getEl("h1.default-header")[0].innerText,
      body_text: this.getEl(".main-text-container").html(),
    };
    Tracking.irisEmit(this.IRIS_EVENT_NAME, {
      eventName: 'Marketing - Submitted Content Download Request',
      email: formData.email,
      creator_type: formData.identification,
      content_info: contentInfo,
      instanceId: 1,
      isEmailSubmittedEvent: true,
      target: 'iterable',
    });

     /*
      This iterable event is only temporary. All data SHOULD be able to post
      only to Iris, but something is broken in our flow and to stop the bleeding
      we've submitted the event directly here:
    */
    Iterable.sendEvent({
      eventName: this.IRIS_EVENT_NAME,
      email: formData.email,
      instanceId: 1,
      isEmailSubmittedEvent: true,
      data: {
        ...contentInfo,
        creator_type: formData.identification,
      }
    });
  },

  redirect: function() {
    var customRedirectDiv = $('#custom-redirect')[0];
    if (customRedirectDiv) {
      window.location.href = customRedirectDiv.dataset.url;
    } else {
      var slug = $('#content-download-payload')[0].dataset.slug
      window.location.href = "/get/" + slug + "/thank-you"
    }
  },
}
