$(document).ready(function() {
  EnterpriseForm.initialize();
});

var EnterpriseForm = {

  initialize: function() {
    $('.enterprise-form select').niceSelect();
    $('.form .orange-button').on('click', EnterpriseForm.clickSubmitButton);
    $('.form #form-name').on('keydown', EnterpriseForm.clearNameError);
    $('.form #form-company').on('keydown', EnterpriseForm.clearCompanyError);
    $('.form #form-company-email').on('keydown', EnterpriseForm.clearCompanyEmailError);
    $('.form #form-phone').on('keydown', EnterpriseForm.clearPhoneError);
    $('.form #form-message').on('keydown', EnterpriseForm.clearMessage);
    $($('.form .select-container')[0]).on('change', EnterpriseForm.clearPlanError);
    $($('.form .select-container')[1]).on('change', EnterpriseForm.clearRevenueError);
    $($('.form .select-container')[2]).on('change', EnterpriseForm.clearUsersError);
  },

  clearNameError: function() {
    $("#error-name").removeClass("show");
  },

  clearCompanyError: function() {
    $("#error-company").removeClass("show");
  },

  clearCompanyEmailError: function() {
    $("#error-company-email").removeClass("show");
  },

  clearMessage: function() {
    $("#error-message").removeClass("show");
  },

  clearPhoneError: function() {
    $("#error-phone").removeClass("show");
  },

  clearPlanError: function() {
    $("#error-plan").removeClass("show");
  },

  clearRevenueError: function() {
    $("#error-revenue").removeClass("show");
  },

  clearUsersError: function() {
    $("#error-users").removeClass("show");
  },

  clickSubmitButton: function(e) {
    e.preventDefault();
    var name = $('.form #form-name').val();
    var company = $('.form #form-company').val();
    var companyEmail = $('.form #form-company-email').val();
    var phone = $('.form #form-phone').val();
    var message = $('.form #form-message').val();
    var plan = $('.form #form-plan').val();
    var revenue = $('.form #form-revenue').val();
    var users = $('.form #form-users').val();

    var nameOK = name.length > 0;
    var companyOK = company.length > 0;
    var companyEmailOK = Common.validateEmail(companyEmail);
    var phoneOK = phone.length > 0;
    var messageOK = message.length > 0;
    var planOK = plan.length > 0;
    var revenueOK = revenue.length > 0;
    var usersOK = users.length > 0;

    if (!nameOK) {
      $("#error-name").addClass("show");
    }
    if (!companyOK) {
      $("#error-company").addClass("show");
    }
    if (!companyEmailOK) {
      $("#error-company-email").addClass("show");
    }
    if (!phoneOK) {
      $("#error-phone").addClass("show");
    }
    if (!messageOK) {
      $("#error-message").addClass("show");
    }
    if (!planOK) {
      $("#error-plan").addClass("show");
    }
    if (!revenueOK) {
      $("#error-revenue").addClass("show");
    }
    if (!usersOK) {
      $("#error-users").addClass("show");
    }

    if (nameOK && companyOK && companyEmailOK && phoneOK && messageOK && planOK && revenueOK && usersOK) {
      EnterpriseForm.sendEnterpriseData(name, company, companyEmail, phone, message, plan, revenue, users);
    }
  },

  sendEnterpriseData: function(name, company, companyEmail, phone, message, plan, revenue, users) {
    var data = {
      name: name,
      enterprise_company: company,
      phone: phone,
      enterprise_sales_message: message,
      enterprise_how_will_use_teachable: plan,
      enterprise_project_revenue: revenue,
      enterprise_projected_users: users
    };
    Tracking.irisEmit('enterprise_request_submitted', {
      email: companyEmail,
      name: name,
      enterprise_company: company,
      phone: phone,
      enterprise_sales_message: message,
      enterprise_how_will_use_teachable: plan,
      enterprise_project_revenue: revenue,
      enterprise_projected_users: users,
      target: 'segment'
    });
    EnterpriseForm.showConfirmation();
  },

  showConfirmation: function() {
    var $form = $('.form');
    $form.empty();
    $form.append("<div class='thank-you'><p class='default-paragraph'>Thank you for your interest in Teachable! We'll be in touch shortly.</p></div>");
    $form.css('padding-bottom', '45px');
    $('html, body').animate({
      scrollTop: $(".enterprise-form .form").offset().top
    }, 100);
  }
}
