$(document).ready(function() {
  EventSeriesSchedule.initialize();
});

var ONE_SECOND = 1000;
var ONE_MINUTE = ONE_SECOND * 60;
var ONE_HOUR = ONE_MINUTE * 60;
var MAX_TIMER = 2147483647; // setInterval/setTimeout has a max timer duration of 2^31ms

var webinarStatuses = {
  scheduled: 'SCHEDULED',
  startingSoon: 'SOON', //https://dreamcurrents.files.wordpress.com/2008/05/horse_window_4.jpg
  live: 'STARTED',
  ended: 'ENDED',
  replayUploaded: 'UPLOADED',
};

var webinarData;
var initialLoad = $.Deferred();
var EventSeriesSchedule = {

  initialize: function() {
    if ($('.event-series-schedule')[0]) {
      EventSeriesSchedule.startUpdater();
      initialLoad.then(EventSeriesSchedule.updateLinks.bind(this));
      $('section.event-series-schedule .add-event-to-calendar-button').click(EventSeriesSchedule.clickAddToCalendarButton);
      $('li.add-event-to-calendar-link').click(EventSeriesSchedule.clickCalendarType);
      EventSeriesSchedule.showDay();
    }
  },

  showDay: function() {
    if (Common.params.day) {
      var tabId;
      var dayParam = Common.params.day;
      if (dayParam === 'additional') {
        tabId = 'additional-content';
      } else if (!isNaN(+dayParam)) {
        tabId = 'day-' + dayParam;
      }
      if (tabId) {
        var navElement = document.querySelector('[href="#' + tabId + '"]');
        navElement.click();
        document.querySelector('.event-series-schedule').scrollIntoView();
      }
      return;
    }
    var now = Math.floor(Date.now() / 1000);
    var dates = document.querySelectorAll('[data-day-start]');
    dates.forEach(function (el) {
      var dayStart = +el.dataset.dayStart;
      if ((dayStart < now) && (now < (dayStart + (24 * ONE_HOUR) / 1000))) {
        el.click();
      }
    });
  },

  getWebinarData: function() {
    if (webinarData) {
      return webinarData;
    }
    var webData = $('.event-series-schedule .event-link').map(function(_idx, el) {
      return {
        webinarId: el.dataset.webinarId,
        eventStart: parseInt(el.dataset.eventStart),
        eventUrl: el.dataset.eventUrl,
        eventDay: el.dataset.eventDay
      };
    }).get();
    webinarData = webData.reduce(function(memo, data) {
      memo[data.webinarId] = data;
      return memo;
    }, {});
    return webinarData;
  },

  startUpdater: function() {
    var webinarData = EventSeriesSchedule.getWebinarData();
    var webinarsToCheck = Object.keys(webinarData);
    var monitoring = false;
    firebase.database().ref().on('value', function(snap) {
      var firebaseData = snap.val();
      if (firebaseData) {
        var endedWebinars = webinarsToCheck
            .filter(function(webinarId) { return firebaseData[webinarId].controls.hangoutStatus === 'End' });
        if (endedWebinars.length) {
          webinarsToCheck = webinarsToCheck.filter(function(webinarId) { return endedWebinars.indexOf(webinarId) === -1 });
          EventSeriesSchedule.checkForUpdates(endedWebinars);
        } else {
          initialLoad.resolve();
        }
      }
    }.bind(this));
  },

  checkForUpdates: function(endedWebinars) {
    var eventSeriesId = $('.event-series-schedule').data('event-series-id');
    var curWebinars = endedWebinars;
    var checkInterval = 20 * ONE_SECOND;
    var checkCms = function() {
      return $.ajax({
        url: '/api/event_series_schedule/' + eventSeriesId,
      }).then(function(cmsData) {
        initialLoad.resolve();
        // partition our webinars into ones with urls and ones without
        var statuses = curWebinars.reduce(function(memo, webinarId) {
          if (cmsData[webinarId]) {
            memo.hasReplay.push(webinarId);
          } else {
            memo.noReplay.push(webinarId);
          }
          return memo;
        }, { hasReplay: [], noReplay: []});

        if (!statuses.noReplay.length) {
          clearInterval(interval);
        } else {
          curWebinars = statuses.noReplay;
        }
        statuses.hasReplay.forEach(function(webinarId) { EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.replayUploaded) });
        return statuses;
      }.bind(this));
    };
    var interval = function () {
      return checkCms().then(function (statuses) {
        if (!!statuses.noReplay.length) {
          setTimeout(interval, checkInterval);
        }
        return statuses;
      }.bind(this));
    };
    interval().then(function (statuses) {
      statuses.noReplay.forEach(function(webinarId) { EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.ended) });
    });
  },

  updateEventLink: function(webinarId, status) {
    var $eventLink = $('.event-series-schedule .event-link[data-webinar-id=' + webinarId + ']').first();
    var href;
    switch (status) {
      case webinarStatuses.startingSoon: {
        href = '/webinars/' + webinarData[webinarId].eventUrl + '/play';
        $eventLink.html('<a href="' + href +'" class="event-type">Starting Soon - Grab Your Seat</a>');
        break;
      }
      case webinarStatuses.live: {
        href = '/webinars/' + webinarData[webinarId].eventUrl + '/play';
        $eventLink.html('<a href="' + href +'" class="event-type">We\'re Live! Click to Join</a>');
        break;
      }
      case webinarStatuses.ended: {
        $eventLink.html('<span class="event-type disabled">Session ended -- Replay Coming Soon</span>');
        break;
      }
      case webinarStatuses.replayUploaded: {
        href = '/webinars/' + webinarData[webinarId].eventUrl + '/replay';
        $eventLink.html('<a href="' + href +'" class="event-type">Watch Replay</a>');
        $eventLink.parents('.tab-pane').first().find('.replay-header-text').removeClass('hidden');
        break;
      }
      case webinarStatuses.scheduled: {
        $eventLink.find('.spinner').remove();
        $eventLink.find('.add-event-to-calendar-button-container').removeClass('hidden');
      }
    }
  },

  updateLinks: function() {
    var webinarData = EventSeriesSchedule.getWebinarData();
    var currentTime = Math.round(new Date().getTime() / 1000);
    var reload = function () { window.location.reload() };
    Object.keys(webinarData).forEach(function(webinarId) {
      var webinar = webinarData[webinarId];
      var liveDiff = Math.min(MAX_TIMER, Math.max(webinar.eventStart - currentTime, 0) * 1000); // switch back to milliseconds for setTimeout
      var soonDiff = Math.min(MAX_TIMER, Math.max(liveDiff - ONE_HOUR, 0));
      // skip all this if the event's already started
      if (liveDiff === 0) {
        EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.live);
      } else {
        var soonCb = soonDiff === MAX_TIMER
            ? reload
            : function() { EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.startingSoon) }.bind(this);
        var liveCb = liveDiff === MAX_TIMER
            ? reload
            : function() { EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.live) }.bind(this);

        EventSeriesSchedule.updateEventLink(webinarId, webinarStatuses.scheduled);
        setTimeout(soonCb, soonDiff);
        setTimeout(liveCb, liveDiff);
      }
    });
  },

  clickAddToCalendarButton: function(e) {
    $(e.target).parent().next('.dropdown-container').toggle();
    Tracking.irisEmit('button_clicked');
  },

  clickCalendarType: function(e) {
    Tracking.irisEmit('button_clicked');
    var $startDateDiv = $(e.target).closest(".dropdown-container").find(".formatted-start-time");
    var $endDateDiv = $(e.target).closest(".dropdown-container").find(".formatted-end-time");

    var startTimeString = $startDateDiv.html();
    var startTime = new Date(startTimeString);

    var endTimeString = $endDateDiv.html();
    var endTime = new Date(endTimeString);

    var webinarTitle = $(e.target).closest(".event-link").data("event-title");
    var playUrl = $(e.target).closest(".event-link").data("event-url");
    switch (e.target.innerHTML) {
      case 'Google calendar':
        $(e.target).closest('.add-event-to-calendar-button-container').find('a.google')[0].click();
        break
      case 'Yahoo! calendar':
        $(e.target).closest('.add-event-to-calendar-button-container').find('a.yahoo')[0].click();
        break
      case 'iCal':
        var href = EventSeriesAddToCalendar.getIcsHref('icon-ical', 'iCal', webinarTitle, startTime, endTime, playUrl);
        window.location.href = href;
        break
      case 'Outlook calendar':
        var href = EventSeriesAddToCalendar.getIcsHref('icon-outlook', 'Outlook', webinarTitle, startTime, endTime, playUrl);
        window.location.href = href;
    }
    $(e.target).parent().next('.dropdown-container').toggle();
  },

  getIcsHref: function(eClass, calendarName, title, startTime, endTime, playUrl) {
    return encodeURI(
      'data:text/calendar;charset=utf8,' + [
        'BEGIN:VCALENDAR',
        'VERSION:2.0',
        'BEGIN:VEVENT',
        'URL:' + playUrl,
        'DTSTART:' + EventSeriesAddToCalendar.formatDateForHref(startTime),
        'DTEND:' + EventSeriesAddToCalendar.formatDateForHref(endTime),
        'DESCRIPTION:' + playUrl,
        'SUMMARY:' + title,
        'END:VEVENT',
        'END:VCALENDAR'
      ].join('\n'));
  },

  formatDateForHref: function(date) {
    return date.toISOString().replace(/-|:|\.\d+/g, '');
  }
}
