const { sendEmailSubmitConversion } = require('./conversion-events');

$(document).ready(function() {
  HeroPartial.initialize();
});

var HeroPartial = {

  initialize: function() {
    Common.clickOrEnter({
      click: '.hero-partial #email-submit-button',
      enter: '.hero-partial #email-input',
      action: HeroPartial.submitForm
    });
    $('.hero-partial #email-input').on('input', HeroPartial.clearError);
    $('.hero-partial .marketing-email-opt-in-container').each(function(index) {
      Common.verifyIfMarketingEmailOptInNeeded(this);
    });
  },

  submitForm: function(e) {
    var $formContainer = $(e.currentTarget).parent();
    var $emailInput = $formContainer.find('#email-input');
    var $optedIntoMarketingEmailsInput = $formContainer.find('.marketing-email-opt-in-input')
    var email = $.trim($emailInput.val());

    if (Common.validateEmail(email)) {
      if (!Common.localStorageDisabled) {
        localStorage.setItem('formEmail', email);
        // expiration will be set for five minutes
        localStorage.setItem('formEmailExpiration', new Date().getTime() + 300000);
      }
      var redirectUrl = e.target.parentElement.children[0].innerHTML || '/create-account';
      var iterableEventName = e.target.parentElement.children[1].innerHTML || 'Marketing - Submitted Create Account Email - Hero Block';
      var iterableCampaign = e.target.parentElement.children[2].innerHTML;

      var iterableObj = {
        eventName: iterableEventName,
        email: email,
        instanceId: 1,
        isEmailSubmittedEvent: (iterableEventName !== 'Marketing - Email Submitted'),
        data: {
          eu: Common.checkIfNeedsMarketingEmailOptIn()
        }
      };
      if (iterableCampaign) {
        iterableObj.overrideTracking = {
          utm: {
            campaign: iterableCampaign
          }
        }
      }

      Iterable.sendEvent(iterableObj);

      var optedIntoMarketingEmails = $optedIntoMarketingEmailsInput[0] && $optedIntoMarketingEmailsInput[0].checked;

      if (Common.checkIfOptedIntoEmails() || optedIntoMarketingEmails) {
        Common.updateMarketingEmailOptInChoice(true, email);
      }

      Tracking.irisEmit('create_account_email_hero_block_submitted', {
        description: $('.hero-partial').has($formContainer).find('#segment-event').attr('value'),
        email: email,
        iterable: {  // TODO: Remove this and handle within iterable-client
          event_name: iterableEventName,
          instance_id: 1,
          email_submitted_event: true,
        },
        local_storage_disabled: Common.localStorageDisabled,
        target: ['iterable', 'segment']
      });

      sendEmailSubmitConversion().then(() => {
        var preservedParamsObject = Common.preservedParamsObject();
        if (Object.keys(preservedParamsObject).length > 0) {
          window.location.href = redirectUrl + '?' + Tools.getStringFromObject(preservedParamsObject, '&');
        } else {
          window.location.href = redirectUrl
        };
      });
    } else {
      $emailInput.addClass('error');
    }
  },

  clearError: function(e) {
    $(e.currentTarget).removeClass('error');
  },
};
