$(document).ready(function() {
  LogIn.initialize();
});

var LogIn = {

  initialize: function() {
    Common.clickOrEnter({
      click: "#login-final",
      enter: ".login-form input",
      action: LogIn.clickLogInButton
    });
    $('#login-email-field').on('keydown', LogIn.clearEmailError);
    $('#login-password-field').on('keydown', LogIn.clearPasswordError);
    if (Common.params["reset"]) {
      $('#notification-reset').addClass("show");
    }
  },

  clearEmailError: function() {
    $("#error-invalid-email").removeClass("show");
    $("#error-server").removeClass("show");
  },

  clearPasswordError: function() {
    $("#error-password").removeClass("show");
    $("#error-server").removeClass("show");
  },

  clickLogInButton: function(e) {
    e.preventDefault();
    var $selectCollection = $("select");
    if ($selectCollection.length > 0) {
      var url = $selectCollection[0].value;
      $("#login-final").addClass("disabled");
      $("#right-box-spinner").removeClass("hidden");
      Tracking.irisEmit('logged_in', { target: 'segment', has_my_teachable_account: LogIn.school_selected_has_teachable_account, shown_dropdown: true});
      window.location.href = url + Common.getParamsToPassToProduct() + (Common.params["plan_id"] ? '&to=admin.settings.plan&from_external=true' : '');
    } else {
      var email = $("#login-email-field").val().toLowerCase();
      var pw = $("#login-password-field").val();
      var emailOK = Common.validateEmail(email);
      var passwordOK = pw.length >= 6;

      if (!emailOK) {
        $("#error-invalid-email").addClass("show");
      }
      if (!passwordOK) {
        $("#error-password").addClass("show");
      }
      if (emailOK && passwordOK) {
        $("#login-final").addClass("disabled");
        $("#right-box-spinner").removeClass("hidden");
        localStorage.setItem("email", email);
        if ($("#recaptcha-enabled").length > 0){
          LogIn.sendLogInDataWithRecaptcha(email, pw);
        } else {
          LogIn.sendLogInData(email, pw);
        }
      }
    }
  },

  sendLogInDataWithRecaptcha: function(email, pw) {
    grecaptcha.ready(function() {
      // do request for recaptcha token
      // response is promise with passed token
      grecaptcha.execute(Common.getRecaptchaSiteKey(), {action: 'login'}).then(function(token) {
        // add token to form
        $('#login-fields').prepend('<input type="hidden" name="g-recaptcha-response" value="' + token + '">');
        LogIn.sendLogInData(email,pw)
      });
    });
  },

  sendLogInData: function(email, pw) {
    $.ajax({
      type: "POST",
      url: "/api/login/validate",
      data: {
        email: email,
        password: pw,
        referral: localStorage.getItem("referral"),
        recaptcha: $("input[name='g-recaptcha-response']").attr("value")
      },
      error: function(response) {
        $("#login-final").removeClass("disabled");
        $("#right-box-spinner").addClass("hidden");
      },
      success: function(response) {
        if (response.teachable_account) {
          if (Common.params["plan_id"]) { // the user is upgrading
            if (response.sign_ins.length == 1 && response.sign_ins[0].authenticated_url == null) { // the user has no schools associated with their myteachable account
              Tracking.irisEmit('logged_in', { email: email, target: 'segment', has_my_teachable_account: true, shown_dropdown: false});
              window.location.href = response.teachable_account.authenticated_url + Common.getParamsToPassToProduct();
            } else if (response.sign_ins.length == 1) { // the user has only one school associated with their myteachable account
              Tracking.irisEmit('logged_in', { email: email, target: 'segment', has_my_teachable_account: true, shown_dropdown: false});
              window.location.href = response.sign_ins[0].authenticated_url + Common.getParamsToPassToProduct() + '&to=admin.settings.plan&from_external=true';
            } else { // the user has multiple schools associated with their myteachable account
              $("#right-box-spinner").addClass("hidden");
              LogIn.school_selected_has_teachable_account = true;
              LogIn.populateSchoolList(response.sign_ins);
            }
          } else { // the user is not upgrading
            Tracking.irisEmit('logged_in', { email: email, target: 'segment', has_my_teachable_account: true, shown_dropdown: false});
            if (response.sign_ins.length == 1 && response.sign_ins[0].authenticated_url) { // the user has only one school associated with their myteachable account, so take them directly there
              window.location.href = response.sign_ins[0].authenticated_url + Common.getParamsToPassToProduct();
            } else { // the user has either no schools or multiple schools associated with their myTeachable account, so take them to myTeachable
              window.location.href = response.teachable_account.authenticated_url + Common.getParamsToPassToProduct();
            }
          }
        } else {
          if (response.sign_ins.length == 0) {
            $("#error-server").addClass("show");
            $("#login-final").removeClass("disabled");
            $("#right-box-spinner").addClass("hidden");
          } else if (response.sign_ins.length == 1) {
            Tracking.irisEmit('logged_in', { email: email, target: 'segment', has_my_teachable_account: false, shown_dropdown: false});
            window.location.href = response.sign_ins[0].authenticated_url + Common.getParamsToPassToProduct() + (Common.params["plan_id"] ? '&to=admin.settings.plan&from_external=true' : '');
          } else {
            $("#right-box-spinner").addClass("hidden");
            LogIn.school_selected_has_teachable_account = false;
            LogIn.populateSchoolList(response.sign_ins);
          }
        }
      }
    });
  },

  populateSchoolList: function(schools) {
    var school_list = "";
    schools.forEach(function(element, index, array) {
      school_list = school_list + '<option value="' + element.authenticated_url + '">' + element.school_name + '</option>'
    })
    $("#login-password-field").after(function() {
      options = '<select class="list" id="modal_login_school_list">' + school_list + '</select>';
      return options
    });
    $button = $("#login-final");
    $button.html("Select School")
    $button.removeClass("disabled");
  },

  school_selected_has_teachable_account: null,
}
