$(document).ready(function () {
  if (window.location.pathname.replace(/\/\//g, '/').split('/')[1] === 'blog') {
    BlogSignupModal.initialize();
  }
});

var BlogSignupModal = {

  initialize: function () {
    $('.blog-signup-modal-button').on('click', function () {
      $('.modal-background').show();
      $('.blog-signup-modal-container').css('display', 'flex');
    });

    Common.clickOrEnter({
      click: '.blog-signup-modal .cta-button',
      enter: '.blog-signup-modal .email-input',
      action: BlogSignupModal.handleEmailSubmit,
    });

    $('.blog-signup-modal .email-input').on('keydown', BlogSignupModal.clearError);
  },

  clearError: function () {
    $('.blog-signup-modal .email-input-container').removeClass('error');
  },

  handleEmailSubmit: function () {
    var email = $('.email-input').val();
    if (!Common.validateEmail(email)) {
      $('.blog-signup-modal .email-input-container').addClass('error');
    } else {
      localStorage.setItem("email", email);
      var eventName = 'Marketing - Signed Up for Blog Newsletter';
      var contentInfo = {
        title: 'Welcome to the Teachable online community!',
        body_text: [
          '<h4>We’re so happy you’re here.</h4>',
          '<p>At Teachable, our goal is to share the resources and advice you need to create, launch, and grow your online business.</p>',
          '<p>As a thanks for joining the community, here’s a free eBook for you to check out: How to Teach Online: Lessons from 100,000+ Schools.</p>',
          '<p>This eBook will give you targeted insights into how to create an online course and build a successful business. We promise to give you even more valuable resources, and can’t wait to see what you have to add to the community!</p>',

          '<p>Cheers,</p>',
          '<p>Team Teachable</p>',

          '<p>PS: Not sure where to get started? Join our webinar on <a href="https://workshops.teachable.com/7-steps-to-launch/register" target="_blank">7 Steps to Create Your Own Profitable Online Course!</a></p>',
        ].join('\n')
      };
      Tracking.irisEmit('marketing_signed_up_for_blog_newsletter', {
        email: email,
        target: ['segment', 'iterable', ],
        content_info: contentInfo,
        iterable: {  // TODO: Remove this and handle within iterable-client
          instance_id: 1,
          email_submitted_event: true
        },
      });
      BlogSignupModal.sendDataToIterable(email, eventName, contentInfo);
    }
  },

  sendDataToIterable: function (email, eventName, contentInfo) {
    Iterable.sendEvent({
      eventName: eventName,
      email: email,
      instanceId: 1,
      data: {},
      isEmailSubmittedEvent: true
    }).then(function() {
      BlogSignupModal.showThankYouMessage();
    });
  },

  showThankYouMessage: function () {
    var header = 'You’ve been added to our list!';
    var subheader = 'Check your inbox to confirm.';
    var $modal = $('.blog-signup-modal');

    $modal.find('h1.header')[0].innerText = header;
    $modal.find('p.subheader')[0].innerText = subheader;
    $modal.find('.email-input-container').addClass('hidden');
  }
}
