const { sendEmailSubmitConversion } = require('../conversion-events');
$(document).ready(function() {
  TeachableLiveSignup.initialize();
});

var TeachableLiveSignup = {

  currentModalStep: "1",

  email: "",

  initialize: function() {
    if ($('.teachable-live-hero-partial')[0]) {
      TeachableLiveSignup.populateEmailFieldIfCookied();
      Common.clickOrEnter({
        click: '.teachable-live-signup-modal .step-1 .email-input-form input.orange-button',
        enter: '.teachable-live-signup-modal .inputs-container input',
        action: TeachableLiveSignup.submitRegistration
      });
      $('.teachable-live-signup-modal .email-input-form input:not([type="submit"])').on('keydown', TeachableLiveSignup.clearError);
      Common.clickOrEnter({
        click: '.fb-messenger-link',
        action: TeachableLiveSignup.completeStep2
      });
      $('.mobile-survey input[type="radio"], .mobile-survey label').on('click', TeachableLiveSignup.activateSurveyButton);
      $('#teachable-live-survey-mobile .next-question-button').on('click', TeachableLiveSignup.goToNextQuestion);
      $('#teachable-live-survey-mobile .submit-button').on('click', TeachableLiveSignup.submitSurveyResponse.bind(this, { usingMobile: true }));
      $('#teachable-live-survey').submit(TeachableLiveSignup.submitSurveyResponse.bind(this, { usingMobile: false }));
      $('#teachable-live-survey-mobile label').on('click', TeachableLiveSignup.clickMobileOptionLabel);
      $('.teachable-live-signup-modal button.close-modal-button').on('click', TeachableLiveSignup.onClose);
    }
  },

  onClose: function(e) {
    var modalStep = function() {
      for (var i = 1; i < 4; i++) {
        var stepEl = $(e.target.parentElement).find('.step-' + i)
        if (!stepEl.hasClass('hidden'))
          return i
      }
    }();

    if (modalStep !== 1)
      window.location.href = '/live/2019/thankyou';
  },

  populateEmailFieldIfCookied: function() {
    var email = localStorage.getItem('email');
    if (email) {
      document.getElementById('form-email').value = email;
    }
  },

  clearError: function(e) {
    e.target.classList.remove('input-error');
    var children = e.target.parentElement.children;
    e.target.parentElement.nextElementSibling.classList.remove("show");
  },

  submitRegistration: function(e) {
    e.preventDefault();
    var form = $('.teachable-live-signup-modal .step-1 .email-input-form');
    var nameField = $(form).find("#form-name")[0];
    var emailField = $(form).find("#form-email")[0];
    var nameVisible = !!nameField;
    if (nameVisible) {
      var name = nameField.value.trim();
    }
    var email = emailField.value.trim();
    var emailOK = Common.validateEmail(email);
    var nameOK = !!(!nameVisible || name);
    if (!nameOK) {
      $(form).find('#form-name').addClass('input-error');
      $(form).find('#error-name').addClass('show');
    }
    if (!emailOK) {
      $(form).find('#form-email').addClass('input-error');
      $(form).find('#error-email').addClass('show');
    }
    if (nameOK && emailOK) {
      localStorage.setItem("email", email);
      TeachableLiveSignup.email = email;
      Tracking.irisEmit('email_submitted', {
        email: email
      });
      gtag_report_conversion()
      sendEmailSubmitConversion();
      $('.step-1').addClass('hidden');
      $('.step-2').removeClass('hidden');
      Tracking.irisEmit('modal_loaded', {type: 'FB Messenger'});
      Iterable.sendEvent({
        eventName: 'Marketing - Email Submitted',
        email: email,
        instanceId: 1,
        isEmailSubmittedEvent: true,
        data: {
          id: 201909,
          start_date: '2019-09-24 00:00:00 -0400',
          end_date: '2019-09-26 00:00:00 -0400',
          title: 'Teachable Live',
          campaign_group: 'summit'
        }
      });
    }
  },

  completeStep2: function(e) {
    if (e.target.dataset.skipFbMessenger === 'false') {
      Tracking.irisEmit('button_clicked');
    }
    $('.step-2').addClass('hidden');
    $('.step-3').removeClass('hidden');
    Tracking.irisEmit('modal_loaded', {type: 'Survey'});
  },

  goToNextQuestion: function(e) {
    var currentElement = $(".mobile-survey ."+e.target.dataset.currentTarget)[0];
    var nextElement = $(".mobile-survey ."+e.target.dataset.nextTarget)[0];
    currentElement.classList.add("hidden");
    nextElement.classList.remove("hidden");
    var survey = TeachableLiveSignup.getSurvey({ usingMobile: true });
    Tracking.irisEmit('modal_loaded', {
      survey: survey,
      type: 'Survey'
    });
  },

  activateSurveyButton: function(e) {
    var button = $(e.target).parents('.survey-question').find('.orange-button');
    button.removeClass('disabled');
    button.removeAttr('disabled');
  },

  submitSurveyResponse: function(args, e) {
    e.preventDefault();
    var survey = TeachableLiveSignup.getSurvey(args);
    Tracking.irisEmit('survey_submitted', {
      survey: survey
    });
    window.location.href = document.getElementById('post-modal-url').dataset.url;
  },

  getSurvey: function(args) {
    var surveyForm;
    if (args.usingMobile) {
      surveyForm = document.getElementById('teachable-live-survey-mobile');
    } else {
      surveyForm = document.getElementById('teachable-live-survey');
    }
    var values = $(surveyForm).serialize().split("&").map(function(item) {
      return item.split("=")
    }).reduce(function(hash, item) {
      hash[item[0]] = item[1]
      return hash
    }, {});
    var survey = Object.assign(
      values,
      {
        email: localStorage.getItem('email') || decodeURIComponent(Common.params.email),
        path: location.pathname
      }
    );
    return survey;
  },

  clickMobileOptionLabel: function(e) {
    var row = e.target.parentElement.parentElement;
    var radioButton = row.children[0].children[0];
    radioButton.checked = true;
  }
};
