const moment = require('moment');
const { sendEmailSubmitConversion } = require('../conversion-events');

$(document).ready(function() {
  if (window.location.pathname.indexOf('/register') >= 0) {
    WebinarRegistrationModal.initialize();
  }
});

var WebinarRegistrationModal = {

  initialize: function() {
    Common.clickOrEnter({
      click: '.webinar-registration-modal:not(.automated-webinar-modal) .register-button',
      enter: '.webinar-registration-modal:not(.automated-webinar-modal) input',
      action: WebinarRegistrationModal.clickRegisterButton
    });

    Common.clickOrEnter({
      click: '.webinar-registration-modal.automated-webinar-modal .register-button',
      enter: '.webinar-registration-modal.automated-webinar-modal input',
      action: WebinarRegistrationModal.clickAutomatedRegisterButton
    });

    $('.automated-webinar-countdown-timer:not(:first)').fadeOut('fast');

    $('.automated-webinar-time-select').change(WebinarRegistrationModal.selectAutomatedTime);
    $('.automated-webinar-time-select').niceSelect()
    $('.webinar-registration-modal input').on('keydown', WebinarRegistrationModal.clearError);
  },

  selectAutomatedTime: function() {
    var selectedOption = $(this).children("option:selected");
    var order = selectedOption.data("order");

    $('.automated-webinar-countdown-timer:not(#webinar-countdown-' + order + ')').fadeOut("slow");
    $('.automated-webinar-countdown-timer:not(#webinar-countdown-' + order + ')').addClass('hidden');
    $('.automated-webinar-countdown-timer#webinar-countdown-' + order).removeClass('hidden');
    $('.automated-webinar-countdown-timer#webinar-countdown-' + order).fadeIn("slow");
  },

  clickAutomatedRegisterButton: function() {
    var $emailInput = $('.webinar-registration-modal input[name="email"]')
    var $nameInput  = $('.webinar-registration-modal input[name="full_name"]')
    var email       = $emailInput.val();
    var name        = $nameInput.val();
    var emailOK     = Common.validateEmail(email)
    var nameOK      = name.length > 0
    var errorsPresent = false

    if (!emailOK) {
      $emailInput.addClass('error')
      $('.webinar-registration-modal .automated-webinar-input-error[name="email"]').removeClass("hidden")
      errorsPresent = true
    }

    if (!nameOK) {
      $nameInput.addClass('error')
      $('.webinar-registration-modal .automated-webinar-input-error[name="name"]').removeClass("hidden")
      errorsPresent = true
    }

    if (!errorsPresent) {
      localStorage.setItem('email', email);
      WebinarRegistrationModal.updateFormStatus('submitting');
      WebinarRegistrationModal.sendAutomatedWebinarDataToServer(email);

      sendEmailSubmitConversion();

      Tracking.irisEmit('automated_webinar_registration_modal_registered', {
        email: email,
        target: 'segment'
      });
    }
  },


  clickRegisterButton: function() {
    var $input = $('.webinar-registration-modal input');
    var email = $input.val();
    var emailOK = Common.validateEmail(email);

    if (!emailOK) {
      $input.addClass('error');
    } else {
      localStorage.setItem('email', email);
      WebinarRegistrationModal.updateFormStatus('submitting');
      WebinarRegistrationModal.sendDataToServer(email);


      sendEmailSubmitConversion();
      Tracking.irisEmit('webinar_registration_modal_registered', {
        email: email,
        target: 'segment'
      });
    }
  },

  clearError: function() {
    $('.webinar-registration-modal input').removeClass('error');
    $('.automated-webinar-input-error').addClass('hidden')
    $('.duplicate').hide();
  },

  updateFormStatus: function(status, otherError) {
    var $button = $('.webinar-registration-modal .register-button');
    var $input = $('.webinar-registration-modal input');
    var $spinner = $('.spinner');
    var $thankYouDiv = $('div.thank-you');
    var $duplicate = $('div.duplicate');
    if (status == 'submitting') {
      $button.addClass('btn disabled');
      $input.prop('disabled', true);
      $spinner.css('display', 'block');
    } else if (status == 'duplicate') {
      $spinner.hide();
      $input.addClass('error');
      $input.prop('disabled', false);
      $button.removeClass('btn disabled');
      $duplicate.show();
    } else if (status == 'otherError') {
      $spinner.hide();
      $input.addClass('error');
      $input.prop('disabled', false);
      $button.removeClass('btn disabled');
      $duplicate.show();
      $duplicate.html(otherError);
     }
  },

  sendAutomatedWebinarDataToServer: function(email) {
    var selectedOption = $('.automated-webinar-time-select').children('option:selected').val();
    $.ajax({
      url: '/api/registrants',
      method: 'POST',
      data: {
        registrant: {
          email: email
        },
        webinar_id:        $('.webinar-id').html(),
        session_token:     selectedOption,
        campaignid:        Common.params['i_campaign_id'],
        templateid:        Common.params['i_template_id'],
        utm_campaign:      Common.params['utm_campaign'],
        utm_source:        Common.params['utm_source'],
        utm_medium:        Common.params['utm_medium'],
        utm_term:          Common.params['utm_term'],
        utm_content:       Common.params['utm_content'],
        mt_campaign_group: Common.params['mt_campaign_group'],
        mt_tier:           Common.params['mt_tier'],
        mt_interval:       Common.params['mt_interval'],
        mbsy:              Common.params['mbsy']
      },
      success: function(response) {
        window.location.href = '/webinars/' + response.url + '/thank-you' + '?session_id=' + selectedOption;
      },
      error: function(response) {
        if (response.responseText == 'You have already registered for this webinar.') {
          WebinarRegistrationModal.updateFormStatus('duplicate');
        } else {
          WebinarRegistrationModal.updateFormStatus('otherError', response.responseText);
        }
      }
    });
  },


  sendDataToServer: function(email) {
    var time2 = $('.time2').html();
    $.ajax({
      url: '/api/registrants',
      method: 'POST',
      data: {
        registrant: {
          email: email
        },
        webinar_id: $('.webinar-id').html(),
        time2: time2,
        campaignid: Common.params['i_campaign_id'],
        templateid: Common.params['i_template_id'],
        utm_campaign: Common.params['utm_campaign'],
        utm_source: Common.params['utm_source'],
        utm_medium: Common.params['utm_medium'],
        utm_term: Common.params['utm_term'],
        utm_content: Common.params['utm_content'],
        mt_campaign_group: Common.params['mt_campaign_group'],
        mt_tier: Common.params['mt_tier'],
        mt_interval: Common.params['mt_interval'],
        mbsy: Common.params['mbsy']
      },
      success: function(response) {
        var timeString = $('.date-for-countdown').html();
        var time = new Date(timeString * 1000);
        var difference = time.getTime() - Date.now();
        var duration = moment.duration(difference);
        if (duration < (-90 * 60 * 1000)) {
          window.location.href = '/webinars/' + response.url + '/replay';
        } else if (duration < (10 * 60 * 1000)) {
          window.location.href = '/webinars/' + response.url + '/play';
        } else {
          window.location.href = '/webinars/' + response.url + '/thank-you' + (time2 === 'true' ? '?time=2' : '');
        }
      },
      error: function(response) {
        if (response.responseText == 'You have already registered for this webinar.') {
          WebinarRegistrationModal.updateFormStatus('duplicate');
        } else {
          WebinarRegistrationModal.updateFormStatus('otherError', response.responseText);
        }
      }
    });
  }
}

module.exports = WebinarRegistrationModal;
