const { sendAccountCreatedConversion } = require('./conversion-events');

$(document).ready(function() {
  if ($('.sign-up-form')[0]) {
    SignUp.initialize();
  }
});

var SignUp = {

  initialize: function() {
    Common.clickOrEnter({
      click: "#signup_final",
      enter: ".signup-form-step-2 input",
      action: SignUp.clickSignUpButton
    });
    $('#modal_name_field').on('keydown', SignUp.clearNameError);
    $('#modal_email_field').on('keydown', SignUp.clearEmailErrors);
    $('#modal_password_field').on('keydown', SignUp.clearPasswordErrors);
    $('#modal_confirm_password_field').on('keydown', SignUp.clearPasswordErrors);
    $('.terms-of-service-checkbox').on("change", SignUp.clearServiceTermsError);
    $('#modal_email_field').on('blur', SignUp.checkEmail);
    $('.sign-up-form .marketing-email-opt-in-container').each(function(index) {
      Common.verifyIfMarketingEmailOptInNeeded(this);
    });
  },

  checkEmail: function(){
    var originalEmail = $('#modal_email_field')[0].value;
    var emailValidation = Common.checkEmailValidation($('#modal_email_field')[0]);
    var existingValidationErrorEl = document.querySelector('.email-validation-update-email-prompt');
    if (emailValidation && !existingValidationErrorEl) {
      var suggestionEl = document.createElement('span');
      var suggestionButtonEl = document.createElement('a');
      suggestionEl.classList.add('email-validation-update-email-container');
      suggestionEl.dataset.originalEmail = originalEmail;
      suggestionEl.dataset.suggestedEmail = emailValidation.suggestedEmail;
      suggestionButtonEl.classList.add('email-validation-update-email-prompt');
      suggestionButtonEl.innerText = emailValidation.error;
      suggestionButtonEl.onclick = function(){
        $('#modal_email_field')[0].value = emailValidation.suggestedEmail;
        suggestionEl.remove();
        Tracking.irisEmit("suggested_valid_email_used", {
          original_email: originalEmail,
          corrected_email: emailValidation.suggestedEmail,
        });
      };
      suggestionEl.appendChild(suggestionButtonEl);
      $('#modal_email_field')[0].insertAdjacentElement('afterend', suggestionEl);
      Tracking.irisEmit("suggested_valid_email_shown", {
        original_email: originalEmail,
        corrected_email: emailValidation.suggestedEmail,
      });
    }
  },

  clearNameError: function() {
    $("#error_username").removeClass("show");
  },

  clearEmailErrors: function() {
    $("#error_email").removeClass("show");
    $("#error_duplicate_email").removeClass("show");
    if ($('.email-validation-update-email-container').length > 0) {
      var emailValidationEl = $(".email-validation-update-email-container");
      Tracking.irisEmit("suggested_valid_email_used", {
        original_email: emailValidationEl.data("originalEmail"),
        corrected_email: emailValidationEl.data("suggestedEmail"),
      });
      emailValidationEl.remove();
    }
  },

  clearPasswordErrors: function() {
    $("#error_pw").removeClass("show");
    $("#error_confirm").removeClass("show");
  },

  clearServiceTermsError: function() {
    $("#error_terms").removeClass("show");
  },

  clickSignUpButton: function(e) {
    e.preventDefault();
    var name = $("#modal_name_field").val();
    var email = $("#modal_email_field").val();
    var pw = $("#modal_password_field").val();
    var confirmPw = $("#modal_confirm_password_field").val();

    var userNameOK = name.length > 0;
    var emailOK = Common.validateEmail(email);
    var passwordOK = pw.length >= 6;
    var confirmPasswordOK = (pw === confirmPw);
    var termsOfServiceOK = $('.terms-of-service-checkbox').is(":checked");

    if (!userNameOK) {
      $("#error_username").addClass("show");
    }
    if (!emailOK) {
      $("#error_email").addClass("show");
    }
    if (!passwordOK) {
      $("#error_pw").addClass("show");
    } else {
      if (!confirmPasswordOK) {
        $("#error_confirm").addClass("show");
      }
    }
    if (!termsOfServiceOK) {
      $("#error_terms").addClass("show");
    }

    if (userNameOK && emailOK && passwordOK && confirmPasswordOK && termsOfServiceOK) {
      $("#signup_final").attr('disabled', true);
      $("#right-box-spinner").removeClass("hidden");
      if(!Common.localStorageDisabled) {
        localStorage.setItem("email", email);
      }
      SignUp.sendSignUpData(name, email, pw);
    }
  },

  sendSignUpData: function(name, email, password) {
    var referral = null;
    if (!Common.localStorageDisabled) {
      referral = localStorage.getItem("referral");
    }
    postData = {
      name: name,
      email: email,
      password: password,
      referral: referral
    }
    if (!!Tracking.anonymousId() && Tracking.isProduction) {
      postData._afid = Tracking.anonymousId();
    };
    $.ajax({
      type: "POST",
      url: "/api/register_account",
      data: { registration: postData },
      success: function(response) {
        sendAccountCreatedConversion();
        Tracking.irisEmit('sign_up_email_submitted', {
          email: email,
          target: ['segment', 'iterable', ],
          iterable: {  // TODO: Remove this and handle within iterable-client
            event_name: 'Sign Up - Email Submitted',
            instance_id: 1,
            email_submitted_event: true,
          },
        });
        Iterable.sendEvent({
          eventName: 'Sign Up - Email Submitted',
          email: email,
          instanceId: 1,
          isEmailSubmittedEvent: true,
          data: {
            eu: Common.checkIfNeedsMarketingEmailOptIn()
          }
        });

        var $optedIntoMarketingEmailsInput = $.find('.marketing-email-opt-in-input');

        var optedIntoMarketingEmails = $optedIntoMarketingEmailsInput[0] && $optedIntoMarketingEmailsInput[0].checked;

        if (Common.checkIfOptedIntoEmails() || optedIntoMarketingEmails) {
          Common.updateMarketingEmailOptInChoice(true, email);
        }
        Tracking.waitForTrackingToSend(function() {
          var redirectUrl = response.redirect_url + Common.getParamsToPassToProduct();
          if (!!Tracking.anonymousId() && Tracking.isProduction) {
            redirectUrl += Tracking.afidUrlParameter();
          };
          window.location.href = redirectUrl;
        });
      },
      error: function(response) {
        $("#right-box-spinner").addClass("hidden");
        $("#signup_final").removeAttr('disabled');
        var response = JSON.parse(response.responseText);
        if (response["teachable_account"]["email"][0] == "has already been taken") {
          Tracking.irisEmit(
            'existing_account_error',
            SignUp.getIrisTrackingData()
          );
          $("#error_duplicate_email").addClass("show");
        } else {
          $("#error_server").addClass("show");
        }
      }
    });
  },

  getIrisTrackingData: function() {
    var attributes = {};
    attributes.email = $('#modal_email_field').val();

    if (!Common.localStorageDisabled && localStorage.getItem("referral")) {
      attributes.referral_token = localStorage.getItem("referral");
    };

    var marketingParams = [
      'upgrade_src',
      'utm_source',
      'utm_medium',
      'utm_campaign',
      'utm_content',
      'mt_campaign_group',
      'mt_tier',
      'mt_interval',
      'i_campaign_id',
      'i_template_id'
    ];

    marketingParams.forEach(function(param) {
      var paramValue = Common.params[param];
      if (paramValue) {
        attributes[param] = paramValue;
      }
    });

    return attributes;
  },
}
