const moment = require('moment');
const kebabCase = require('lodash/kebabCase');

const { sendAccountCreatedConversion, sendPurchasedSchoolPlanConversion } = require('./conversion-events');

$(document).ready(function() {
  if ($('.single-page-checkout-v2')[0]) {
    SinglePageCheckoutV2.initialize();
  }
});

var SinglePageCheckoutV2 = {

  pageViewSentToIterable: false,

  initialize: function() {
    var $country = $('#country-code');
    SinglePageCheckoutV2.setUpStripe();
    SinglePageCheckoutV2.setStateVariables();
    $(".single-page-checkout-v2 #purchase-plan").on('click', SinglePageCheckoutV2.clickPurchasePlan);
    $('.single-page-checkout-v2 #add-coupon-box').on('click', SinglePageCheckoutV2.showCouponBox);
    Common.clickOrEnter({
      click: '.single-page-checkout-v2 .dark-green-button',
      enter: "#coupon-code",
      action: SinglePageCheckoutV2.clickApplyCoupon
    });
    $country.on('change', SinglePageCheckoutV2.changeCountryField);
    $('#email').on('blur', SinglePageCheckoutV2.checkEmail)
    $('#email').on('keydown', SinglePageCheckoutV2.clearEmailSuggestionNotice)
    $('#region-code').on('change', SinglePageCheckoutV2.changeRegionCode)
    SinglePageCheckoutV2.setUpEventHandlers(['account-name', 'email', 'school-name'], 'input', 'clearFieldError');
    SinglePageCheckoutV2.setUpEventHandlers(['email'], 'focusout', 'checkForExistingEmail');
    SinglePageCheckoutV2.setUpEventHandlers(['password', 'confirm-password'], 'input', 'clearPasswordErrors');
    SinglePageCheckoutV2.setUpEventHandlers(['password', 'email'], 'focusout', 'checkForExistingAccount');
    SinglePageCheckoutV2.setUpEventHandlers(['coupon-code'], 'input', 'changeCouponField');
    SinglePageCheckoutV2.setUpEventHandlers(['verify-email-with-password'], 'click', 'checkForExistingAccount')
    SinglePageCheckoutV2.setUpEventHandlers(['existing-account-password'], 'input', 'clearExistingAccountPasswordErrors')
    SinglePageCheckoutV2.setUpEventHandlers(['choose-account-information-select'], 'change', 'updatePreselectedAccount')
    SinglePageCheckoutV2.setUpEventHandlers(['region-code', 'country-code'], 'change', 'clearFieldError');
    SinglePageCheckoutV2.setUpEventHandlers(['reset-password-email-link'], 'click', 'triggerResetPasswordEmail')
    SinglePageCheckoutV2.setUpEventHandlers(['new-payment-method-existing-account'], 'click', 'displayNewPaymentInfo')
    SinglePageCheckoutV2.togglePostalCodeAndStateRow($country.val());
    SinglePageCheckoutV2.showCouponBoxOnLoad();

    $('#email').on('keyup', function(e) {
      if (e.keyCode === 13) {
        SinglePageCheckoutV2.checkForExistingEmail()
      }
    })

    $('#existing-account-password').on('keyup', function(e) {
      if (e.keyCode === 13) {
        SinglePageCheckoutV2.checkForExistingAccount()
      }
    })
    if ($('#email').val().length > 0) {
      SinglePageCheckoutV2.checkForExistingEmail();
    }
  },

  // SETUP -----

  setUpEventHandlers: function(arrayOfIds, events, clearFunction) {
    arrayOfIds.forEach(function(id) {
      var errorId = SinglePageCheckoutV2.createFieldErrorId(id);
      $('#' + id).on(events, function() {
        SinglePageCheckoutV2[clearFunction](errorId);
        $(errorId + '-api').removeClass('show');
        $('#' + id).removeClass('input-error');
      });
    });
  },

  changeRegionCode: function(event) {
    var stateInput = event.target.value
    var postalCode = SinglePageCheckoutV2.postalCode
    if (stateInput && postalCode.length >= 5) {
      SinglePageCheckoutV2.updateSalesTax(postalCode, stateInput)
    }  else {
      SinglePageCheckoutV2.taxAmount = 0.0
      SinglePageCheckoutV2.setOrderSummaryValues()
    }
  },

  changePostalCode: function(event) {
    SinglePageCheckoutV2.postalCode = event.value
    var stateInput = $('#region-code')[0]
    if (stateInput.value && event.value.length >= 5) {
      SinglePageCheckoutV2.updateSalesTax(event.value, stateInput.value)
    } else {
      SinglePageCheckoutV2.taxAmount = 0.0
      SinglePageCheckoutV2.setOrderSummaryValues()
    }
  },

  setStateVariables: function() {
    SinglePageCheckoutV2.existingAccoutNewPaymentMethod = false;
    SinglePageCheckoutV2.postCheckoutUrl        = $('meta[name="post-checkout-redirect"]')[0].content;
    SinglePageCheckoutV2.cardNumberComplete     = false;
    SinglePageCheckoutV2.cardExpirationComplete = false;
    SinglePageCheckoutV2.cvcCodeComplete        = false;
    SinglePageCheckoutV2.postalCodeComplete     = false;
    SinglePageCheckoutV2.postalCode             = "";
    SinglePageCheckoutV2.schoolId               = null;
    SinglePageCheckoutV2.cardType               = null;
    SinglePageCheckoutV2.lastFour               = null;
    SinglePageCheckoutV2.teachableAccountID     = null;
    SinglePageCheckoutV2.accountName            = null;
    SinglePageCheckoutV2.originalPrice          = parseFloat($('.final-plan-price').html().replace('$', '').replace(',', ''))
    SinglePageCheckoutV2.taxAmount              = 0.0
    SinglePageCheckoutV2.freeCouponTaxAmount = 0.0
    SinglePageCheckoutV2.discountAmount         = 0.0
  },

  showCouponBoxOnLoad: function() {
    SinglePageCheckoutV2.paramInCouponBox();
    if ($('#coupon-code').val()) {
      this.enableCouponSubmit();
      SinglePageCheckoutV2.showCouponBox();
      this.clickApplyCoupon();
    } else {
      // Wait a tick for Iris to initialize, since this is called during initializaton
      setTimeout(function() {
        SinglePageCheckoutV2.sendPageViewToIterable();
      });
    }
  },

  paramInCouponBox: function() {
    if (Common.params.coupon_code) {
      $('#coupon-code').val(decodeURIComponent(Common.params.coupon_code));
    }
  },

  sendPageViewToIterable: function() {
    var email = localStorage.getItem('email');
    if (email) {
      var adjusted_price = $('.final-plan-price').html().replace('$', '');
      var normal_price = $('#iterable-normal-price').html();
      var data = {
        plan_id: Common.params.plan_id,
        plan_name: $('#iterable-plan-name').html(),
        normal_price: normal_price,
        coupon_code: Common.params.coupon_code,
        coupon_discount: (+normal_price - +adjusted_price),
        adjusted_price: adjusted_price,
        upgrade_src: Common.params.upgrade_src || ''
      };
      Iterable.sendEvent({
        eventName: 'Marketing - Visited SPC V2 Page',
        email: email,
        instanceId: 1,
        data: data
      });
      Tracking.irisEmit('marketing_visited_spc_v2_page', {
        eventName: 'Marketing - Visited SPC V2 Page',
        email: email,
        instanceId: 1,
        data: data,
        target: 'iterable'
      });
    }
    SinglePageCheckoutV2.pageViewSentToIterable = true;
  },

  // HANDLERS -----

  checkEmail: function(){
    var originalEmail = $('#email')[0].value;
    var emailValdiation = Common.checkEmailValidation($('#email')[0]);
    var existingValidationErrorEl = document.querySelector('.email-validation-update-email-prompt');
    if (emailValdiation && !existingValidationErrorEl) {
      var suggestionEl = document.createElement('span');
      var suggestionButtonEl = document.createElement('a');
      suggestionButtonEl.classList.add('email-validation-update-email-prompt');
      suggestionButtonEl.innerText = emailValdiation.error;
      suggestionButtonEl.onclick = function(){
        $('#email')[0].value = emailValdiation.suggestedEmail;
        suggestionEl.remove();
        SinglePageCheckoutV2.checkForExistingEmail();
        Tracking.irisEmit("suggested_valid_email_used", {
          original_email: originalEmail,
          corrected_email: emailValdiation.suggestedEmail,
        });
      };
      suggestionEl.appendChild(suggestionButtonEl)
      $('#email')[0].insertAdjacentElement('afterend', suggestionEl)
      Tracking.irisEmit("suggested_valid_email_shown", {
        original_email: originalEmail,
        corrected_email: emailValdiation.suggestedEmail,
      });
    }
  },

  changeCountryField: function(e) {
    var country = $(e.currentTarget).val();
    $('#error-region-code').removeClass('show');
    $('#error-country-code-verification').removeClass('show');
    SinglePageCheckoutV2.togglePostalCodeAndStateRow(country);
  },

  togglePostalCodeAndStateRow: function(country) {
    var $stateCol = $('#state-col');
    var $postalCodeCol = $('#postal-code-col');
    if (country === 'US') {
      $stateCol.removeClass('hidden');
      $postalCodeCol.addClass("col-md-6");
      var zipInput   = SinglePageCheckoutV2.postalCode
      var stateInput = $('#region-code')[0]
      if (zipInput.length >= 5 && stateInput.value) {
        SinglePageCheckoutV2.updateSalesTax(zipInput, stateInput.value)
      } else {
        SinglePageCheckoutV2.taxAmount = 0.0
        SinglePageCheckoutV2.setOrderSummaryValues()
      }


    } else {
      if (!($stateCol.hasClass('hidden'))) {
        $stateCol.addClass('hidden');
        $postalCodeCol.removeClass("col-md-6");
        SinglePageCheckoutV2.taxAmount = 0.0;
        SinglePageCheckoutV2.setOrderSummaryValues();
      }
    }
  },

  showCouponBox: function() {
    // if you are focused on a hidden button, can still click on it in safari.
    // if statement prevents needless changing of the DOM
    if(!$('#add-coupon-box').hasClass('hidden')) {
      $('.single-page-checkout-v2 #coupon-box').removeClass('hidden');
      $('.single-page-checkout-v2 #add-coupon-box').addClass('hidden');
    }
  },

  disablePurchaseButton: function() {
    var $button = $('#purchase-plan');
    $button.addClass('disabled');
  },

  disableCouponSubmit: function() {
    $('.dark-green-button').addClass('disabled');
  },

  enableCouponSubmit: function(enable) {
    if (enable || $('.button-wave').css('opacity') !== '1') {
      $('.dark-green-button').removeClass('disabled');
    }
  },

  enablePurchaseButton: function() {
    var $button = $('#purchase-plan');
    $button.removeClass('disabled');
  },

  clickPurchasePlan: function() {
    if (!$('.single-page-checkout-v2 #purchase-plan').hasClass('disabled')) {
      var fieldValues = SinglePageCheckoutV2.getFieldValuesObject();
      var valuesOK = SinglePageCheckoutV2.getFieldValuesOkObject(fieldValues);
      var allValuesOK = Object.keys(valuesOK).every(function(k) {
        return valuesOK[k];
      });

      $('.error.show').removeClass('show');
      SinglePageCheckoutV2.showFieldErrors(valuesOK);
      Tracking.irisEmit('purchase_plan_v2_clicked', {
        target: 'segment'
      });
      if (allValuesOK) {
        SinglePageCheckoutV2.disablePurchaseButton();
        if (SinglePageCheckoutV2.lastFour && SinglePageCheckoutV2.cardType && fieldValues.school_id && !SinglePageCheckoutV2.existingAccoutNewPaymentMethod) {
          fieldValues.school_plan_id = Common.params.plan_id;
          SinglePageCheckoutV2.createSchoolAndUpgrade(fieldValues);
        } else {
          SinglePageCheckoutV2.createStripeToken(fieldValues);
        }
      } else {
        SinglePageCheckoutV2.autoScrollToTopError();
      }
    }
  },

  clickApplyCoupon: function() {
    if (!$('.dark-green-button').hasClass('disabled')) {
      $('#error-five-oh-three').removeClass('show');
      $('#error-coupon-code').removeClass('show');
      SinglePageCheckoutV2.disableCouponSubmit();
      SinglePageCheckoutV2.showButtonWave();
      SinglePageCheckoutV2.submitCouponCode();
    }
  },

  showButtonWave: function() {
    $('#coupon-button-text').css('opacity', '0');
    $('.button-wave').css('opacity', '1');
  },

  removeButtonWave: function() {
    $('#coupon-button-text').css('opacity', '1');
    $('.button-wave').css('opacity', '0');
  },

  changeCouponField: function() {
    $('#error-coupon-code').removeClass('show');
    if (SinglePageCheckoutV2.getCouponCode()) {
      SinglePageCheckoutV2.enableCouponSubmit();
    } else {
      SinglePageCheckoutV2.disableCouponSubmit();
    }
  },

  updatePreselectedAccount: function(){
    var selectedOption = $('#choose-account-information-select').children("option:selected");
    if (selectedOption.val() === 'create_new_school') {
      $('.account-school-name-group').addClass('show');
      SinglePageCheckoutV2.displayNewPaymentInfo();
    } else {
      var schoolId =  selectedOption.val();
      $('.account-school-name-group').removeClass('show');
      $('#school-name').val(null);

      SinglePageCheckoutV2.schoolId = schoolId;
      SinglePageCheckoutV2.lastFour = selectedOption.data().lastFour;
      SinglePageCheckoutV2.cardType = selectedOption.data().cardType;

      if (SinglePageCheckoutV2.lastFour && SinglePageCheckoutV2.cardType) {
        $('#existing-payment-information-last-four').text(SinglePageCheckoutV2.lastFour);
        $('#existing-payment-information-card-type').text(SinglePageCheckoutV2.cardType);
        SinglePageCheckoutV2.displayExistingPaymentInfo();
      } else {
        SinglePageCheckoutV2.displayNewPaymentInfo();
      }
    }
  },

  displayNewPaymentInfo: function(){
    if (SinglePageCheckoutV2.schoolId) {
      SinglePageCheckoutV2.existingAccoutNewPaymentMethod = true;
    }
    $('.existing-payment-information').addClass("hidden");
    $('.new-payment-information').removeClass("hidden");
  },

  displayExistingPaymentInfo: function(){
    $('.existing-payment-information').removeClass("hidden");
    $('.new-payment-information').addClass("hidden");
  },

  checkForExistingEmail: function(){
    var email = $('#email').val();
    if (email.length < 1) {
      return
    }
    $.ajax({
      type: "GET",
      url: SinglePageCheckoutV2.getProperEndPoint() + "/check_email",
      data: {
        email: email
      },
      success: function(response) {
        if (response.existing_account) {
          $('.choose-account-information').removeClass('hidden');
          $('#validate-email').addClass('hidden');
          $('.account-password-row').removeClass('show');
          $('.account-school-name-group').removeClass('show');
          $('.account-name-group').removeClass('show');
        } else {
          $('#validate-email').addClass('hidden');
          SinglePageCheckoutV2.resetExistingAccountInformation();
          $('.account-password-row').addClass('show');
          $('.account-school-name-group').addClass('show');
          $('.account-name-group').addClass('show');
        }

        $('#existing-account-password').focus()
      }
    })
  },

  resetExistingAccountInformation: function(){
    $('.choose-account-information').addClass('hidden');
    SinglePageCheckoutV2.schoolId = null;
    SinglePageCheckoutV2.cardType = null;
    SinglePageCheckoutV2.lastFour = null;
    SinglePageCheckoutV2.teachableAccountID = null;
  },

  checkForExistingAccount: function(){
    var email    = $('#email').val();
    var password = $('#existing-account-password').val();

    if (email && password) {
      $('#verify-email-with-password').addClass('disabled')
      $('#verify-email-with-password .verify-button-text').css('opacity', 0)
      $('#verify-email-with-password .button-wave').css('opacity', 1)
      $.ajax({
        type: "POST",
        url: "/api/login/check_account",
        data: {
          email: email,
          password: password
        },
        success: function(response) {
          $('#verify-email-with-password').removeClass('disabled')
          $('#verify-email-with-password .verify-button-text').css('opacity', 1)
          $('#verify-email-with-password .button-wave').css('opacity', 0)

          var schoolsInfo = response.sign_ins;
          if (schoolsInfo.length > 0 && schoolsInfo[0]["school_id"]) {
            SinglePageCheckoutV2.teachableAccountID = response.teachable_account.id;
            SinglePageCheckoutV2.accountName = response.teachable_account.name || email;
            SinglePageCheckoutV2.promptToChooseAccount(schoolsInfo);
          } else if (response.teachable_account){
            SinglePageCheckoutV2.teachableAccountID = response.teachable_account.id
            $('.account-name-group').addClass('hidden');
            $('.account-school-name-group').removeClass('hide');
            $('.choose-account-information').addClass('hidden');
            $('.existing-account-password').addClass('hidden');
          } else {
            $('#existing-account-password').addClass('input-error');
            $('#error-existing-account-password').addClass('show');
          }
        }
      })
    }
  },

  promptToChooseAccount: function(signInData) {
    $("#choose-account-information-select option[disabled!='disabled']").remove();
    select = document.getElementById('choose-account-information-select')
    var i;

    for (i = 0; i < signInData.length; i++) {
      var option = document.createElement('option');
      option.text = signInData[i].school_name;
      option.value = signInData[i].school_id;

      var lastFour = document.createAttribute('data-last-four');
      lastFour.value = signInData[i].last_four;

      var cardType = document.createAttribute('data-card-type');
      cardType.value = signInData[i].card_type;

      var schoolPlanId = document.createAttribute('data-school-plan-id');
      schoolPlanId.value = signInData[i].school_plan_id;

      if (signInData[i].school_plan_id === parseInt(Common.params.plan_id)) {
        var disabled = document.createAttribute('disabled');
        disabled.value = 'disabled';
        option.setAttributeNode(disabled);
        option.text = option.text + ' - (Currently on this plan)'
      }
      option.setAttributeNode(lastFour);
      option.setAttributeNode(cardType);
      option.setAttributeNode(schoolPlanId);

      select.add(option);
    }
    var newOption = document.createElement('option');
    newOption.text = 'Create a new school';
    newOption.value = 'create_new_school';

    select.add(newOption)

    $('.account-name-group').removeClass('show');
    $('.account-school-name-group').removeClass('show');
    $('.existing-account-password').addClass('hidden');
    $('.choose-existing-school-information').removeClass('hidden');
  },

  triggerResetPasswordEmail: function() {
    var email = $('#email').val();
    $.ajax({
      type: "GET",
      url: SinglePageCheckoutV2.getProperEndPoint() + "/api/v1/account_retrieval",
      data: {
        email: email
      },
      success: function() {
        $("#success-reset-password-email-link").addClass("show")
      },
      error: function(XMLHttpRequest, textStatus, errorThrown) {
        $("#error-reset-password-email-link").addClass("show")
      }
    });
  },

  // HELPERS -----

  getFieldValuesObject: function() {
    return {
      school_name: $('#school-name').val(),
      account_name: $('#account-name').val() || SinglePageCheckoutV2.accountName,
      email: $('#email').val(),
      password: $('#password').val() || $('#existing-account-password').val(),
      country_code: $('#country-code').val() || SinglePageCheckoutV2.schoolId || '',
      school_id: SinglePageCheckoutV2.schoolId,
      teachable_account_id: SinglePageCheckoutV2.teachableAccountID,
    };
  },

  getFieldValuesOkObject: function(fieldValues) {
    var okObject = {};
    if (fieldValues.country_code === 'US') {
      okObject.regionCodeOK = $('#region-code').val() !== null;
    } else {
      okObject.regionCodeOK = true;
    }

    if (this.couponIsValid()) {
      okObject.couponCodeOK = true;
    }

    okObject.accountNameOK            = (fieldValues.account_name && fieldValues.account_name.length > 0) || !!fieldValues.teachable_account_id;
    okObject.emailOK                  = Common.validateEmail(fieldValues.email);
    okObject.passwordOK               = fieldValues.password.length >= 6;
    okObject.paymentInfoOK            = SinglePageCheckoutV2.getPaymentInfoStatus(fieldValues);
    okObject.confirmPasswordOK        = (fieldValues.password === $('#confirm-password').val()) || okObject.paymentInfoOK;
    okObject.schoolNameOK             = fieldValues.school_name.length > 0 || okObject.paymentInfoOK;
    okObject.countryCodeOK            = fieldValues.country_code !== null && fieldValues.country_code.length > 0;
    okObject.cardNumberOK             = SinglePageCheckoutV2.cardNumberComplete || okObject.paymentInfoOK;
    okObject.expirationOK             = SinglePageCheckoutV2.cardExpirationComplete || okObject.paymentInfoOK;
    okObject.cardCvcOK                = SinglePageCheckoutV2.cvcCodeComplete || okObject.paymentInfoOK;
    okObject.postalCodeOK             = SinglePageCheckoutV2.postalCodeComplete || okObject.paymentInfoOK;

    return okObject;
  },

  getPaymentInfoStatus: function(fieldValues){
    return (!!fieldValues.school_id && SinglePageCheckoutV2.lastFour && SinglePageCheckoutV2.cardType) ||
           (
             SinglePageCheckoutV2.cardNumberComplete &&
             SinglePageCheckoutV2.cardExpirationComplete &&
             SinglePageCheckoutV2.cvcCodeComplete &&
             SinglePageCheckoutV2.postalCodeComplete
           )
  },

  getProperEndPoint: function() {
    // this is for testing environment
    if ($('meta[id=environment]').attr('data-env') === 'test') {
      return '';
    } else {
      return Common.getProperMarketingApiEndpoint();
    }
  },

  createFieldErrorId: function(key) {
    return `#error-${kebabCase(key)}`;
  },

  countryVerified: function(stripeCountryCode, fieldCountryCode) {
    var codes = JSON.parse($('#country-codes').attr('value'));
    if (!stripeCountryCode || !codes.includes(stripeCountryCode)) {
      return true;
    } else if (stripeCountryCode === fieldCountryCode) {
      return true;
    }
    return false;
  },

  getCouponCode: function() {
    return $.trim($('#coupon-code').val()).toUpperCase();
  },

  showBillingStart: function(dateString, freeOrDiscounted, discountedMonths) {
    // date string comes back in ISO8601 format in UTC time.
    if (freeOrDiscounted === 'free' && !this.isForever(discountedMonths)) {
      var date = moment.utc(dateString).local().format('DD MMM YYYY');
      $('#top-billing-cycle-start-date').text(date);
      $('#bottom-billing-cycle-start-date').text('on ' + date);
      $('#billing-cycle-start').removeClass('hidden');
    }
  },

  isForever: function(discountedMonths) {
    return discountedMonths === null;
  },

  getPlanData: function(responseData) {
    for(var i = 0; i < responseData.length; i++) {
      if (responseData[i].id === parseInt(Common.params.plan_id)) {
        return responseData[i];
      }
    }
  },

  showCouponDuration: function(attributes, freeOrDiscounted) {
    $('#coupon-box').addClass('hidden');
    $('#coupon-duration').removeClass('hidden');
    var formatedAttributes = SinglePageCheckoutV2.formatPlanAttributes(attributes);

    var discountFormatted = ""

    if (attributes.discount_percent_off != null) {
      discountFormatted = attributes.discount_percent_off + '%'
    } else {
      discountFormatted = '$' + (attributes.discount_amount / 100)
    }

    this.addCouponDurationToOrderSummary(discountFormatted, formatedAttributes);
    this.addCouponDurationToBottom(freeOrDiscounted, formatedAttributes);
  },

  formatPlanAttributes: function(attributes) {
  var information = {};
    information.isForever = SinglePageCheckoutV2.isForever(attributes.discount_duration_in_months);
    if (attributes.interval === 'year' && !information.isForever) {
      information.duration = 'year';
      information.discount_duration = Math.ceil(attributes.discount_duration_in_months / 12);
    } else {
      information.duration = 'month';
      information.discount_duration = attributes.discount_duration_in_months;
    }
    information.sOrEmpty = information.discount_duration === 1 ? '' : 's';
    return information;
  },

  addCouponDurationToOrderSummary: function(discountFormatted, planSpecifics) {
    var monthText = '';
    if (!planSpecifics.isForever) {
      monthText = ' for ' + planSpecifics.discount_duration + ' ' + planSpecifics.duration + planSpecifics.sOrEmpty;
    }
    $('#coupon-duration .duration').text(discountFormatted + ' OFF' + monthText);
  },

  addCouponDurationToBottom: function(freeOrDiscounted, planSpecifics) {
    if (!planSpecifics.isForever) {
      if (freeOrDiscounted === 'free') {
        var discountDuration = planSpecifics.discount_duration;
        isOrAre = discountDuration === 1 ? 'is' : 'are';
        discountedTime = discountDuration === 1 ? '' : discountDuration + ' ';
        $('.bottom-information .duration').text('Your first ' + discountedTime + planSpecifics.duration + planSpecifics.sOrEmpty + ' ' + isOrAre + ' ' + 'free. ');
      }
    }
  },

  changeBottomPrice: function(freeOrDiscounted, discountedMonths) {
    if(freeOrDiscounted === 'discounted' || this.isForever(discountedMonths)) {
      $('#bottom-plan-price').empty();
    }
  },

  setOrderSummaryValues: function() {
    $('.subtotal-value').text('$' + Tools.formatCurrency(SinglePageCheckoutV2.originalPrice.toFixed(2).replace('.00', '')))
    $('.estimated-tax-value').text('$' + Tools.formatCurrency(SinglePageCheckoutV2.taxAmount.toFixed(2)))
    finalPrice = SinglePageCheckoutV2.originalPrice - parseFloat(SinglePageCheckoutV2.discountAmount.toFixed(2)) + parseFloat(SinglePageCheckoutV2.taxAmount.toFixed(2))
    $('.final-plan-price').text('$' + Tools.formatCurrency(finalPrice.toFixed(2)).replace('.00', ''))

    isFreeCoupon = SinglePageCheckoutV2.originalPrice === parseFloat(SinglePageCheckoutV2.discountAmount.toFixed(2))
    nextBillingPrice = isFreeCoupon ? (SinglePageCheckoutV2.originalPrice + parseFloat(SinglePageCheckoutV2.freeCouponTaxAmount.toFixed(2))) : finalPrice
    $('.next-billing-plan-price').text('$' + Tools.formatCurrency(nextBillingPrice.toFixed(2)).replace('.00', ''))
  },

  changePlanPrice: function(discountedPrice) {
    price = discountedPrice / 100;
    if (price % 1 !== 0) {
      price = price.toFixed(2);
    }

    SinglePageCheckoutV2.discountedPrice = price
    SinglePageCheckoutV2.discountAmount  = SinglePageCheckoutV2.originalPrice - price // Stored in Dollars

    var zipInput   = SinglePageCheckoutV2.postalCode
    var stateInput = $('#region-code')[0]
    if (zipInput.length >= 5 && stateInput.value) {
      SinglePageCheckoutV2.updateSalesTax(zipInput, stateInput.value)
    } else {
      SinglePageCheckoutV2.taxAmount = 0.0
      SinglePageCheckoutV2.setOrderSummaryValues()
    }
  },

  couponIsValid: function() {
    return $('#coupon-code').attr('data-valid') === 'true';
  },

  // ERROR HANDLING -----

  clearEmailSuggestionNotice: function(){
    $(".email-validation-update-email-prompt").remove();
  },

  clearFieldError: function(id) {
    $(id).removeClass('show');
  },

  clearPasswordErrors: function() {
    $("#error-password").removeClass("show");
    $("#error-confirm-password").removeClass("show");
  },

  clearExistingAccountPasswordErrors: function() {
    $('#existing-account-password').removeClass('input-error');
    $('#error-existing-account-password').removeClass('show');
    $('#success-reset-password-email-link').removeClass('show');
    $('#error-reset-password-email-link').removeClass('show');
  },

  showFieldErrors: function(okObject) {
    Object.keys(okObject).forEach(function(key) {
      if (!okObject[key] && !(key === 'passwordOK' || key === 'confirmPasswordOK')) {
        $(SinglePageCheckoutV2.createFieldErrorId(key.slice(0, -2))).addClass('show');
        $('#' + kebabCase(key.slice(0, -2))).addClass('input-error');
      }
    });

    if (!okObject.passwordOK) {
      $("#error-password").addClass("show");
      $("#password").addClass('input-error');
    } else {
      if (!okObject.confirmPasswordOK) {
        $('#error-confirm-password').addClass('show');
        $('#confirm-password').addClass('input-error');
      }
    }

    if (!okObject.expirationOK && !okObject.paymentInfoOK) {
      $('#error-card-expiration').addClass('show');
      $('#card-expiration').addClass('input-error');
    }

    if (!okObject.cardCvcOK && !okObject.paymentInfoOK) {
      $('#error-cvc-code').addClass('show');
      $('#cvc-code').addClass('input-error');
    }
  },

  showStripeError: function(message) {
    $('#stripe-errors').text(message);
    $('#stripe-errors').addClass('show');
  },

  receiveFiveOhThreeError: function() {
    $('#error-five-oh-three').addClass('show');
  },

  receiveNonFiveOhThreeError: function(errors) {
    errors.forEach(function(error) {
      var errorSource = SinglePageCheckoutV2.getErrorSource(error.source.pointer);
      var section = SinglePageCheckoutV2.getErrorSection(errorSource);
      var errorSectionId = '#' + section + '-' + 'api-errors';
      var errorSourceWithDashes = errorSource.replace('_', '-');
      var inputId = '#' + errorSourceWithDashes;
      var $element = $('<div class="error show">' + error.title + '.</div>');
      switch(error.title) {
        case 'Email has already been taken':
          var href = "/login";
          var preservedParamsObject = Common.preservedParamsObject();
          if (Object.keys(preservedParamsObject).length > 0) {
            href = href + "?" + Tools.getStringFromObject(preservedParamsObject, '&');
          }
          errorHtml = '<span>An account with that email already exists. <a class="email-error-message-login-link" href="' + href + '">Please log in.</a></span>';
          break;
        default:
          errorHtml = '<span>' + error.title + '</span>';
      }
      $('#error-' + errorSourceWithDashes + '-api').html(errorHtml);
      $(inputId).addClass('input-error');
      $('#error-' + errorSourceWithDashes + '-api').addClass('show');
      SinglePageCheckoutV2.addEventListenerToApiErrors($element, inputId);
      $(errorSectionId).append($element);
    });
  },

  addEventListenerToApiErrors: function($element, inputId) {
    var $input = $(inputId);
    if (!$input.hasClass('stripe-element')) {
      $input.on('change input', function() {
        $element.remove();
      });

      if (inputId == '#password') {
        $('#confirm-password').on('change input', function() {
          $element.remove();
        });
      }
    }
  },

  getErrorSource: function(pointer) {
    return pointer.split('/').slice(-1)[0];
  },

  getErrorSection: function(errorSource) {
    var errorSectionsObject = {
      'order-summary': ['coupon_code'],
      'account-information': ['email', 'school_name', 'account_name', 'password'],
      'payment-information': ['stripe_token', 'country_code', 'region_code', 'postal_code']
    };
    var matchedSections = Object.keys(errorSectionsObject).filter(function(section) {
      return (errorSectionsObject[section].indexOf(errorSource) > -1);
    });
    return matchedSections[0] || 'order-summary';
  },

  autoScrollToTopError: function() {
    var topError = $('.error.show')[0];
    $(window).scrollTop(0);
  },

  errorReceived: function() {
    SinglePageCheckoutV2.enablePurchaseButton();
    $('.LoadingIcon-container').addClass('hidden');
    SinglePageCheckoutV2.autoScrollToTopError();
  },

  receiveCouponError: function() {
    SinglePageCheckoutV2.autoScrollToTopError();
    SinglePageCheckoutV2.removeButtonWave();
    SinglePageCheckoutV2.enableCouponSubmit(true);
  },

  // SUBMISSION -----

  createStripeToken: function(fieldValues) {
    var tokenData = { address_country: fieldValues["country_code"] }
    if (fieldValues["country_code"] === "US") {
      tokenData.address_state = $('#region-code').val()
    }
    SinglePageCheckoutV2.stripe.createToken(SinglePageCheckoutV2.cardNumber, tokenData).then(function(response) {
      if (response.token) {
        if (SinglePageCheckoutV2.countryVerified(response.token.card.country, fieldValues.country_code)) {
          fieldValues.stripe_token = response.token.id;
          fieldValues.school_plan_id = Common.params.plan_id;
          fieldValues.postal_code = response.token.card.address_zip;
          SinglePageCheckoutV2.createSchoolAndUpgrade(fieldValues);
        } else {
          $('#error-country-code-verification').addClass('show');
          SinglePageCheckoutV2.errorReceived();
        }
      } else {
        SinglePageCheckoutV2.showStripeError(response.error.message);
        SinglePageCheckoutV2.errorReceived();
        Tracking.irisEmit('single_page_checkout_v2_stripe_token_error', {
          email: fieldValues.email,
          target: 'segment'
        });
      }
    });
  },

  getRemainingAttributes: function(attributes) {
    var coupon = SinglePageCheckoutV2.getCouponCode();

    if (this.couponIsValid()) {
      attributes.coupon_code = coupon;
    }

    var referralToken = localStorage.getItem("referral");

    if (referralToken) {
      attributes.referral_token = referralToken;
    }

    if (attributes.country_code === 'US') {
      attributes.region_code = $('#region-code').val();
    }

    var marketingParams = [
      'upgrade_src',
      'utm_source',
      'utm_medium',
      'utm_campaign',
      'utm_content',
      'mt_campaign_group',
      'mt_tier',
      'mt_interval',
      'i_campaign_id',
      'i_template_id'
    ];

    marketingParams.forEach(function(param) {
      var paramValue = Common.params[param];
      if (paramValue) {
        attributes[param] = paramValue;
      }
    });

    return attributes;
  },

  createSchoolAndUpgrade: function(fieldValues) {
    var allAttributes = SinglePageCheckoutV2.getRemainingAttributes(fieldValues);
    $('.LoadingIcon-container').removeClass("hidden")
    $.ajax({
      url: SinglePageCheckoutV2.getProperEndPoint() + '/school_sign_ups',
      method: 'POST',
      data: {
        data: {
          type: 'schools',
          attributes: allAttributes
        }
      },
      success: function(response) {
        SinglePageCheckoutV2.receiveApiSuccess(response.data, fieldValues.email);
      },
      error: function(response) {
        var responseJSON =  response.responseJSON;
        if (responseJSON && responseJSON.errors) {
          SinglePageCheckoutV2.receiveNonFiveOhThreeError(responseJSON.errors);
        } else {
          SinglePageCheckoutV2.receiveFiveOhThreeError();
        }
        SinglePageCheckoutV2.errorReceived();
      }
    });
  },

  poll: function(url, email) {
    setTimeout(function() {
      $.ajax({
        url: SinglePageCheckoutV2.formatUrl(url),
        method: 'GET',
        cache: false,
        success: function(response) {
          SinglePageCheckoutV2.receiveApiSuccess(response.data, email);
        },
        error: function(response) {
          SinglePageCheckoutV2.receiveFiveOhThreeError();
          SinglePageCheckoutV2.errorReceived();
        }
      });
    }, 1000);
  },

  formatUrl: function(url) {
    //this is for testing environment
    if ($('meta[id=environment]').attr('data-env') === 'test') {
      var index = url.indexOf('/school_sign_ups');
      return (url.slice(index));
    }
    return url;
  },

  receiveApiSuccess: function(responseData, email) {
    if (responseData.links.poll) {
      SinglePageCheckoutV2.poll(responseData.links.poll, email);
    } else {
      localStorage.setItem('email', email);
      const iterableEvent = new Promise(resolve => Iterable.sendEvent({
        eventName: 'Single Page Checkout V2 - Account Created',
        email: email,
        instanceId: 1,
        isEmailSubmittedEvent: true
      }).then(resolve));
      // if we had a teachable account id before, the account wasn't created
      const accountCreatedEvent = SinglePageCheckoutV2.teachableAccountID
        ? Promise.resolve()
        : Promise.all([sendAccountCreatedConversion(), sendPurchasedSchoolPlanConversion()]);

      Promise.all([accountCreatedEvent, iterableEvent]).then(() => {
        Tracking.irisEmit('checked_out', {
          email: email,
          checkoutVersion: 'v2',
        });
        redirectUrl = SinglePageCheckoutV2.postCheckoutUrl || responseData.links.redirect_to;
        Common.redirectWithPreservedParams(redirectUrl);
      });
    }
  },

  updateSalesTax: function(zipCode, stateAbr) {
    const nextAmount = SinglePageCheckoutV2.originalPrice - SinglePageCheckoutV2.discountAmount;
    return new Promise(function(resolve, reject) {
      // do a thing, possibly async, then…
      $.ajax({
        url: '/api/sales_tax',
        method: 'POST',
        data: {
          zip_code: zipCode,
          state:    stateAbr,
          amount:   (nextAmount === 0 ? SinglePageCheckoutV2.originalPrice : nextAmount) * 100
        },
        success: function(response) {
          SinglePageCheckoutV2.taxAmount = nextAmount === 0 ? 0 : response.tax_amount / 100;
          SinglePageCheckoutV2.freeCouponTaxAmount = response.tax_amount / 100;
          SinglePageCheckoutV2.setOrderSummaryValues();
          resolve(response)
        },
        error: function(response) {
          console.log('Uh-oh bruv');
          reject(Error('It broke'));
        }
      })
    });
  },

  submitCouponCode: function() {
    var couponCode = encodeURI(SinglePageCheckoutV2.getCouponCode());
    let couponApplicationSucceeded;
    $.ajax({
      url: SinglePageCheckoutV2.getProperEndPoint() + '/school_plans?coupon_code=' + couponCode + '&school_plan_id=' + Common.params.plan_id,
      method: 'GET',
      cache: false,
      success: function(response) {
        var responseData = response.data;
        var planData = SinglePageCheckoutV2.getPlanData(responseData);
        if (planData && planData.attributes.discount_name) {
          $('#coupon-code').attr('data-valid', true);
          var planAttributes = planData.attributes;
          var freeOrDiscounted = planAttributes.discount_percent_off === 100 ? 'free' : 'discounted';
          SinglePageCheckoutV2.changePlanPrice(planAttributes.discounted_amount);
          SinglePageCheckoutV2.showCouponDuration(planAttributes, freeOrDiscounted);
          SinglePageCheckoutV2.showBillingStart(planAttributes.payment_date, freeOrDiscounted, planAttributes.discount_duration_in_months);
          SinglePageCheckoutV2.changeBottomPrice(freeOrDiscounted, planAttributes.discount_duration_in_months);
          couponApplicationSucceeded = true
        } else {
          $('#error-coupon-code').addClass('show');
          SinglePageCheckoutV2.receiveCouponError();
          couponApplicationSucceeded = false
        }
        if (!SinglePageCheckoutV2.pageViewSentToIterable) {
          SinglePageCheckoutV2.sendPageViewToIterable();
        }
        Tracking.irisEmit('submitted_coupon', {
          data: {
            coupon_code: couponCode,
            application_succeeded: couponApplicationSucceeded,
            teachable_account_id: SinglePageCheckoutV2.teachableAccountID,
            plan_id: Common.params.plan_id,
          },
        });
      },
      error: function(response) {
        SinglePageCheckoutV2.receiveFiveOhThreeError();
        SinglePageCheckoutV2.receiveCouponError();
      }
    });
  },

  // STRIPE -----

  setUpStripe: function() {
    var fonts = {
      fonts: [
        {
          family: 'TeachableSans-Regular',
          src: 'local("TeachableSans-Regular"), url("https://d2oz8i5n9se8ej.cloudfront.net/fonts/teachable-sans/TeachableSans-Medium.woff") format("woff"), url("https://d2oz8i5n9se8ej.cloudfront.net/fonts/teachable-sans/TeachableSans-Medium.ttf") format("truetype")',
        }
      ],
    };

    var style = {
      base: {
        fontFamily: 'TeachableSans-Regular',
        fontSize: '13px',
        letterSpacing: '0.8px',
        color: '#2C2F33',
        lineHeight: '15px',
        fontSmoothing: 'antialiased',
        '::placeholder': {
          color: '#ADADAD',
        }
      },
      invalid: {
        color: '#2C2F33',
      }
    };

    SinglePageCheckoutV2.stripe = Stripe($('#public-stripe-key').attr('value'));
    var elements = SinglePageCheckoutV2.stripe.elements(fonts);
    SinglePageCheckoutV2.cardNumber = elements.create('cardNumber', {style: style});
    var expiration = elements.create('cardExpiry', {style: style});
    var cardCvc = elements.create('cardCvc', {style: style});
    var postalCode = elements.create('postalCode', {style: style});
    SinglePageCheckoutV2.cardNumber.mount('#card-number');
    expiration.mount('#card-expiration');
    cardCvc.mount('#cvc-code');
    postalCode.mount('#postal-code');
    postalCode.addEventListener('change', SinglePageCheckoutV2.changePostalCode)
    SinglePageCheckoutV2.addListenersToElements({
      'cardNumber': SinglePageCheckoutV2.cardNumber,
      'cardExpiration': expiration,
      'cvcCode': cardCvc,
      'postalCode': postalCode
    });
  },

  addListenersToElements: function(elementsObject) {
    SinglePageCheckoutV2.toggleCreditCardImage();
    Object.keys(elementsObject).forEach(function(name) {
      SinglePageCheckoutV2.toggleElementComplete(elementsObject[name], name);
    });
  },

  toggleCreditCardImage: function() {
    SinglePageCheckoutV2.cardNumber.on('change', function(event) {
      SinglePageCheckoutV2.removeCreditCardImage();
      if (event.brand !== 'unknown') {
        var $image = $('#' + event.brand);
        if (!$image.hasClass('show')) {
          $('#' + event.brand).addClass('show');
        }
      }
    });
  },

  removeCreditCardImage: function() {
    var $creditCardImages = $('.card-number-group img');
    $creditCardImages.each(function(image) {
      $($creditCardImages[image]).removeClass('show');
    });
  },

  toggleElementComplete: function(element, name) {
    var property = name + 'Complete';
    var id = '#error-' + kebabCase(name);
    element.on('change', function(event) {
      $(id).removeClass('show');
      $(element._parent).removeClass('input-error');
      if (event.complete) {
        SinglePageCheckoutV2[property] = true;
      } else {
        if (SinglePageCheckoutV2[property]) {
          SinglePageCheckoutV2[property] = false;
        }
      }
    });
  }
};
