$(document).ready(function () { StickyScroll.initialize(); });

var StickyScroll = {

  rectData: {},

  initialize: function() {
    StickyScroll.getRectData();
    $(window).scroll(StickyScroll.onScroll);
    $(window).resize(StickyScroll.onResize);
  },

  onScroll: function() {
    StickyScroll.getRectData();
    StickyScroll.update();
  },

  onResize: function() {
    for (var id in StickyScroll.rectData) {
      var el = StickyScroll.rectData[id].element;
      var isActive = el.classList.contains('sticky');
      if (isActive) {
        StickyScroll.removeFromStickyFlow(el);
      }
    }

    StickyScroll.getRectData(willReset = false);
    StickyScroll.update();
  },

  update: function () {
    for (var id in StickyScroll.rectData) {
      var el = StickyScroll.rectData[id].element;
      var rect = StickyScroll.rectData[id].rect;
      var stackedOffset = StickyScroll.getStackedOffsetTop();
      var pageY = window.pageYOffset + stackedOffset;
      var isActive = el.classList.contains('sticky');
      var boundingElement = null
      if (el.nextElementSibling.classList.contains("sticky-scroll-end")) {
        boundingElement = el.nextElementSibling
      }

      if (isActive && (pageY < rect.top + rect.height || (boundingElement && pageY > $(boundingElement).offset().top + boundingElement.offsetHeight))) {
        StickyScroll.removeFromStickyFlow(el);
      } else if (!isActive && ((pageY > rect.top) && (!boundingElement || pageY < $(boundingElement).offset().top + boundingElement.offsetHeight))) {
        StickyScroll.addToStickyFlow(el, stackedOffset);
      }
    }
  },

  addToStickyFlow: function(el, top) {
    var rect = StickyScroll.rectData[$(el).attr('id')].rect;
    var $el = $(el);
    var $surrogateEl = $el.clone().addClass('sticky-surrogate');

    if (el.tagName === 'THEAD') {
      $el.find('th').each(function(i, elTH) {
        $elTH = $(elTH);
        $elTH.css('width', $elTH.css('width')); // ensure width for table columns
      });
    }

    $('.sticky:not(.permanent-stick)').each(function(i,element) {
      StickyScroll.removeFromStickyFlow(element)
    });

    var stackedOffset = StickyScroll.getStackedOffsetTop();

    $el
      .addClass('sticky')
      .css({ top: stackedOffset, left: rect.left, width: rect.width })
      .before($surrogateEl);
  },

  removeFromStickyFlow: function(el) {
    $(el)
      .removeClass('sticky')
      .removeAttr('style')
      .prev('.sticky-surrogate').remove()

    if (el.tagName === 'THEAD') {
      $(el).find('th').each(function(i, elTH) {
        $(elTH).removeAttr('style');
      });
    }
  },

  getRectData: function(willReset) {
    if (willReset === undefined) {
      willReset = true;
    }
    $('.sticky-scroll').each(function(i, el) {
      var $el = $(el);
      var id = $el.uniqueId().attr('id');
      if (StickyScroll.rectData[id] && willReset) { return true; }

      var rect = el.getBoundingClientRect();
      var offset = $el.offset();

      StickyScroll.rectData[id] = {
        element: el,
        rect: {
          top: offset.top,
          left: offset.left,
          width: rect.width,
          height: rect.height,
          margin: $el.css('margin'),
        },
      };
    });
  },

  getStackedOffsetTop: function () {
    return $('.sticky:not(.hidden)').toArray()
      .map(function (el) {
        return el.getBoundingClientRect().height;
      })
      .reduce(function (totalHeight, elHeight) {
        return totalHeight + elHeight;
      }, 0);
  },
};
