$(document).ready(function() {
  SubscriptionPreferences.initialize();
});

const arraysIntersect = (arr1, arr2) => arr1.some(el => arr2.includes(el));

var UNSUBSCRIBE_FROM_ALL_CHANNEL_IDS = [
  15680,
  15681,
]

var SubscriptionPreferences = {

  email: null,
  unsubscribedChannelIds: [0],
  unsubscribedMessageTypeIds: [0],

  initialize: function() {
    var email = Common.params.email;
    var emailId = Common.params.id;
    if ((window.location.pathname.indexOf('subscription-preferences') === -1) || !email) {
      SubscriptionPreferences.showErrorMessage();
      return;
    }
    SubscriptionPreferences.email = email || "";
    SubscriptionPreferences.emailId = emailId || "";
    $('#subscription-preferences-content li.my-checkbox-container.message-types').on('click', SubscriptionPreferences.toggleMessageIdCheckbox);
    $('#subscription-preference-channel').on('click', SubscriptionPreferences.toggleChannelIdCheckbox);
    $('#subscription-preferences-submit').on('click', SubscriptionPreferences.submitPreferences);
    SubscriptionPreferences.fetchPreferences();
  },

  fetchPreferences: function() {
    $.ajax({
      url: '/api/subscription_preferences?instance_id=1&email=' + SubscriptionPreferences.email + '&email_id=' + SubscriptionPreferences.emailId,
      success: function(response) {
        SubscriptionPreferences.unsubscribedMessageTypeIds = response.unsubscribedMessageTypeIds || [0];
        $('li.my-checkbox-container.message-types').each(function(i, el) {
          var ids = SubscriptionPreferences.getDataIds($(el));
          if (arraysIntersect(ids, SubscriptionPreferences.unsubscribedMessageTypeIds)) {
            $(el).click();
          }
        });
        for (var i = 0; i < UNSUBSCRIBE_FROM_ALL_CHANNEL_IDS.length; i++) {
          var id = UNSUBSCRIBE_FROM_ALL_CHANNEL_IDS[i];
          if (response.unsubscribedChannelIds.indexOf(id) > -1) {
            $('#subscription-preference-channel div.btn.checkbox').click();
            break;
          }
        }
        $('.subscription-preferences-large-spinner').addClass('hidden');
        $('#subscription-preferences-content').removeClass('hidden');
      },
      error: function(response) {
        SubscriptionPreferences.showErrorMessage();
        $('.subscription-preferences-large-spinner').addClass('hidden');
      }
    });
  },

  submitPreferences: function() {
    SubscriptionPreferences.showSpinner();
    var url = '/api/update_subscription_preferences?instance_id=1&email=' + SubscriptionPreferences.email + '&email_id=' + SubscriptionPreferences.emailId;
    var data = {
      unsubscribed_channel_ids: SubscriptionPreferences.unsubscribedChannelIds,
      unsubscribed_message_type_ids: SubscriptionPreferences.unsubscribedMessageTypeIds
    };
    $.ajax({
      method: 'POST',
      url: url,
      data: {
        unsubscribed_channel_ids: SubscriptionPreferences.unsubscribedChannelIds,
        unsubscribed_message_type_ids: SubscriptionPreferences.unsubscribedMessageTypeIds
      },
      success: function(response) {
        SubscriptionPreferences.hideSpinner()
        SubscriptionPreferences.showThankYouMessage()
      },
      error: function(response) {
        SubscriptionPreferences.hideSpinner()
        SubscriptionPreferences.showErrorMessage()
      }
    });
  },

  toggleMessageIdCheckbox: function(e) {
    e.preventDefault()
    var $el = $(e.currentTarget);
    SubscriptionPreferences.toggleCheckboxElement($el);
    var toggledIds = SubscriptionPreferences.getDataIds($el);
    var isToggled = $($el).find("div.btn.checkbox.checked")[0];
    if (isToggled) {
      SubscriptionPreferences.unsubscribedMessageTypeIds = SubscriptionPreferences.unsubscribedMessageTypeIds.filter(function(id) { return toggledIds.indexOf(id) === -1 });
    } else {
      SubscriptionPreferences.unsubscribedMessageTypeIds = SubscriptionPreferences.unsubscribedMessageTypeIds.concat(toggledIds);
    }
  },

  toggleChannelIdCheckbox: function(e) {
    e.preventDefault();
    var $el = $(e.currentTarget);
    SubscriptionPreferences.toggleCheckboxElement($el);
    var toggledIds = SubscriptionPreferences.getDataIds($el);
    var isToggled = !arraysIntersect(SubscriptionPreferences.unsubscribedChannelIds, toggledIds);
    if (isToggled) {
      SubscriptionPreferences.unsubscribedChannelIds = SubscriptionPreferences.unsubscribedChannelIds.concat(toggledIds)
      $('li.my-checkbox-container.message-types:not(.checked)').each(function(i, messageTypeEl) {
        $(messageTypeEl).click();
      });
    } else {
      SubscriptionPreferences.unsubscribedChannelIds = SubscriptionPreferences.unsubscribedChannelIds.filter(function(id) {
        return toggledIds.indexOf(id) === -1
      });
      $('li.my-checkbox-container.message-types.checked').each(function(i, messageTypeEl) {
        $(messageTypeEl).click()
      });
    }
  },

  showSpinner: function (){
    $('#subscription-preferences-small-spinner').removeClass('hidden')
  },

  hideSpinner: function () {
    $('#subscription-preferences-small-spinner').addClass('hidden')
  },

  getDataIds: function($el) {
    return $el.attr('data-ids').split(',').map(function(id) {
      return parseInt(id)
    });
  },

  toggleCheckboxElement: function($el) {
    $el.toggleClass('checked');
    $el.find('div.btn.checkbox').toggleClass('checked');
  },

  showThankYouMessage: function() {
    $('#subscription-preferences-container').css('display', 'none');
    $('#subscription-preferences-thank-you-container').addClass('show');
  },

  showErrorMessage: function() {
    $('#subscription-preferences-container').css('display', 'none');
    $('#subscription-preferences-error-container').addClass('show');
  }
};
