const { sendEmailSubmitConversion } = require('./conversion-events');

$(document).ready(function() {
  SummitRegisterForm.initialize();
});

var SummitRegisterForm = {

  initialize: function() {
    Common.clickOrEnter({
      click: ".summit-register-form-button",
      enter: ".summit-register-form input",
      action: SummitRegisterForm.submit
    });
    $(".summit-register-form input").on("keydown", SummitRegisterForm.clearFieldError);
    SummitRegisterForm.preFillEmail();
  },

  preFillEmail: function() {
    if (Common.params.email) {
      $(".summit-register-form .email-field").val(unescape(Common.params.email));
    }
  },

  submit: function(e) {
    var id = e.target.dataset.id;
    var $emailField = $(SummitRegisterForm.partialClass(id) + " .email-field");
    var $emailError = $(SummitRegisterForm.partialClass(id) + " .error-missing-email");
    var email = $.trim($emailField.val());
    var emailOK = Common.validateEmail(email);
    if (emailOK) {
      localStorage.setItem("email", email);
      SummitRegisterForm.sendDataThenRedirect({
        redirectUrl: e.target.dataset.redirect,
        summitId: e.target.dataset.summitId,
        title: e.target.dataset.title,
        startDate: e.target.dataset.startDate,
        endDate: e.target.dataset.endDate,
        email: email
      });
    } else {
      $emailError.removeClass("hidden");
    }
  },

  sendDataThenRedirect: function(args) {
    Tracking.irisEmit("marketing_enrolled_in_major_promotion", {
      target: "iterable",
      email: args.email,
      eventName: "Marketing - Enrolled in Major Promotion",
      instanceId: 1,
      data: {
        id: args.summitId,
        title: args.title,
        campaign_group: "summit",
        start_date: args.startDate,
        end_date: args.endDate
      },
      isEmailSubmittedEvent: true
    });
    const iterableSend = new Promise((resolve) => Iterable.sendEvent({
      email: args.email,
      eventName: "Marketing - Enrolled in Major Promotion",
      instanceId: 1,
      data: {
        id: args.summitId,
        title: args.title,
        campaign_group: "summit",
        start_date: args.startDate,
        end_date: args.endDate
      },
      isEmailSubmittedEvent: true
    }).then(resolve));
    Promise.all([sendEmailSubmitConversion(), iterableSend]).then(function() {
      SummitRegisterForm.redirect(args.redirectUrl, args.email);
    });
  },

  redirect: function(url, email) {
    var cleanUrl = url.split("?")[0];
    var params = Common.params;
    params.email = email;
    window.location.href = cleanUrl + Common.paramsObjectToSearchString(params);
  },

  clearFieldError: function(e) {
    var children = e.target.parentElement.children;
    var errorDiv = children[children.length - 1];
    errorDiv.classList.add("hidden");
  },

  partialClass: function(id) {
    return "._" + id.toString() + ".summit-register-form";
  }
};
