const { iris } = require('@teachable/iris');

$(document).ready(function() {
  if (window.location.pathname.indexOf("admin") === -1) {
    Tracking.initialize();
  }
});

var Tracking = {

  initialize: function() {
    Tracking.isProduction = (Tracking.pullEnvironmentFromDOM() === "production");
    Tracking.initializeIris();
    Tracking.firePageLoadEvent();
  },

  firePageLoadEvent: function() {
    Tracking.pageLoadTrackerReady = false;
    Tracking.elapsedLoadTime = 0;
    Tracking.poller = setInterval(function() {
      Tracking.pollForAnalyticsData();
    }, 200);
  },

  initializeIris: function() {
    if (Tracking.isProduction) {
      Tracking.iris = iris.init({
        selector: "body",
        source: "cms",
        noun: "user",
        noun_id: Tracking.anonymousId(),
        url: document.getElementById("environment").dataset.eventable
      });
    }
  },

  irisEmit: function(label, metadata) {
    if (Tracking.isProduction) {
      metadata = metadata || {};
      var regex = /\-+|\s+/gm;
      var formattedLabel = label.toLowerCase().replace(regex,'_');

      // If adblock is not turned on for the user, gaData should be set by Google Analytics
      // Within that we have access to the experiments/cohorts that folks are dropped into by
      // Google Optimize. For all experiments that they are in, we want to send that as metadata
      // to eventable so that we can see how experiments perform when they might occur on the same
      // URL. Non-Google Analytics experiment data can also be grabbed from utm_expid parameter.
      if (typeof gaData !== 'undefined' || !!Common.params.utm_expid) {
        metadata = Tracking.populateExperimentData(metadata);
      };

      // send user email (if we have it cookied) by default, unless "email" is already included in metadata
      if (!metadata.email && localStorage.getItem("email")) {
        metadata.email = localStorage.getItem("email");
      }
      
      Tracking.iris.emit(formattedLabel, metadata);
    }
  },

  anonymousId: function() {
    return document.cookie.replace(/(?:(?:^|.*;\s*)_afid\s*\=\s*([^;]*).*$)|^.*$/, "$1");
  },

  gaAccountId: function() {
    return $('meta[name="ga-optimize-account-id"]').attr('content');
  },

  populateExperimentData: function(metadata) {
    if (!!Common.params.utm_expid) {
      var parsedParam = Common.params.utm_expid;
      // example parsedParam: .wXks5izpR0aJC9g4diDdDg.1
      metadata.experiment_id = parsedParam.split('.')[1];
      metadata.variant_group = parsedParam.split('.')[2];
    } else if (typeof gaData !== 'undefined' && gaData[Tracking.gaAccountId()] && gaData[Tracking.gaAccountId()].experiments) {
      metadata.experiment_id = Object.keys((gaData[Tracking.gaAccountId()].experiments))[0];
      metadata.variant_group = Object.values((gaData[Tracking.gaAccountId()].experiments))[0];
    };
    return metadata;
  },

  trackPageLoad: function() {
    if (Tracking.isProduction) {
      // ensure the page and all its javascripts are fully loaded before logging page_loaded event
      var url = window.location.href;
      var referrer = null;
      if (document.referrer.indexOf("teachable.com") === -1) {
        referrer = document.referrer;
      }

      // iris
      Tracking.irisEmit("page_loaded", { url: url, email: localStorage.getItem("email"), referrer: referrer });
    }
  },

  pollForAnalyticsData: function() {
    Tracking.checkGaDataInitialized();
    Tracking.elapsedLoadTime += 200;
    if (Tracking.pageLoadTrackerReady === true) {
      clearInterval(Tracking.poller);
      Tracking.trackPageLoad();
    };
  },

  checkGaDataInitialized: function() {
    if ((typeof gaData !== 'undefined' && gaData[Tracking.gaAccountId()]) || !!Common.params.utm_expid || Tracking.elapsedLoadTime >= 3000) {
      Tracking.pageLoadTrackerReady = true;
    };
  },

  isEnvironmentProduction: function() {
    // conditional needed in case Tracking.initialize has not yet been called
    return Tracking.isProduction || (Tracking.pullEnvironmentFromDOM() === "production")
  },

  pullEnvironmentFromDOM: function() {
    return document.getElementById("environment").dataset.env;
  },

  waitForTrackingToSend: function(callback) {
    // this is the same wait time as analytics.trackLink and analytics.trackForm
    setTimeout(callback, 300);
  },

  afidUrlParameter: function() {
    if (!!Tracking.anonymousId() && Tracking.isProduction) {
      return '&_afid=' + Tracking.anonymousId();
    };
  }
}

module.exports = Tracking;
