const moment = require('moment');
const escape = require('lodash/escape');

$(document).ready(function() {
  var directories = Common.getUrlDirectories();
  if (directories[directories.length - 3] === 'webinars' && directories[directories.length - 1] === 'play') {
    WebinarPlay.initialize();
  }
});

var WebinarPlay = {

  userCookie: null,

  downloadSwitches: {},

  initialize: function() {
    $('.email-gate .orange-button').on('click', WebinarPlay.clickWatch);
    $('.email-gate input').on('keydown', WebinarPlay.clearGateError);
    $('.chat-sidebar ul.clearfix li').click(WebinarPlay.clickChatBarSection);
    $('.chat-sidebar > img').click(WebinarPlay.hideSidebar);
    $('.background img.chat').click(WebinarPlay.showSidebar);
    WebinarPlay.startCountdown();
    WebinarPlay.checkToSkipEmailGate();
    WebinarPlay.showFirstChatSection();
    if (navigator.cookieEnabled) {
      Common.clickOrEnter({
        click: '#join-chat-button',
        enter: '#join-chat-input',
        action: WebinarPlay.joinChat
      });
      $('#join-chat-input').on('keydown', WebinarPlay.clearError);
      WebinarPlay.checkForCookie();
    } else {
      WebinarPlay.displayCookiesDisabledError();
    }
  },

  showFirstChatSection: function() {
    var firstSection = $(".sidebar-section")[1];
    if (firstSection.classList.contains("chat-section") || firstSection.classList.contains("replies-section")) {
      $(".join-chat-section").show();
    } else {
      $(firstSection).show();
    }
  },

  checkToSkipEmailGate: function() {
    var urlEmail = decodeURIComponent(Common.params.email);
    if (Common.validateEmail(urlEmail)) {
      WebinarPlay.hideEmailGate();
      WebinarPlay.sendTracking(urlEmail);
    } else if (navigator.cookieEnabled) {
      var cookiedEmail = localStorage.getItem("email");
      if (Common.validateEmail(cookiedEmail)) {
        WebinarPlay.hideEmailGate();
        WebinarPlay.sendTracking(cookiedEmail);
      }
    }
  },

  clickWatch: function(e) {
    e.preventDefault();
    var firstName = $(".email-gate .input-container:first-of-type input").val();
    var email = $(".email-gate .input-container:last-of-type input").val();
    var emailOK = Common.validateEmail(email);
    var firstNameOK = firstName !== "";
    if (emailOK && firstNameOK) {
      WebinarPlay.hideEmailGate();
      WebinarPlay.sendTracking(email);
      if (navigator.cookieEnabled) {
        localStorage.setItem("email", email);
        WebinarPlay.joinChat(firstName);
      }
    } else {
      if (!firstNameOK) {
        $(".input-container:first-of-type .error").show();
      }
      if (!emailOK) {
        $(".input-container:last-of-type .error").show();
      }
    }
  },

  clearGateError: function(e) {
    var errorDiv = e.target.parentElement.children[1];
    errorDiv.setAttribute("style", "display: none");
  },

  hideEmailGate: function() {
    $(".email-gate").hide();
  },

  sendTracking: function(email) {
    var date = new Date;
    var dataDiv = document.getElementById("tracking-data");
    Tracking.irisEmit("webinar_attended", {
      email: email,
      description: date.getFullYear() + "_" + (date.getMonth() + 1) + "_webinar_attended",
      webinar_date: dataDiv.dataset.date,
      webinar_id: dataDiv.dataset.id,
      target: ["segment", "iterable", ],
      iterable: {  // TODO: Remove this and handle within iterable-client
        instance_id: 1
      }
    });
    Iterable.sendEvent({
      eventName: "Webinar - Attended Webinar",
      email: email,
      instanceId: 1
    });
  },

  checkForCookie: function() {
    var name = localStorage.getItem(WebinarPlay.cookieKeyPrefix() + '-name');
    if (name) {
      var color = localStorage.getItem(WebinarPlay.cookieKeyPrefix() + '-color');
      var id = localStorage.getItem(WebinarPlay.cookieKeyPrefix() + '-id');
      WebinarPlay.userCookie = { id: id, name: name, color: color };
      $('.chat-sidebar ul.clearfix li')[0].click();
    }
  },

  firebaseUpdate: function(snap) {
    var obj = snap.val();

    // hangout status
    var classNames = {};
    classNames['Starting'] = ".status-starting";
    classNames['Go live'] = ".status-live";
    classNames['End'] = ".status-end";
    var className = classNames[obj.controls.hangoutStatus];
    $('.status-section').hide();
    $(className).show();

    // youtube id
    var $video = $('.video');
    if (obj.controls.hangoutStatus === 'Go live') {
      var videoUrl;
      if (document.getElementById("platform").dataset.platform === "vimeo") {
        videoUrl = "https://player.vimeo.com/video/" + obj.controls.youTubeID;
      } else {
        videoUrl = "https://www.youtube.com/embed/" + obj.controls.youTubeID + "?showinfo=0&rel=0&loop=1&playlist=" + obj.controls.youTubeID + "&modestbranding=1&autoplay=true";
      }
      if ($video.children().length === 0) {
        $video.html('<iframe src="' + videoUrl + '" allowfullscreen />');
      }
    } else {
      $video.html('');
    }

    // cta
    if (obj.controls.showUpgradeButton) {
      $('.cta').show();
    } else {
      $('.cta').hide();
    }

    // downloads
    obj.controls.downloadSwitches.forEach(function(downloadSwitch, index) {
      $li = $('.downloads-section li:nth-of-type(' + (index + 1) + ')');
      if (downloadSwitch) {
        $li.show();
      } else {
        $li.hide();
      }
      if (downloadSwitch && !WebinarPlay.downloadSwitches[index]) {
        if (!$('ul.clearfix > li:nth-of-type(2)').hasClass('active')) {
          $('ul.clearfix > li:nth-of-type(2) img.notification').show();
        }
      }
      WebinarPlay.downloadSwitches[index] = downloadSwitch;
    });

    // chat
    Chat.updateChatSection(obj);
  },

  startCountdown: function() {
    var $countDownDiv = $('.date-for-countdown');
    if ($countDownDiv.length > 0) {
      var timeString = $countDownDiv.html();
      var time = new Date(timeString * 1000);
      var countDown = $('.inner-countdown')[0];
      var daysDiv = $('.count.days p')[0];
      var hoursDiv = $('.count.hours p')[0];
      var minutesDiv = $('.count.minutes p')[0];
      var secondsDiv = $('.count.seconds p')[0];
      window.setInterval(WebinarPlay.renderCountdown.bind(this, time, daysDiv, hoursDiv, minutesDiv, secondsDiv), 1000);
    }
  },

  renderCountdown: function(time, daysDiv, hoursDiv, minutesDiv, secondsDiv) {
    var difference = time.getTime() - Date.now();
    var duration = moment.duration(difference);
    var days, hours, minutes, seconds;
    if (duration > 0) {
      days = Math.floor(duration.asDays());
      hours = Math.floor(duration.hours());
      minutes = Math.floor(duration.minutes());
      seconds = Math.floor(duration.seconds());
    } else {
      days = 0;
      hours = 0;
      minutes = 0;
      seconds = 0;
    }
    daysDiv.innerHTML = days;
    hoursDiv.innerHTML = hours;
    minutesDiv.innerHTML = minutes;
    secondsDiv.innerHTML = seconds;
  },

  hideSidebar: function() {
    $('.background').addClass("full");
    $('.chat-sidebar').hide();
    $('.background img.chat').show();
  },

  showSidebar: function() {
    $('.background').removeClass("full");
    $('.chat-sidebar').show();
    $('.background img.chat').hide();
  },

  clickChatBarSection: function(e) {
    var target = e.target.dataset.section ? e.target : e.target.parentElement;
    var section = target.dataset.section;
    var titles = {
      'chat': 'Chat Room',
      'downloads': 'Resources',
      'replies': 'Replies',
      'faq': 'FAQ'
    };
    $('.chat-sidebar > h1').html(titles[section]);
    if ((section == 'chat' || section =='replies') && !WebinarPlay.userCookie) {
      section = 'join-chat'
    }
    $('.sidebar-section').hide();
    $('.chat-sidebar li').removeClass('active');
    $('.sidebar-section' + '.' + section + '-section').show();
    target.classList.add('active');
    if (section == 'chat') {
      Common.scrollOverflowElementToBottom($('.chat-messages')[0]);
    } else if (section == 'downloads') {
      $('li:nth-of-type(2) img.notification').hide();
    } else if (section == 'replies') {
      Common.scrollOverflowElementToBottom($('.reply-chat-messages')[0]);
      $('li:last-of-type img.notification').hide();
    }
  },

  displayCookiesDisabledError: function() {
    WebinarPlay.displayJoinError('Please enable cookies in your browser to join the chat');
    $('h1.default-header, #join-chat-input, #join-chat-button, .inner h2, .inner ul').hide();
  },

  joinChat: function(emailGateName) {
    var name = (typeof emailGateName === 'string' ? emailGateName : $('#join-chat-input').val());
    if (name.length === 0) {
      WebinarPlay.displayJoinError("Please enter your name");
    } else {
      var color = WebinarPlay.getRandomColor();
      var newRef = Chat.webinarReference.child('users').push();
      var obj = { id: newRef.key, name: escape(name), color: color, blocked: false };
      WebinarPlay.userCookie = obj;
      localStorage.setItem(WebinarPlay.cookieKeyPrefix() + '-id', newRef.key);
      localStorage.setItem(WebinarPlay.cookieKeyPrefix() + '-name', name);
      localStorage.setItem(WebinarPlay.cookieKeyPrefix() + '-color', color);
      newRef.set(obj, function() {
        $('.chat-sidebar ul.clearfix li')[0].click();
        Common.scrollOverflowElementToBottom($('.chat-messages')[0]);
      });
    }
  },

  displayJoinError: function(text) {
    $input = $('#join-chat-input');
    $input.addClass('error');
    $error = $('#join-chat-error');
    $error.html(text);
    $error.show();
  },

  clearError: function() {
    $input = $('#join-chat-input');
    $input.removeClass('error');
    $error = $('#join-chat-error');
    $error.html('');
    $error.hide();
  },

  getRandomColor: function() {
    var colors = ['#843b62', '#e04b7e', '#f42941', '#e9563d', '#ffc800', '#3cb878', '#10a7f3', '#1f6bde', '#6172e9', '#d252e8'];
    var n = Math.floor(Math.random() * colors.length);
    return colors[n];
  },

  cookieKeyPrefix: function() {
    return 'chat-' + Chat.webinarId + '-' + $('.time-utc').html();
  }
}

module.exports = WebinarPlay;
