import padStart from 'lodash/padStart';
import moment from 'moment';

$(document).ready(function () {
  var directories = Common.getUrlDirectories();
  if (
    directories[directories.length - 3] === 'webinars' &&
    directories[directories.length - 1] === 'register'
  ) {
    WebinarRegistration.initialize();
  }
});

var WebinarRegistration = {
  initialize: function () {
    WebinarRegistration.startCountdown();
    WebinarRegistration.checkForEmailParam();
    $('section.hero .orange-button').click(WebinarRegistration.clickHeroButton);
    $(
      'section.webinars-registration-automated .automated-webinar-registration-button'
    ).click(WebinarRegistration.clickHeroButton);
    $('.date-double p:last-of-type').click(
      WebinarRegistration.clickRegisterNow
    );
  },

  startCountdown: function () {
    var $countDownDiv = $('.date-for-countdown');
    if ($countDownDiv.length > 0) {
      var timeString = $countDownDiv.html();
      var time = new Date(timeString * 1000);
      var countDown = $('.inner-countdown')[0];
      var daysDiv = $('.count.days p')[0];
      var hoursDiv = $('.count.hours p')[0];
      var minutesDiv = $('.count.minutes p')[0];
      var secondsDiv = $('.count.seconds p')[0];
      window.setInterval(
        WebinarRegistration.renderCountdown.bind(
          this,
          time,
          daysDiv,
          hoursDiv,
          minutesDiv,
          secondsDiv
        ),
        1000
      );
    }
  },

  checkForEmailParam: function () {
    if (Common.params.email) {
      WebinarRegistration.clickHeroButton();
      $(
        '.webinar-registration-modal input[name=email]'
      )[0].value = decodeURIComponent(Common.params.email);
    }
  },

  renderCountdown: function (time, daysDiv, hoursDiv, minutesDiv, secondsDiv) {
    var difference = time.getTime() - Date.now();
    var duration = moment.duration(difference);
    var days, hours, minutes, seconds;
    if (duration > 0) {
      days = Math.floor(duration.asDays());
      hours = Math.floor(duration.hours());
      minutes = Math.floor(duration.minutes());
      seconds = Math.floor(duration.seconds());
    } else {
      days = 0;
      hours = 0;
      minutes = 0;
      seconds = 0;
    }
    daysDiv.innerHTML = padStart(days, 2, '0');
    hoursDiv.innerHTML = padStart(hours, 2, '0');
    minutesDiv.innerHTML = padStart(minutes, 2, '0');
    secondsDiv.innerHTML = padStart(seconds, 2, '0');
  },

  clickHeroButton: function () {
    var date = $('div.modal-date').html();
    WebinarRegistration.displayModal(date);
    Tracking.irisEmit('webinar_registration_button_click', {
      target: 'segment',
    });
  },

  clickRegisterNow: function (e) {
    var date = e.target.parentElement.parentElement.children[0].innerHTML;
    var time2 = e.target.dataset.time2;
    WebinarRegistration.displayModal(date, time2);
    Tracking.irisEmit('webinar_registration_button_click', {
      target: 'segment',
    });
  },

  displayModal: function (date, time2) {
    $('.modal-background').css('display', 'block');
    $('.webinar-modal-container').css('display', 'flex');
    $('.webinar-registration-modal .text-container p:last-of-type').html(date);
    $('.webinar-registration-modal div.time2').html(
      time2 ? time2.toString() : ''
    );
    $('.webinar-registration-modal input')[0].value = '';
    WebinarRegistrationModal.clearError();
  },
};
