class Admin::Api::BannersController < AdminController

  def index
    @banners = Banner.all
    render "index.json.jbuilder"
  end

  def new
    @banner_keys = Banner.keys
    render "new.json.jbuilder"
  end

  def create
    banner = Banner.new(banner_params)
    if banner.save
      @banners = Banner.all
      render "index.json.jbuilder"
    else
      render json: banner.errors.full_messages, status: 422
    end
  end

  def show
    @banner, @banner_fields_config, @banner_fields_object, @pages, @banner_pages, @webinars, @banner_webinars = show_data
    render "show.json.jbuilder"
  end

  def update
    @banner = Banner.find(params[:id])
    if @banner.update(banner_params)
      params[:banner_fields].each do |key, value|
        BannerField.find_or_create_by!(banner_id: @banner.id, banner_field_key: key).update(value: value)
      end
      @banner, @banner_fields_config, @banner_fields_object, @pages, @banner_pages, @webinars, @banner_webinars = show_data
      render "show.json.jbuilder"
    else
      render json: @banner.errors.full_messages, status: 422
    end
  end

  def destroy
    banner = Banner.find(params[:id])
    banner.destroy!
    render json: banner
  end

  private

  def banner_params
    params[:banner].permit(:name, :banner_key, :active, :inclusion_method)
  end

  def show_data
    banner = @banner || Banner.find(params[:id])
    banner_fields_config = banner.banner_config["fields"]
    banner_fields_object = banner.field_values_hash
    banner_pages = BannerPage.includes(:page).where(banner_id: banner.id)
    banner_webinars = BannerWebinar.includes(:webinar).where(banner_id: banner.id)
    [banner, banner_fields_config, banner_fields_object, Page.all, banner_pages, Webinar.all, banner_webinars]
  end

end
