class Admin::Api::BlogController < AdminController

  expire_page :@blog_settings, only: [:update_settings]

  def settings
    @blog_settings_hash = Objekt.blog_settings[0].field_values_hash
    @categories = ContentCategory.general.order(:id)
    @content_downloads = ContentDownload.js_menu_items
    render :settings
  end

  def update_settings
    @content_downloads = ContentDownload.js_menu_items
    @categories = ContentCategory.general.order(:id)
    @errors = []
    @blog_settings = Objekt.blog_settings.first
    template_fields = @blog_settings.template_fields
    @blog_settings_hash = @blog_settings.field_values_hash

    begin
      ActiveRecord::Base.transaction do
        # Update 'Blog Settings' Objekt's objekt_fields:
        @blog_settings.objekt_fields.each_with_index do |objekt_field, i|
          template_field_name = template_fields[i].name
          value = params[template_field_name.split(' ')[0].downcase][template_field_name]
          if !objekt_field.update(value: value)
            @errors += objekt_field.errors.full_messages.map { |e| blog_settings_error_messages[e] || "***THIS ERROR WAS NOT IN THE ERRORS HASH: #{e}" }
          end
        end

        # Update 'ContentCategories:
        ContentCategory.general.order(:id).each_with_index do |category, i|
          update_params = category_params("category#{i + 1}")
          if !category.update(update_params)
            @errors += category.errors.full_messages.map { |e| "Category #{i + 1} #{category_error_messages[e] || e}" }
          end
        end
        fail if @errors.length > 0
      end
      @blog_settings = Objekt.blog_settings.first
      @blog_settings_hash = @blog_settings.field_values_hash
      render :settings
    rescue
      render json: @errors, status: 422
    end
  end

  private

  def category_params(category_name)
    params[category_name].permit(
      :name, :url, :title_text, :description, :header, :subheader,
      :content_download_1_id, :content_download_1_event_name, :content_download_1_workflow_id,
      :content_download_2_id, :content_download_2_event_name, :content_download_2_workflow_id,
    )
  end

  def category_error_messages
    {
      "Content download 1 event name can't be blank" => "Event name 1 can't be blank",
      "Content download 1 workflow can't be blank" => "Workflow ID 1 can't be blank",
      "Content download 1 workflow is not a number" => "Workflow ID 1 is not a number",
      "Content download 2 event name can't be blank" => "Event name 2 can't be blank",
      "Content download 2 workflow can't be blank" => "Workflow ID 2 can't be blank",
      "Content download 2 workflow is not a number" => "Workflow ID 2 is not a number",
    }
  end

  def blog_settings_error_messages
    {
      # HOMEPAGE -----
      'Homepage - header is mandatory' => 'Homepage Header is mandatory',
      'Homepage - subheader is mandatory' => 'Homepage Subheader is mandatory',

      'Homepage - content download 1 event name is mandatory' => 'Homepage Event 1 is mandatory',
      ' Homepage - Content Download 1 Workflow ID is mandatory' => 'Homepage Workflow ID 1 is mandatory',
      ' Homepage - Content Download 1 Workflow ID is not a number' => 'Homepage Workflow ID 1 is not a number',

      'Homepage - content download 2 event name is mandatory' => 'Homepage Event 2 is mandatory',
      ' Homepage - Content Download 2 Workflow ID is mandatory' => 'Homepage Workflow ID 2 is mandatory',
      ' Homepage - Content Download 2 Workflow ID is not a number' => 'Homepage Workflow ID 2 is not a number',

      # RESOURCES -----
      'Resources - header is mandatory' => 'Resources Header is mandatory',
      'Resources - subheader is mandatory' => 'Resources Subheader is mandatory',

      'Resources - featured content download event name is mandatory' => 'Resources Featured CU Event Name is mandatory',
      'Resources - featured content download workflow id is mandatory' => 'Resources Featured CU Workflow ID is mandatory',
      ' Resources - Featured Content Download Workflow ID is not a number' => 'Resources Featured CU Workflow ID is not a number',
    }
  end

end
