class Admin::Api::CheckDeleteController < AdminController

  def content_downloads
    id = params[:id]

    content_download_partial_field_ids = PartialField.where(field_type: "content_download").map(&:id)
    page_partial_fields = PagePartialField.where(partial_field_id: content_download_partial_field_ids, value: id).includes(page_partial: [:page])
    page_partial_issues = page_partial_fields.map { |ppf| "Page: #{ppf.page_partial.page.name}" }.uniq

    content_download_template_field_ids = Field.where(field_type: "content_download").map(&:id)
    objekt_fields = ObjektField.where(field_id: content_download_template_field_ids, value: id).includes(:objekt)
    objekt_issues = objekt_fields.map { |of| "Object: #{of.objekt.template.name}" }.uniq

    categories = ContentCategory.where("(content_download_1_id = #{id}) OR (content_download_2_id) = #{id}")
    category_issues = categories.map { |c| "Category: #{c.name}" }.uniq

    @issues = page_partial_issues + objekt_issues + category_issues
    render 'index.json.jbuilder'
  end

  def objekts

    # is the object referenced in a page partial?
    object_partial_field_ids = PartialField.where(field_type: "object").map(&:id)
    page_partial_fields = PagePartialField.where(partial_field_id: object_partial_field_ids, value: params[:id]).includes(page_partial: [:page])
    @issues = page_partial_fields.map { |ppf| "Page: #{ppf.page_partial.page.name}" }.uniq

    # is the object referenced in another object?
    object_template_field_ids = Field.where(field_type: "object").map(&:id)
    object_fields = ObjektField.where(field_id: object_template_field_ids, value: params[:id]).includes(:objekt, field: [template: [:fields]])
    @issues += object_fields.map do |of|
      first_field = of.field.template.fields.first
      if first_field.name == "Name"
        first_objekt_field = ObjektField.find_by({ field_id: first_field.id, objekt_id: of.objekt.id })
        "#{of.field.template.name}: #{first_objekt_field.value}"
      else
        "#{of.field.template.name}: ##{of.objekt.id}"
      end
    end.uniq

    # is the object referenced in the blog?
    objekt = Objekt.find(params[:id])
    if objekt.template.name == "Guest Author"
      blog_posts = BlogPost.where(author_type: "Objekt", author_id: objekt.id)
      @issues += blog_posts.map { |blog_post| "Blog Post: #{blog_post.title}" }
    end

    # is the object referenced in a webinar?
    webinars = Webinar.where("basic_replay_bundle = ? OR basic_play_bundle = ? OR pro_replay_bundle = ? OR pro_play_bundle = ?", objekt.id, objekt.id, objekt.id, objekt.id)
    @issues += webinars.map { |webinar| "Webinar: #{webinar.name}" }
    webinar_objects = WebinarObject.where(objekt_id: objekt.id)
    @issues += webinar_objects.map { |webinar_object| "Webinar: #{webinar_object.webinar.name}" }
    render 'index.json.jbuilder'
  end

  def partials
    page_partials = PagePartial.where(partial_id: params[:id]).includes(:page)
    @issues = page_partials.map do |pp|
      "Page: #{pp.page.name}"
    end.uniq
    render 'index.json.jbuilder'
  end

  def templates
    template = Template.find(params[:id])
    first_field = template.fields.first
    objekts = Objekt.where(template_id: template.id)
    @issues = objekts.map do |objekt|
      if first_field.name == "Name"
        first_objekt_field = ObjektField.find_by({ field_id: first_field.id, objekt_id: objekt.id })
        "Object: #{first_objekt_field.value}"
      else
        "Object: ##{objekt.id}"
      end
    end.uniq
    render 'index.json.jbuilder'
  end

  def tracking_links
    tracking_link = TrackingLink.find(params[:id])
    if TrackingLinkRedirectEvent.where(tracking_link_id: tracking_link.id).length > 0
      render json: { message: { header: "This tracking link cannot be deleted.", subheader: "There are tracking events associated with this tracking link. Please speak with a developer if you really need to delete this." } }
    else
      @issues = []
      render 'index.json.jbuilder'
    end
  end

  def webinars
    webinar_partial_field_ids = PartialField.where(field_type: "webinar").map(&:id)
    page_partial_fields = PagePartialField.where(partial_field_id: webinar_partial_field_ids, value: params[:id]).includes(page_partial: [:page])
    @issues = page_partial_fields.map { |ppf| "Page: #{ppf.page_partial.page.name}" }.uniq
    render 'index.json.jbuilder'
  end

end
