class Admin::Api::ContentDownloadsController < AdminController
  expire_page :@content_download, only: [:update, :destroy]

  def initialize
    super
    @content_types = ContentDownload.content_types.map { |text, i| { displayText: text.titlecase, value: text } }
    @funnel_stages = ContentFunnelStage.js_menu_items
    @content_categories = ContentCategory.js_menu_items
  end

  def index
    @content_downloads = ContentDownload.with_content_info.all
    render :index
  end

  def new
    @content_funnel_stages = ContentFunnelStage.all
    @content_categories = ContentCategory.all
    render "new.json.jbuilder"
  end

  def create
    @content_download = ContentDownload.new(content_download_params)
    if @content_download.save
      render :show
    else
      render json: @content_download.errors.full_messages, status: 422
    end
  end

  def show
    @content_download = ContentDownload.with_all_data.find(params[:id])
    render :show
  end

  def update
    @content_download = ContentDownload.find(params[:id])
    if @content_download.update(content_download_params)
      render :show
    else
      render json: @content_download.errors.full_messages, status: 422
    end
  end

  def destroy
    @content_download = ContentDownload.find(params[:id])
    if @content_download.destroy
      render json: @content_download
    else
      render json: @content_download.errors.full_messages, status: 422
    end
  end

  private

  def content_download_params
    params.require(:content_download).permit(
      :name,
      :content_type,
      :content_category_id,
      :content_score,
      :content_funnel_stage_id,
      :preview_card_image_id,
      :hero_form_background_image_id,
      :hero_form_header,
      :hero_form_button_text,
      :active,
      :upload_id,
      :external_link,
      :landing_page_slug,
      :description,
      :title_text,
      :landing_page_background_image_id,
      :landing_page_large_image_id,
      :landing_page_header,
      :landing_page_button_text,
      :landing_page_bullet_header,
      :landing_page_bullet_points,
      :landing_page_event_name,
      :rich_text_description,
      :custom_redirect
    )
  end
end
