class Admin::Api::EventSeriesController < AdminController

  before_action :verify_admin_user, only: :destroy

  def index
    @event_series = EventSeries.all
    render "index.json.jbuilder"
  end

  def show
    @event_series = EventSeries.find(params[:id])
    fetch_data_for_show_template(params[:id])
    render "show.json.jbuilder"
  end

  def create
    @event_series = EventSeries.new(event_series_params)
    if @event_series.save
      @event_series = EventSeries.all
      render "index.json.jbuilder"
    else
      render json: @event_series.errors.full_messages, status: 422
    end
  end

  def update
    @event_series = EventSeries.find(params[:id])
    if @event_series.update(event_series_params)
      fetch_data_for_show_template(params[:id])
      render "show.json.jbuilder"
    else
      render json: @event_series.errors.full_messages, status: 422
    end
  end

  def destroy
    @event_series = EventSeries.find(params[:id])
    @event_series.destroy
    render json: @event_series
  end

  def add_webinar
    @event_series = EventSeries.find(params[:id])
    @webinar = Webinar.find(params[:webinar_id])
    @event_series.webinars << @webinar
    fetch_data_for_webinars_template
    render "webinars.json.jbuilder"
  end

  def remove_webinar
    @event_series = EventSeries.find(params[:id])
    @webinar = Webinar.find(params[:webinar_id])
    @event_series.webinars.delete(@webinar)
    fetch_data_for_webinars_template
    render "webinars.json.jbuilder"
  end

  private

  def fetch_data_for_show_template(id)
    @webinars = @event_series.webinars
    @other_webinars = Webinar.where.not(recurring: true, id: @webinars.pluck(:id))
  end

  def fetch_data_for_webinars_template
    @webinars = @event_series.webinars
    @other_webinars = Webinar.where.not(recurring: true, id: @webinars.pluck(:id))
  end

  def event_series_params
    params.require(:event_series).permit(:name, :description)
  end

end
