class Admin::Api::ObjektsController < AdminController

  expire_page :@objekt, only: [:update]

  def create
    errors = []
    error_present = false
    begin
      ActiveRecord::Base.transaction do
        @objekt = Objekt.new(objekt_params)
        @objekt.save!
        fields = params[:fields]
        fields.each do |key, value|
          objekt_field = ObjektField.new(objekt_id: @objekt.id, field_id: key, value: value.gsub("\n", ""))
          if !objekt_field.save
            errors += objekt_field.errors.full_messages
            error_present = true
          end
        end
        fail if error_present
      end
      @template = Template.find(objekt_params[:template_id])
      send_down_data_for_objects_index(@objekt.template_id)
    rescue
      render json: errors, status: 422
    end
  end

  def all
    @template = Template.find(params[:id])
    send_down_data_for_objects_index(params[:id])
  end

  def show
    objekt = Objekt.find(params[:id])
    @object = objekt.object
    @template = objekt.template
    @fields = Field.where(template_id: @template.id).includes(:menu_options).order(:order)
    @images = {}
    @files = {}
    @objekts = {} # <-- cache queries of object instances, in case there are multiple object fields of the same object type
    @instances = {} # <-- menu options for object fields
    @fields.each do |field|
      if field.field_type == "image"
        image_id = ObjektField.where(field_id: field.id, objekt_id: objekt.id)[0].value
        image = (image_id == '' ? nil : Upload.find(image_id.to_i))
        @images[field.id] = image ? image.url_and_reusability : { url: '', reusability: false }
      elsif field.field_type == "file"
        file_id = ObjektField.where(field_id: field.id, objekt_id: objekt.id)[0].value
        file = file_id == '' ? nil : Upload.find(file_id.to_i)
        @files[field.id] = file ? { url: file.url } : { url: '' }
      elsif field.field_type == "object"
        @templates ||= Hash[ Template.all.includes(:fields).map { |template| [template.name, { id: template.id, first_field_id: template.fields.first.id }] } ]
        template_id = @templates[field.object_type][:id]
        unless @objekts[template_id]
          @objekts[template_id] = get_object_field_options(field, template_id)
        end
        @instances[field.id] = @objekts[template_id]
      end
    end
    render "show.json.jbuilder"
  end

  def update
    errors = []
    error_present = false
    begin
      ActiveRecord::Base.transaction do
        @objekt = Objekt.find(params[:id])
        @objekt_fields = @objekt.fields
        @objekt_fields.each do |objekt_field|
          if !objekt_field.update(value: params[:object][objekt_field.field.id.to_s].gsub("\n", ''))
            errors += objekt_field.errors.full_messages
            error_present = true
          end
        end
        fail if error_present
      end
      # get everying to send down to object details component: ---
      @object = @objekt.object
      @fields = Field.where(template_id: @objekt.template.id)
      @images = {}
      @files = {}
      @fields.each do |field|
        if field.field_type == "image"
          image_id = ObjektField.where(field_id: field.id, objekt_id: @objekt.id)[0].value
          image = image_id == '' ? nil : Upload.find(image_id.to_i)
          @images[field.id] = image ? image.url_and_reusability : {url: '', reusability: false}
        elsif field.field_type == "file"
          file_id = ObjektField.where(field_id: field.id, objekt_id: @objekt.id)[0].value
          file = file_id == '' ? nil : Upload.find(file_id.to_i)
          @files[field.id] = file ? { url: file.url } : { url: '' }
        end
      end
      # ----------------------------------------------------------
      render "update.json.jbuilder"
    rescue
      render json: errors, status: 422
    end
  end

  def destroy
    @objekt = Objekt.find(params[:id])
    @objekt.destroy
    render json: @objekt
  end

  private

  def objekt_params
    params.require(:objekt).permit(:template_id)
  end

  def send_down_data_for_objects_index(template_id)
    @result = []
    @fields = Field.where(template_id: template_id).order(:order).includes(:objekt_fields)
    # using .includes(objekt_fields: [:images]) causes an error within rails, so we do it manually: ---
      objekt_fields = @fields.inject([]) { |fields, field| fields += field.objekt_fields }
      images = Upload.where(id: objekt_fields.map(&:value))
    # -------------------------------------------------------------------------------------------------
    objekts = Objekt.where(template_id: template_id)
    objekts.each do |objekt, index|
      fields = []
      @fields.each do |field|
        objekt_field = field.objekt_fields.select { |objekt_field| objekt_field.objekt_id == objekt.id }[0]
        if field.field_type == 'image'
          fields << (objekt_field.value == '' ? '' : images.select { |image| image.id == objekt_field.value.to_i }[0].url)
        else
          fields << objekt_field.value
        end
      end
      @result << { 'id' => objekt.id, 'fields' => fields }
    end
    render 'all.json.jbuilder'
  end

  def get_object_field_options(field, template_id)
    # we need to create an array of menu options for an object field
    # each option needs the objekt id, and the value of the objekt_field matching the template's first field
    # example: [{ id: 12, first_field_value: "The Profitable Teacher" }]
    objekts = Objekt.where(template_id: template_id).includes(:objekt_fields)
    first_template_field_id = @templates[field.object_type][:first_field_id]
    result = objekts.map { |objekt| { id: objekt.id, first_field_value: objekt.objekt_fields.select { |objekt_field| objekt_field.field_id == first_template_field_id }.first.value } }
    result.sort_by { |element| element[:first_field_value].downcase }
  end

end
