class Admin::Api::PagePartialsController < AdminController

  include Reorderable

  expire_page :@page_partial, only: [:create, :update, :destroy, :rearrange, :destroy_nested]
  expire_page :@parent_page_partial_record, only: [:create_nested]

  def show
    @page_partial = PagePartial.find(params[:id])
    @partial_fields = PartialField.where(partial_id: @page_partial.partial_id).includes(:menu_options).order(:order)
    @page_partial_object = @page_partial.object_from_partial_fields(@partial_fields)
    @images       = {}
    @files        = {}
    @instances    = {}
    @webinars     = []
    @partial_fields.each do |partial_field|
      if partial_field.field_type == "image"
        image_id = PagePartialField.where(partial_field_id: partial_field.id, page_partial_id: @page_partial.id)[0].value
        image = image_id == '' ? nil : Upload.find(image_id.to_i)
        @images[partial_field.id] = image ? image.url_and_reusability : { url: '', reusability: false }
      elsif partial_field.field_type == "file"
        file_id = PagePartialField.where(partial_field_id: partial_field.id, page_partial_id: @page_partial.id)[0].value
        file = file_id == '' ? nil : Upload.find(file_id.to_i)
        @files[partial_field.id] = file ? { url: file.url } : { url: '' }
      elsif partial_field.field_type == "object"
        template_id = Template.find_by_name(partial_field.object_type).id
        @instances[partial_field.id] = Objekt.where(template_id: template_id).map { |objekt| objekt.id_and_first_field }
      elsif partial_field.field_type == "webinar" && @webinars.empty?
        @webinars = Webinar.all
      end
    end
    @header_types = HeaderType.all
    @nested_page_partials = format_nested_page_partials(page_partial_field_ids: @page_partial.page_partial_field_ids)
    render "show.json.jbuilder"
  end

  def create
    # create new page partial:
    page_partials_length = PagePartial.where(page_id: page_partial_params[:page_id]).length
    @page_partial = PagePartial.new(page_id: page_partial_params[:page_id], partial_id: page_partial_params[:partial_id], order: page_partials_length)
    @page_partial.save!
    PagePartialField.create_page_partial_fields!(@page_partial)
    # query data for response:
    @page_partials = PagePartial.where(page_id: @page_partial.page_id).includes(:partial)
    field_counts = Partial.all.select('partials.*, COUNT(*) AS fields_count').joins(:partial_fields).group('partials.id')
    @field_counts = Hash[field_counts.map { |partial| [partial.id, partial.fields_count] }]
    render "index.json.jbuilder"
  end

  def create_nested
    new_page_partial = PagePartial.new(
      partial_id: params[:page_partial][:partial_id],
      page_partial_field_id: params[:page_partial][:page_partial_field_id],
      order: 0
    )
    new_page_partial.save!
    PagePartialField.create_page_partial_fields!(new_page_partial)
    # used for expiring cache in expire_page_cache
    @parent_page_partial_record = PagePartial.find(params[:page_partial][:parent_id])
    update_parent_page_partial_value(@parent_page_partial_record, new_page_partial.id)
    @parent_page_partial = @parent_page_partial_record.object
    @nested_page_partials = format_nested_page_partials(
      page_partial_field_ids: @parent_page_partial_record.page_partial_field_ids,
      changed_nested_page_partials: { params[:page_partial][:page_partial_field_id] => new_page_partial.object }
    )
    render "create-nested.json.jbuilder"
  end

  def copy
    existing_page_partial = PagePartial.find(params[:page_partial_id])
    destination_page = Page.find(params[:page_id])
    destination_page_partials = destination_page.page_partials
    new_page_partial = PagePartial.create!({
      partial_id: existing_page_partial.partial_id,
      page_id: params[:page_id],
      order: destination_page_partials.length
    })
    existing_page_partial.page_partial_fields.each do |existing_page_partial_field|
      new_page_partial_field = PagePartialField.new(
        page_partial_id: new_page_partial.id,
        partial_field_id: existing_page_partial_field.partial_field.id,
        value: existing_page_partial_field.value
      )
      new_page_partial_field.save!(validate: false)
    end
    render json: new_page_partial
  end

  def update
    @errors = []
    error_present = false
    begin
      @page_partial = PagePartial.find(params[:id])
      @page_partial_fields = @page_partial.page_partial_fields
      ActiveRecord::Base.transaction do
        @page_partial_fields.each do |page_partial_field|
          # do i have to load each of these partial fields individually?
          update_hash = { value: params[:page_partial][page_partial_field.partial_field.id.to_s] }
          update_hash[:header_type] = params[:page_partial][page_partial_field.partial_field.id.to_s + "_style"] if page_partial_field.partial_field.field_type == "header"
          if !page_partial_field.update(update_hash)
            @errors += page_partial_field.errors.full_messages
            error_present = true
          end
        end
        fail if error_present
      end
      @partial_fields = PartialField.where(partial_id: @page_partial.partial_id).includes(:menu_options).order(:order)
      @page_partial_object = @page_partial.object_from_partial_fields(@partial_fields, @page_partial_fields)
      @nested_page_partials = format_nested_page_partials(page_partial_field_ids: @page_partial.page_partial_field_ids)
      render "update.json.jbuilder"
    rescue
      render "partial_errors.json.jbuilder", status: 422
    end
  end

  def rearrange
    params[:new_order].each do |index, id|
      page_partial = PagePartial.find(id)
      page_partial.update(order: index)
    end
    # query data for response:
    @page_partials = PagePartial.where(page_id: params[:page_id]).includes(:partial)
    # used for expiring cache in expire_page_cache
    @page_partial = @page_partials[0]
    field_counts = Partial.all.select('partials.*, COUNT(*) AS fields_count').joins(:partial_fields).group('partials.id')
    @field_counts = Hash[field_counts.map { |partial| [partial.id, partial.fields_count] }]
    render "index.json.jbuilder"
  end

  def destroy
    # used for expiring cache in expire_page_cache
    @page_partial = PagePartial.find(params[:id])
    @page_partial.destroy
    @page_partials = reorder(PagePartial.where(page_id: @page_partial.page_id).order(:order).includes(:partial))
    # query additional data for response:
    field_counts = Partial.all.select('partials.*, COUNT(*) AS fields_count').joins(:partial_fields).group('partials.id')
    @field_counts = Hash[field_counts.map { |partial| [partial.id, partial.fields_count] }]
    render "index.json.jbuilder"
  end

  def destroy_nested
    # used for expiring cache in expire_page_cache
    @page_partial = PagePartial.where(page_partial_field_id: params[:page_partial_field_id])[0]
    @page_partial.destroy
    parent_page_partial = PagePartial.find(params[:parent_id])
    update_parent_page_partial_value(parent_page_partial, '')
    @parent_page_partial = parent_page_partial.object
    @nested_page_partials = format_nested_page_partials(
      page_partial_field_ids: parent_page_partial.page_partial_field_ids,
      changed_nested_page_partials: { params[:page_partial_field_id] => nil }
    )
    render 'destroy-nested.json.jbuilder'
  end

  private

  def page_partial_params
    params[:page_partial].permit(:partial_id, :page_id)
  end

  def update_parent_page_partial_value(parent_page_partial, value)
    page_partial_field_id = params[:page_partial_field_id] || params[:page_partial][:page_partial_field_id]
    field = parent_page_partial.page_partial_fields.where(id: page_partial_field_id)[0]
    parent_page_partial
      .page_partial_fields
      .where(id: page_partial_field_id)[0]
      .update!(value: value)
  end

  def format_nested_page_partials(options)
    PagePartial
      .where(page_partial_field_id: options[:page_partial_field_ids])
      .map { |page_partial| [page_partial.page_partial_field_id, page_partial.object.deep_transform_keys { |key| key.to_s.camelize(:lower) }] }
      .to_h
      .merge(options[:changed_nested_page_partials].try(:deep_transform_keys) { |key| key.to_s.camelize(:lower) } || {})
  end

end
