class Admin::Api::PagesController < AdminController

  before_action :verify_admin_user, only: :destroy
  expire_page :@page, only: [:update, :destroy]

  def index
    @pages = Page.all.includes(:creator)
    render 'index.json.jbuilder'
  end

  def create
    @page = Page.new(page_params.merge({ canonical_url: "https://teachable.com#{page_params[:url]}" }))
    if params[:page][:clone_id]
      if @page.valid?
        begin
          clone_page(Page.find(params[:page][:clone_id]), @page)
          render 'create.json.jbuilder'
        rescue
          render json: ['failed to copy page'], status: 500
        end
      else
        render json: @page.errors.full_messages, status: 422
      end
    else
      if @page.save
        render 'create.json.jbuilder'
      else
        render json: @page.errors.full_messages, status: 422
      end
    end
  end

  def show
    @page = Page.where(id: params[:id]).includes(:page_partials).first
    @partials = Partial.all.order(:name).includes(:image)
    field_counts = Partial.all.select('partials.*, COUNT(*) AS fields_count').joins(:partial_fields).group('partials.id')
    @field_counts = Hash[field_counts.map { |partial| [partial.id, partial.fields_count] }]
    @header_types = HeaderType.all
    render 'show.json.jbuilder'
  end

  def update
    @page = Page.find(params[:id])
    if @page.update(page_params)
      render 'update.json.jbuilder'
    else
      render json: @page.errors.full_messages, status: 422
    end
  end

  def destroy
    @page = Page.find(params[:id])
    @page.destroy
    render json: @page
  end

  private

  def page_params
    params[:page].permit(:name, :url, :title_text, :description, :creator_id, :active, :index_page, :canonical_url, :notes, :display_exit_intent_modal, :google_optimize, :og_image_id, :live_chat, :theme)
  end

  def clone_page(old_page, new_page)
    ActiveRecord::Base.transaction do
      new_page.update!(
        description: old_page.description,
        index_page: old_page.index_page,
        display_exit_intent_modal: old_page.display_exit_intent_modal,
        google_optimize: old_page.google_optimize
      )
      old_page.page_partials.each do |old_page_partial|
        new_page_partial = old_page_partial.dup
        new_page_partial.page = new_page
        new_page_partial.partial = old_page_partial.partial
        new_page_partial.save!
        old_page_partial.page_partial_fields.each do |old_page_partial_field|
          new_page_partial_field = old_page_partial_field.dup
          new_page_partial_field.page_partial = new_page_partial
          new_page_partial_field.save!
        end
      end
    end
  end

end
