class Admin::Api::PlanFeaturesController < AdminController

  include Reorderable

  expire_page :@plan_feature, only: [:create, :update, :rearrange, :destroy]

  def index
    @plan_features = PlanFeature.where(plan_id: params[:plan_id])
    render "index.json.jbuilder"
  end

  def create
    plan_features_length = PlanFeature.where(plan_id: plan_feature_params[:plan_id]).length
    @plan_feature = PlanFeature.new(plan_id: plan_feature_params[:plan_id], feature_id: plan_feature_params[:feature_id], order: plan_features_length)
    @plan_feature.save!
    @plan_features = PlanFeature.where(plan_id: plan_feature_params[:plan_id])
    render "index.json.jbuilder"
  end

  def update
    @plan_feature = PlanFeature.find(params[:id])
    tooltip_id = plan_feature_params[:tooltip_id] == "none" ? nil : plan_feature_params[:tooltip_id]
    @plan_feature.update!(tooltip_id: tooltip_id)
    @plan_features = PlanFeature.where(plan_id: params[:plan_id])
    render "index.json.jbuilder"
  end

  def rearrange
    params[:new_order].each do |index, id|
      plan_feature = PlanFeature.find(id)
      plan_feature.update(order: index)
    end
    @plan_features = PlanFeature.where(plan_id: params[:plan_id])
    # used for expiring cache in expire_page_cache
    @plan_feature = @plan_features[0]
    render "index.json.jbuilder"
  end

  def destroy
    # used for expiring cache in expire_page_cache
    @plan_feature = PlanFeature.find(params[:id])
    @plan_feature.destroy
    @plan_features = reorder(PlanFeature.where(plan_id: @plan_feature.plan_id).order(:order))
    render "index.json.jbuilder"
  end

  private

  def plan_feature_params
    params[:plan_feature].permit(:tooltip_id, :plan_id, :feature_id)
  end

end
