class Admin::Api::ScheduledTasksController < AdminController

  def index
    @scheduled_tasks = ScheduledTask.all.includes(:schedulable)
    render "index.json.jbuilder"
  end

  def new
    @pages = Page.all
    @redirects = Redirect.all
    @blog_posts = BlogPost.all
    @banners = Banner.all
    render "new.json.jbuilder"
  end

  def create
    @scheduled_task = ScheduledTask.new(scheduled_task_params.merge({ status: 1 }))
    if @scheduled_task.save
      @scheduled_tasks = ScheduledTask.all
      render "index.json.jbuilder"
    else
      render json: @scheduled_task.errors.full_messages, status: 422
    end
  end

  def show
    @scheduled_task, @pages, @redirects, @blog_posts, @banners = fetch_data_for_show_view(params[:id])
    render "show.json.jbuilder"
  end

  def update
    @scheduled_task = ScheduledTask.find(params[:id])
    if @scheduled_task.update(scheduled_task_params)
      @scheduled_task, @pages, @redirects, @blog_posts, @banners = fetch_data_for_show_view(params[:id])
      render "show.json.jbuilder"
    else
      render json: @scheduled_task.errors.full_messages, status: 422
    end
  end

  def destroy
    @scheduled_task = ScheduledTask.find(params[:id])
    @scheduled_task.destroy
    render json: @scheduled_task
  end

  private

  def fetch_data_for_show_view(id)
    [
      ScheduledTask.find(id),
      Page.all,
      Redirect.all,
      BlogPost.all,
      Banner.all
    ]
  end

  def scheduled_task_params
    params[:scheduled_task].permit(:time, :action, :schedulable_id, :schedulable_type)
  end

end
