class Admin::Api::TemplatesController < AdminController

  include Fieldable

  expire_page :@belongs_to_instance, only: [:update]

  def index
    @templates = Template.by_permission(@current_user.permission_id).order('lower(name)').includes(:image)
    render 'index.json.jbuilder'
  end

  def create
    @template = Template.new(template_params)
    if @template.save
      render 'create.json.jbuilder'
    else
      render json: @template.errors.full_messages, status: 422
    end
  end

  def show
    @template = Template.find(params[:id])
    @templates = Template.by_permission(@current_user.permission_id).includes(:image)
    @fields = Field.where(template_id: params[:id]).order(:order)
    @instances = {}
    @fields.each do |field|
      if field.field_type == 'object'
        template_id = Template.find_by_name(field.object_type).id
        @instances[field.id] = Objekt.where(template_id: template_id).map { |objekt| objekt.id_and_first_field }
      end
    end
    render 'show.json.jbuilder'
  end

  def update
    args = {
      field_model: 'field',
      belongs_to_model: 'template',
      belongs_to_params: template_params,
      instance_model: 'objekt',
      instance_field_model: 'objekt_field'
    }
    fieldable_update(args)
  end

  def destroy
    @template = Template.find(params[:id])
    if @template.destroy
      render json: @template, status: 200
    else
      render json: @template.errors.full_messages, status: 422
    end
  end

  private

  def template_params
    params.require(:template).permit(:id, :name, :plural, :image_id, :editable)
  end

end
