class Admin::Api::UsersController < AdminController

  expire_page :@user, only: [:api_update]

  def index
    @users = User.all.includes(:permission, :profile_pic, :team)
    render "index.json.jbuilder"
  end

  def new
    @teams = Team.all
    @permissions = Permission.proper_permissions_for_access(@current_user)
    render "new.json.jbuilder"
  end

  def create
    @user = User.new(user_params)
    @user.password = user_params[:password]
    if @user.save
      @users = User.all.includes(:permission, :profile_pic, :team)
      render "index.json.jbuilder"
    else
      render json: @user.errors.full_messages, status: 422
    end
  end

  def show
    @user = User.find(params[:id])
    @teams = Team.all
    @permissions = Permission.proper_permissions_for_access(@current_user)
    render "show.json.jbuilder"
  end

  def update
    @user = User.find(params[:id])
    user_params_hash = user_params.to_h
    user_params_hash.delete("email")
    if @user.update(user_params_hash)
      @teams = Team.all
      @permissions = Permission.proper_permissions_for_access(@current_user)
      render "show.json.jbuilder"
    else
      render json: @user.errors.full_messages, status: 422
    end
  end

  def update_password
    AAA.client.update_password(password: params[:password], account_id: @aaa_account_id)
    @user = User.find(params[:id])
    @teams = Team.all
    @permissions = Permission.proper_permissions_for_access(@current_user)
    render "show.json.jbuilder"
  end

  def destroy
    @user = User.find(params[:id])
    if @user.destroy
      @user.pages.update_all(deleted_creator_name: @user.name)
      render json: @user, status: 200
    else
      render json: @user.errors.full_messages, status: 422
    end
  end

  def user_privileges
    @user = User.find(params[:id])
    render "user-privileges.json.jbuilder"
  end

  private

  def user_params
    params[:user].permit(:name, :email, :title, :team_id, :permission_id, :image_id, :bio, :twitter_handle, :password)
  end

end
