# frozen_string_literal: true

class AdminController < ApplicationController

  def self.expire_page(instance_variable_name, *options)
    after_action(*options) do |_controller|
      expire_cache(instance_variable_name)
    end
  end

  before_action :require_aaa_login
  before_action :permissions_redirect
  before_action :show_admin_menu

  def root
    render "root.html.erb"
  end

  private

  def require_aaa_login
    get_signed_in_user
    unless @current_user
      redirect_to aaa.start_login_flow_redirect_url(success_url: admin_url)
    end
  end

  def permissions_redirect
    if @current_user.permission.level == 0 && request.env["PATH_INFO"] == "/admin"
      redirect_to "/admin/webinars"
    end
  end

  def show_admin_menu
    @admin_page = !request.original_url.include?("preview")
  end

  def expire_cache(instance_variable_name)
    if response.status.between?(200, 299)
      entity = instance_variable_get(instance_variable_name)
      entity.cached_urls.each do |url|
        Rails.cache.delete("views/#{request.host_with_port}#{url}") unless ENV["RAILS_ENV"] == "development"
      end
    end
  end

  def verify_admin_user
    render json: { 'message': "you are not authorized to do this" }, status: 401 if @current_user.permission.level < 2
  end

end
