class Api::IterableController < ApplicationController

  protect_from_forgery with: :null_session, prepend: true

  def send_event
    data = params[:data]
    data = data.is_a?(String) ? JSON.parse(data) : data
    SendIterableEvent.perform_async(params[:instance_id], params[:email], params[:event_name], data, params[:i_template_id], params[:i_campaign_id])
    render json: { message: "done" }, status: 200
  end

  def subscription_preferences
    response = Iterable.get_user_by_email(params["instance_id"], user_email)
    if response.code == 200 && !response.empty?
      unsubscribed_message_type_ids = response.dig("user", "dataFields", "unsubscribedMessageTypeIds") || []
      unsubscribed_channel_ids      = response.dig("user", "dataFields", "unsubscribedChannelIds")     || []
      render json: {
        unsubscribedMessageTypeIds: unsubscribed_message_type_ids,
        unsubscribedChannelIds: unsubscribed_channel_ids,
      }, status: 200
    else
      render json: {}, status: 400
    end
  end

  def update_subscription_preferences
    unsubscribed_message_type_ids = params["unsubscribed_message_type_ids"].to_a.map(&:to_i)
    unsubscribed_channel_ids = params["unsubscribed_channel_ids"].to_a.map(&:to_i)
    SubscriptionPreferencesWorker.perform_async(params["instance_id"], user_email, unsubscribed_message_type_ids, unsubscribed_channel_ids)
    render json: { message: "Job scheduled." }, status: 200
  end

  def register_token
    RegisterTokenWorker.perform_async(params["instance_id"], params["token"], params["email"])
    render json: { message: "Job scheduled." }, status: 200
  end

  private

  def user_email
    if params[:email_id].present?
      key = ENV.fetch("ITERABLE_HMAC_SECRET", "82f0539915151f1ac5a64c690a23cc8d")
      if OpenSSL::HMAC.hexdigest("SHA1", key, params[:email]) == params[:email_id]
        params[:email]
      else
        raise ArgumentError, "Email ID does not match the hashed email :/"
      end
    else
      raise ArgumentError, "Must provide Email ID that matches the hashed email :/"
    end
  end

end
