# frozen_string_literal: true

class Api::PublicEventSeriesController < ApplicationController

  skip_before_action :verify_authenticity_token, only: [:schedule_full]

  def schedule_partial
    event_series = EventSeries.find(params[:id])
    webinars = event_series.webinars
    result = {}
    webinars.each do |webinar|
      result[webinar.id] = webinar.replay_video_id.present?
    end
    render json: result
  end

  def schedule_full
    if params[:secret] != ENV['EVENT_SERIES_SECRET']
      render json: {
        status: 401,
        message: "Incorrect secret",
      }, status: 401
      return
    end
    event_series = EventSeries.find(params[:id])
    events = event_series.webinars.map do |webinar|
      {
        id: webinar.id,
        baseUrl: "https://teachable.com/webinars/#{webinar.url}",
        title: webinar.title,
        startTime: webinar.proper_time1,
        replayAvailable: webinar.replay_video_id.present?,
        description: webinar.description,
        hosts: webinar.hosts.map do |host|
          host.field_values_hash({ extend_image_urls: true })
        end,
        expectedDuration: webinar.expected_duration,
      }
    end

    render json: { webinars: events }
  end
end
