class Api::RegistrantsController < ApplicationController

  protect_from_forgery with: :null_session, prepend: true

  def create
    create_registrant(registrant_params[:email], params)
  end

  private

  def create_registrant(email, params)
    @registrant = Registrant.find_by(email: email)
    if @registrant
      create_registration(params)
    else
      @registrant = Registrant.new(email: email)
      if @registrant.save
        create_registration(params)
      else
        render json: @registrant.errors.full_messages, status: 422
      end
    end
  end

  def create_registration(params)
    @webinar = Webinar.find(params[:webinar_id].to_i)
    @registrations = Registration.where(registrant_id: @registrant.id, webinar_id: @webinar.id, webinar_start_time: proper_webinar_start_time)
    if @registrations.any?
      render plain: "You have already registered for this webinar.", status: 422
    else
      @registration = Registration.new(
        registrant_id: @registrant.id,
        webinar_id: @webinar.id,
        webinar_start_time: proper_webinar_start_time,
        campaignid: params[:campaignid],
        templateid: params[:templateid],
        utm_campaign: params[:utm_campaign],
        utm_source: params[:utm_source],
        utm_medium: params[:utm_medium],
        utm_term: params[:utm_term],
        utm_content: params[:utm_content],
        mt_tier: params[:mt_tier],
        mt_interval: params[:mt_interval],
        mt_campaign_group: params[:mt_campaign_group],
        mbsy: params[:mbsy]
      )
      if @registration.save
        automated_time = if params[:session_token].present?
                           time_string = @webinar.decrypt_start_time(CGI.unescape(params[:session_token]))
                           DateTime.parse(time_string)
                         else
                           nil
                         end
        SendWebinarRegistrationToIterable.perform_async(@webinar.id, @registrant.id, @registration.id, params[:time2], automated_time)
        render json: { url: @webinar.url }, status: 200
      else
        render text: "There was a problem with your registration. Please let us know.", status: 500
      end
    end
  end

  def registrant_params
    params[:registrant].permit(:email)
  end

  def proper_webinar_start_time
    if @webinar.webinar_type == "live"
      (params[:time2].empty? ? @webinar.proper_time1 : @webinar.time2)
    else
      DateTime.parse(@webinar.decrypt_start_time(CGI.unescape(params[:session_token])))
    end
  end

end
