class BlogController < ApplicationController

  include Blogable

  caches_action :show, :index, :resources

  def initialize
    @blog_page = true
    @js_includes = ["blog-nav", "blog-footer", "blog-signup"]
    super
  end

  def index
    @blog_page_type = 'index'
    @blog_settings = Objekt.blog_settings[0]
    categories = ContentCategory.general.order(:id)
    @nav_items = get_nav_items(categories)
    @posts_by_category = categories.reduce([]) do |acc, category|
      acc << { category: category, posts: get_posts_by_category(category, 3) }
    end
    @js_includes << "content-download-hero-form"
  end

  def show
    @blog_page_type = 'show'
    url = "/#{params[:blog_post_slug]}"
    @blog_post = (
      BlogPost
        .published
        .includes(:cover_photo, :categories, :author)
        .find_by(url: url)
    )
    if @blog_post
      @nav_items = get_nav_items
      @categories = @blog_post.categories
      @related_posts = @categories
        .reduce([]) { |acc, category| acc += get_posts_by_category(category, 4, [@blog_post.id]) }
        .shuffle
        .take(4)
      render 'show.html.erb'
    else
      redirect = Redirect.find_by(url: "/blog#{url}")
      if redirect && redirect.active
        redirect_to redirect.redirect, status: redirect.status_code
      else
        @page = Page.find_by_url('/404_error')
        render './pages/show.html.erb', status: 404
      end
    end
  end

  def category
    category_name = params[:category_name].split('-').map(&:capitalize).join(' ')
    @blog_category = ContentCategory.general.find_by(name: category_name)
    if @blog_category
      @blog_page_type = 'category'
      @nav_items = get_nav_items
      @posts = get_posts_by_category(@blog_category, 10)
      @js_includes << "content-download-hero-form"
    else
      @page = Page.find_by_url('/404_error')
      render './pages/show.html.erb', status: 404
    end
  end

  def resources
    @blog_page_type = 'resources'
    @nav_items = get_nav_items
    @categories = ContentCategory.general.where.not(name: "Teachable Updates").order(:id) # Teachable Updates category is irrelevant to the resources page
    @blog_settings = Objekt.blog_settings[0]
    @js_includes << "content-download-hero-form"
    render :resources
  end

end
