module Blogable
  extend ActiveSupport::Concern

  include Paginateable

  def get_all_authors
    BlogPost.where(author_type: "User").includes(:author) +
      BlogPost.where(author_type: "Objekt").includes(author: [:template_fields, :objekt_fields])
  end

  def get_nav_items(categories = nil)
    categories ||= ContentCategory.general.order(:id) # Allows for pre-fetching if already grabbing categories elsewhere (see #index)
    default_items = [
      { name: 'HOME', url: '/blog' },
    ]
    categories.reduce(default_items) { |acc, category| acc << category.nav_item }
  end

  def get_most_recent_posts(limit)
    posts = BlogPost
      .published
      .includes(:cover_photo, :author)
      .order(date: :desc)

    paginate(posts, limit, 'https://teachable.com/blog/most-recent')
  end

  def get_posts_by_category(category, limit, excluded_ids = [])
    posts = BlogPostCategory
      .includes(blog_post: [:cover_photo, :author])
      .joins(:blog_post)
      .where(blog_category_id: category.id)
      .where(blog_posts: { active: true })
      .where.not(blog_posts: { id: excluded_ids })
      .order('blog_posts.date DESC')
      .map(&:blog_post)

    paginate(posts, limit, 'https://teachable.com' + category.url)
  end
end
