module Fieldable
  extend ActiveSupport::Concern

  def create_empty_error_object(belongs_to_model)
    num_of_fields = params[:fields] ? field_params.permit!.to_h.max_by { |key, value| key.to_i }[0].to_i + 1 : 0
    {
      "#{belongs_to_model}_errors": [],
      fields: Array.new(num_of_fields) {[]}
    }
  end

  def create_field(args, index, belongs_to_id, errors)
    field_model_constant = args[:field_model].classify.constantize
    instance_model_constant = args[:instance_model].classify.constantize
    instance_field_model_constant = args[:instance_field_model].classify.constantize
    @field_model = field_model_constant.new(field_params[index.to_s].permit(:name, :field_type, :char_limit, :object_type, :description))
    @field_model.mandatory = field_params[index.to_s][:mandatory] == "yes" ? true : false
    @field_model.send("#{args[:belongs_to_model]}_id=", belongs_to_id) # attach Field/PartialField to Template/Partial
    @field_model.order = index
    if @field_model.save
      update_menu_options(args, @field_model.id, field_params[index.to_s]["menu_options"])
      @instances = instance_model_constant.where("#{args[:belongs_to_model]}_id": belongs_to_id)
      @instances.each do |instance|
        if @field_model.field_type == "dropdown"
          value = MenuOption.find_by({ optionable_id: @field_model.id, order: 0 }).value
        elsif @field_model.field_type == "object"
          template_id = Template.find_by_name(@field_model.object_type).id
          value = Objekt.where(template_id: template_id).first.id
        elsif @field_model.field_type == "color"
          value = "#FFFFFF"
        elsif @field_model.field_type == "number"
          value = "0"
        elsif @field_model.field_type == "webinar"
          value = Webinar.first.id
        elsif @field_model.field_type == "content_download"
          value = ContentDownload.first.id
        elsif @field_model.field_type == "event_series"
          value = EventSeries.first.id
        else
          value = ""
        end
        @instance_field = instance_field_model_constant.new("#{args[:field_model]}_id": @field_model.id, "#{args[:instance_model]}_id": instance.id, value: value)
        @instance_field.save!(validate: false) # validate is set to false to allow empty string values, in case the fields have been set to mandatory in the object template
      end
    else
      errors[:fields][index] += @field_model.errors.full_messages
    end
  end

  def delete_other_fields(field_model_constant, field_ids_to_remove)
    field_ids_to_remove.each do |id|
      @field_model = field_model_constant.find(id)
      @field_model.destroy
    end
  end

  def error_present(errors, belongs_to_model)
    return true if !errors["#{belongs_to_model}_errors".to_sym].empty?
    errors[:fields].each do |field|
      return true if !field.empty?
    end
    false
  end

  def fieldable_update(args)
    field_model_constant = args[:field_model].classify.constantize # PartialField or TemplateField
    belongs_to_model_constant = args[:belongs_to_model].classify.constantize # Partial or Template
    errors = create_empty_error_object(args[:belongs_to_model])
    begin
      ActiveRecord::Base.transaction do
        @belongs_to_instance = belongs_to_model_constant.find(args[:belongs_to_params][:id])
        if !@belongs_to_instance.update(args[:belongs_to_params])
          errors["#{args[:belongs_to_model]}_errors".to_sym] += @belongs_to_instance.errors.full_messages
        end
        field_ids = get_field_ids_from_params
        field_ids_to_remove = get_field_ids_to_remove(args, field_ids)
        field_ids.each_with_index do |id, index|
          if id == ""
            create_field(args, index, @belongs_to_instance.id, errors)
          else
            update_field(args, id, index, args[:belongs_to_model], @belongs_to_instance.id, errors)
          end
        end
        delete_other_fields(field_model_constant, field_ids_to_remove)
        fail if error_present(errors, args[:belongs_to_model])
        @fields = field_model_constant.where("#{args[:belongs_to_model]}_id": @belongs_to_instance.id).order(:order)
        eval "@#{args[:belongs_to_model]} = @belongs_to_instance"
        render "show.json.jbuilder"
      end
    rescue
      render json: errors, status: 422
    end
  end

  def field_params
    params.require(:fields)
  end

  def get_field_ids_from_params
    if !params[:fields]
      []
    else
      result = []
      params[:fields].each do |id, field|
        result << (field["id"] == "" ? "" : field["id"].to_i)
      end
      result
    end
  end

  def get_field_ids_to_remove(args, field_ids)
    field_model_constant = args[:field_model].classify.constantize
    all_field_ids = field_model_constant.where("#{args[:belongs_to_model]}_id": args[:belongs_to_params][:id]).pluck(:id)
    all_field_ids - field_ids
  end

  def update_field(args, field_id, index, belongs_to_model, belongs_to_id, errors)
    model_constant = args[:field_model].classify.constantize
    @model = model_constant.find(field_id)
    if @model.update(
      name: params.require(:fields)[index.to_s]["name"],
      field_type: params.require(:fields)[index.to_s]["field_type"],
      char_limit: params.require(:fields)[index.to_s]["char_limit"],
      object_type: params.require(:fields)[index.to_s]["object_type"],
      description: params.require(:fields)[index.to_s]["description"],
      mandatory: params.require(:fields)[index.to_s]["mandatory"] == "yes" ? true : false,
      "#{belongs_to_model}_id": belongs_to_id,
      order: index
    )
      update_menu_options(args, field_id, params[:fields][index.to_s]["menu_options"])
    else
      errors[:fields][index] += @model.errors.full_messages
    end
  end

  def update_menu_options(args, field_id, menu_options)
    MenuOption.where(optionable_id: field_id, optionable_type: args[:field_model].classify).destroy_all
    if menu_options
      menu_options.each do |key, menu_option|
        MenuOption.create(optionable_id: field_id, optionable_type: args[:field_model].classify, order: menu_option["order"], value: menu_option["value"])
      end
    end
  end

end
