module Paginateable
  extend ActiveSupport::Concern

  def paginate(elements, limit, canonical_url)
    paginated_elements = elements.paginate(page: params[:page], per_page: limit)
    create_pagination_metadata!(elements, paginated_elements, limit, canonical_url)
    paginated_elements
  end

  def create_pagination_metadata!(elements, paginated_elements, elements_per_page, canonical_url)
    return "".html_safe if (elements.length < elements_per_page)
    
    total_pages = (elements.length / elements_per_page) + 1
    first_element_index = elements.index(paginated_elements[0]) + 1
    current_page_index = (first_element_index / elements_per_page) + (first_element_index % elements_per_page > 0 ? 1 : 0)
    prev_page_index, next_page_index = current_page_index - 1, current_page_index + 1

    @pagination_metadata = ""
    @pagination_metadata += output_rel("prev", prev_page_index, canonical_url) if prev_page_index > 0
    @pagination_metadata += output_rel("next", next_page_index, canonical_url) if next_page_index <= total_pages
    @pagination_metadata = @pagination_metadata.html_safe
  end

  def output_rel(direction, page_index, canonical_url)
    "<link rel=\"#{direction}\" href=\"#{canonical_url}?page=#{page_index}\" />"
  end
end
