module Teachable
  module Api
    module V1
      class BonusesController < Teachable::Api::V1::ApplicationController

        def index
          if params[:upgrade_src]
            @bonuses = []
            page_bundle_upgrade_src_field_id = 95 # <-- hardcoded to avoid two hits on DB
            page_bundle_field_with_upgrade_src = ObjektField.where(field_id: page_bundle_upgrade_src_field_id, value: params[:upgrade_src]).first
            if page_bundle_field_with_upgrade_src
              field_values_hash = page_bundle_field_with_upgrade_src.objekt.field_values_hash
              timeframe         = field_values_hash["Timeframe"].downcase
              basic_bonus_ids   = field_values_hash.select { |key, value| key.include?("Basic Bonus") }.values
              pro_bonus_ids     = field_values_hash.select { |key, value| key.include?("Pro Bonus") }.values
              retrieve_bonuses({ "basic": basic_bonus_ids, "professional": pro_bonus_ids }, timeframe)
              render json: { bonuses: @bonuses.flatten }
            else
              upgrade_src = params[:upgrade_src].sub("-replay", "").sub("-play", "").sub("-email", "")
              webinar = Webinar.find_by_url(upgrade_src)
              if webinar
                retrieve_bonuses({ "basic": bonus_ids(webinar.basic_replay_bundle), "professional": bonus_ids(webinar.pro_replay_bundle) }, webinar.upgrade_timeframe.downcase)
                retrieve_bonuses({ "basic": bonus_ids(webinar.basic_play_bundle), "professional": bonus_ids(webinar.pro_play_bundle) }, webinar.upgrade_timeframe.downcase) if params[:upgrade_src].include?("-play")
                render json: { bonuses: @bonuses.flatten }
              else
                render json: { error: "Unable to find bonuses with that upgrade_src: #{params[:upgrade_src]}" }, status: :not_found
              end
            end
          else
            render json: { error: "Missing upgrade_src param" }, status: 422
          end

        end

        private

        def retrieve_bonuses(ids, timeframe)
          ids.each do |plan_type, plan_bonus_ids|
            Objekt.where(id: plan_bonus_ids).each do |objekt|
              field_values_hash = objekt.field_values_hash
              field_values_hash["Icon"] = Upload.find(field_values_hash["Icon"]).url
              field_values_hash.delete("Image")
              bonus = field_values_hash.merge(bonus_type: plan_type, bonus_type_interval: timeframe)
              bonus[:bonus_type_interval] = 'month' if bonus[:bonus_type_interval] == 'monthly'
              bonus[:bonus_type_interval] = 'year' if bonus[:bonus_type_interval] == 'annual'
              @bonuses << bonus
            end
          end
        end

        def bonus_ids(bundle_id)
          objekt = Objekt.find_by_id(bundle_id)
          objekt ? objekt.field_values_hash.select { |key, value| key.include?("Bonus") }.values : []
        end
      end
    end
  end
end
