module Teachable
  module Api
    class WebinarsController < Teachable::Api::ApplicationController

      include UpdateIterableMetadata

      after_action :update_iterable_webinar

      def replay_notification
        unless params["wistia_id"] && params["webinar_id"]
          render json: { message: "missing proper params" }, status: 422
        else
          begin
            slack = Slack::Web::Client.new
            firebase = Firebase::Client.new(Webinar.get_firebase_url, Webinar.firebase_auth_json)
            wistia_id = params["wistia_id"]
            @webinar = Webinar.find(params["webinar_id"])
            @webinar.update(replay_video_id: wistia_id)
            if @webinar.recurring
              firebase.update("", {
                "#{@webinar.id}/controls/replayVideoStatus" => "unclicked",
                "#{@webinar.id}/controls/hangoutStatus" => "Starting",
                "#{@webinar.id}/controls/youTubeID" => "",
                "#{@webinar.id}/controls/showUpgradeButton" => false,
                "#{@webinar.id}/controls/downloadSwitches/0" => false,
                "#{@webinar.id}/controls/downloadSwitches/1" => false,
                "#{@webinar.id}/controls/downloadSwitches/2" => false,
                "#{@webinar.id}/controls/downloadSwitches/3" => false,
                "#{@webinar.id}/controls/downloadSwitches/4" => false,
                "#{@webinar.id}/controls/downloadSwitches/5" => false,
                "#{@webinar.id}/controls/downloadSwitches/6" => false,
                "#{@webinar.id}/controls/downloadSwitches/7" => false
              })
              @webinar.clear_chat
            else
              firebase.update("", {
                "#{@webinar.id}/controls/replayVideoStatus" => "uploaded"
              })
            end
            slack.chat_postMessage(channel: ENV["WEBINAR_NOTIFICATIONS_CHANNEL"], text: "Replay for \"#{@webinar.title}\" has been uploaded successfully", as_user: true)
          rescue Exception => e
            firebase.update("", {
              "#{@webinar.id}/controls/replayVideoStatus" => "error"
            })
            slack.chat_postMessage(channel: channel, text: "Replay upload ERROR for \"#{@webinar.title}\"", as_user: true)
            slack.chat_postMessage(channel: channel, text: e, as_user: true)
          end
          render json: { message: "upload replay job complete" }, status: 200
        end
      end

    end
  end
end
