class WebinarsController < ApplicationController

  before_action :redirect_based_on_live_dates, except: [:automated_play]

  # do not cache register, thank you, play, or replay
  # all 4 pages need to get time on page load

  def register
    if @webinar
      # redirect if this is a non-recurring live webinar and it's been over 3 hours since the final webinar play time
      if !@webinar.recurring && @webinar.webinar_type != "automated" && @webinar.redirect && (DateTime.now.to_i > (@webinar.later_time.to_i + (3 * 60 * 60)))
        redirect_to @webinar.redirect
      else
        @webinar_page = true
        @js_includes = ["registration", "webinar-registration"]
        render "register.html.erb"
      end
    else
      @page = Page.find_by(url: "/404_error")
      render "/pages/show.html.erb", status: 404
    end
  end

  def thank_you
    if @webinar
      @webinar_page = true
      @js_includes = ["thank-you"]
      render "thank-you.html.erb"
    else
      @page = Page.find_by(url: "/404_error")
      render "/pages/show.html.erb", status: 404
    end
  end

  def play
    @ignore_summit_banner = true # TODO: Remove after summit
    if @webinar
      # redirect to replay if it's been over 3 hours since the final webinar play time
      if !@webinar.recurring && (DateTime.now.to_i > (@webinar.later_time.to_i + (3 * 60 * 60)))
        redirect_to "/webinars/#{@webinar.url}/replay"
      else
        @webinar_page = true
        @js_includes = ["play"]
        render "play.html.erb"
      end
    else
      @page = Page.find_by(url: "/404_error")
      render "/pages/show.html.erb", status: 404
    end
  end

  def automated_play
    @webinar = Webinar.find_by(url: params[:url])
    if params[:session_id].blank?
      @page = Page.find_by(url: "/404_error")
      render "/pages/show.html.erb", status: 404
    else
      # if params session_id is escaped, unescape it. Otherwise, just use the param directly
      @session_id = if param_escaped?(params[:session_id])
                      CGI.unescape(params[:session_id] || "")
                    else
                      params[:session_id]
                    end
      start_time = @webinar.decrypt_start_time(@session_id)
      @current_timestamp = Time.now.to_i - DateTime.parse(start_time).to_i
      if @webinar.duration.present? && @current_timestamp > @webinar.duration
        redirect_to "/webinars/#{@webinar.url}/replay"
      end
      @webinar_page = true
      @js_includes = ["automated-play"]
    end
  rescue ArgumentError => e # in the case that someone submits an incorrect session id
    @page = Page.find_by(url: "/404_error")
    render "/pages/show.html.erb", status: 404
  end

  def replay
    if @webinar
      @webinar_page = true
      @js_includes = ["wistia-tracker"]
      render "replay.html.erb"
    else
      @page = Page.find_by(url: "/404_error")
      render "/pages/show.html.erb", status: 404
    end
  end

  private

  def param_escaped?(param)
    param == CGI.escape(CGI.unescape(param))
  end

  def redirect_based_on_live_dates
    @webinar = Webinar.find_by(url: params[:url])
    if @webinar && !@webinar.recurring
      if @webinar.live_start_date && (DateTime.now < @webinar.live_start_date)
        @page = Page.find_by(url: "/404_error")
        render "/pages/show.html.erb", status: 404
      elsif @webinar.live_end_date && (DateTime.now > @webinar.live_end_date)
        redirect_to @webinar.redirect
      end
    end
  end

end
