# coding: utf-8
module ApplicationHelper

  def generate_partials_js(page)
    page.page_partials.map(&:partial).map do |partial|
      webpack_bundle_tag("#{partial.name.parameterize}.js")
    end.compact.join("").html_safe
  end

  def bonus_card_offset_col(index, n) # rubocop:disable Naming/UncommunicativeMethodParamName
    if index == 0 || index == 2
      if n == 2 || n == 4
        "col-md-offset-2"
      elsif n == 1
        "col-sm-offset-3 col-md-offset-4"
      end
    end
  end

  def css_value_from_color_name(color_name)
    {
      "Dark Orange" => "#FF6D2C",
      "Orange" => "#FF7F45",
      "Light Orange" => "#FFE5DA",
      "Pale Orange" => "#FFF2EC",
      "Background Orange" => "#FEF8F5",
      "White" => "#FFFFFF",
      "Background Green" => "#F4FAFA",
      "Pale Green" => "#E8F6F5",
      "Light Green" => "#D1EDEB",
      "Light Seafoam Green" => "#B5E1DD",
      "Seafoam Green" => "#7DCDC6",
      "Green" => "#1EA69A",
      "Dark Green" => "#0F8D82",
      "Black" => "#000000",
      "Orange Gradient" => "linear-gradient(73.02deg, #FFA239 0%, #FF2922 100%)",
      "Deep Emerald" => "#1B4942",
      "Goat Milk" => "#F8FAF4",
      "Celadon" => "#C8E3D8",
      "Teachable Green" => "#21CD9C",
      "Slate with Green Text" => "#222222",
      "Slate with White Text" => "#222222"

    }[color_name.titleize]
  end

  def date_iso_format(time)
    time.utc.iso8601.gsub(/-|:|\.\d+/, "")
  end

  def date_iso_format_day(time)
    time.to_date.iso8601.gsub(/-|:|\.\d+/, "")
  end

  def empty_or_other(entity_1, entity_2)
    entity_1.presence || entity_2
  end

  def filestack_url(image)
    id = image.url.split("/")[-1]
    URI.encode("https://process.filestackapi.com/#{ENV['FILESTACK_API_KEY']}/auto_image/compress/quality=value:90/#{id}")
  end

  def format_time_for_registration_modal(time)
    time.strftime("%A, %B") + " " + time.strftime("%-d").to_i.ordinalize + " @" + time.strftime("%-l%p") + " ET | " + (time - 3.hours).strftime("%-l%p") + " PT"
  end

  def generate_ics_link(url:, start_time:, end_time:, title:, description: nil)
    <<~ICS
      data:text/calendar;charset=utf8,
      BEGIN:VCALENDAR
      VERSION:2.0
      BEGIN:VEVENT
      URL:#{url}
      DTSTART:#{start_time}
      DTEND:#{end_time}
      DESCRIPTION:#{description || url}
      SUMMARY:#{title}
      END:VEVENT
      END:VCALENDAR
    ICS
  end

  def get_nested_objekt_fields(id)
    Objekt.find(id).field_values_hash
  end

  def get_schedulable_title(schedulable)
    case schedulable.class.name
    when "Banner"
      schedulable.name
    when "BlogPost"
      schedulable.title
    when "Page"
      schedulable.name
    when "Redirect"
      "#{schedulable.url} → #{schedulable.redirect}"
    end
  end

  def header_class(header_type)
    case header_type
    when "Large Black"
      "default-header"
    when "Small Green"
      "default-green-header"
    when "Strong Black"
      "strong-header"
    end
  end

  def header_color_from_background(background_color)
    {
      "#FF6D2C" => "#ffffff", # Dark Orange
      "#FF7F45" => "#ffffff", # Orange
      "#FFE5DA" => "#2c2f33", # Light Orange
      "#0F8D82" => "#ffffff", # Dark Green
      "#1EA69A" => "#ffffff", # Green
      "#D1EDEB" => "#2c2f33", # Light Green
      "#E8F6F5" => "#2c2f33", # Pale Green
      "#F4FAFA" => "#2c2f33", # Background Green
      "#FFFFFF" => "#2c2f33", # White
    }[background_color]
  end

  def hide_if_empty(input)
    input.empty? ? "hidden" : ""
  end

  def markdown(text, options = {})
    unless options[:skip_html_escape]
      text = html_escape(text)
    end
    renderer_options = { hard_wrap: true }
    renderer_options[:link_attributes] = { target: "_blank" } if options[:links_open_new_tabs]
    renderer = Redcarpet::Render::HTML.new(renderer_options)
    markdown = Redcarpet::Markdown.new(renderer, tables: true, superscript: true)
    result = markdown.render(text)
    if options[:match_height_list]
      result.gsub!("<li>", "<li data-mh=\"#{options[:match_height_list]}\" >")
    end
    result.html_safe #rubocop:disable Rails/OutputSafety
  end

  def number_of(field_name, field_values)
    regex = /(#{Regexp.quote(field_name)}) ([0-9])*/
    values = field_values.select { |field, _value| field =~ regex }.values
    i = 0
    values.each do |value|
      break if ["", "(None)"].include?(value)
      i += 1
    end
    i
  end

  def style_attr_with_background_image(image_id, partial = nil, css = nil)
    image = Upload.find_by(id: image_id)
    image_url = image && filestack_url(image)
    partial_index = partial && @page.page_partials.index(partial)

    if partial_index && partial_index > 1 && image
      style = css ? " style=\"#{css}\"" : ""
      raw("data-background-image=\"#{image_url}\"#{style}")
    elsif image
      esc_pattern = /(\(|\))/
      style_prefix = css ? "style=\"#{css} " : "style=\""
      raw("#{style_prefix}background-image: url(#{image_url.gsub(esc_pattern) { |match| "\\" + match }});\"")
    else
      style_prefix = css ? "style=\"#{css} " : "style=\""
      raw("#{style_prefix}background-image: none;\"")
    end
  end

  def output_background_image(image_id)
    image = Upload.find_by(id: image_id)
    if image
      img_url = filestack_url(image)
      esc_pattern = /(\(|\))/
      "background-image: url(#{img_url.gsub(esc_pattern) { |match| "\\" + match }});"
    else
      "background-image: none;"
    end
  end

  def output_image_src(image_id, partial = nil)
    # we're just going to assume any page partial after the second is below the fold
    partial_index = partial && @page.page_partials.index(partial)
    attr = partial_index && partial_index > 1 ? "data-src" : "src"
    image = Upload.find_by(id: image_id)
    image ? raw("#{attr}=\"#{filestack_url(image)}\"") : ""
  end

  def output_margin_bottoms(klass, obj)
    result = "#{klass} {\n"
    result += "      margin-bottom: #{obj[:computer]}px; \n"
    result += "    }\n"
    result += "  @media (max-width: 991px) {\n"
    result += "    #{klass} {\n"
    result += "      margin-bottom: #{obj[:tablet]}px; \n"
    result += "    }\n"
    result += "  }\n"
    result += "  @media (max-width: 767px) {\n"
    result += "    #{klass} {\n"
    result += "      margin-bottom: #{obj[:mobile]}px; \n"
    result += "    }\n"
    result += "  }"
  end

  def output_padding_tops(klass, obj)
    result = "#{klass} {\n"
    result += "      padding-top: #{obj[:computer]}px; \n"
    result += "    }\n"
    result += "  @media (max-width: 991px) {\n"
    result += "    #{klass} {\n"
    result += "      padding-top: #{obj[:tablet]}px; \n"
    result += "    }\n"
    result += "  }\n"
    result += "  @media (max-width: 767px) {\n"
    result += "    #{klass} {\n"
    result += "      padding-top: #{obj[:mobile]}px; \n"
    result += "    }\n"
    result += "  }"
  end

  def output_padding_bottoms(klass, obj)
    result = "#{klass} {\n"
    result += "      padding-bottom: #{obj[:computer]}px; \n"
    result += "    }\n"
    result += "  @media (max-width: 991px) {\n"
    result += "    #{klass} {\n"
    result += "      padding-bottom: #{obj[:tablet]}px; \n"
    result += "    }\n"
    result += "  }\n"
    result += "  @media (max-width: 767px) {\n"
    result += "    #{klass} {\n"
    result += "      padding-bottom: #{obj[:mobile]}px; \n"
    result += "    }\n"
    result += "  }"
  end

  def p(message)
    Rails.logger.debug(message)
  end

  def plan_params(plan, field_values, length)
    result = "?plan_id=#{length == 'Monthly' ? plan.app_monthly_plan_id : plan.app_annual_plan_id}"
    result += "&upgrade_src=#{field_values["Upgrade Src"]}" unless field_values["Upgrade Src"].empty?
    result += "&utm_sales_page_cta=#{field_values["utm_sales_page_cta"]}" unless field_values["utm_sales_page_cta"].empty?
    result += "&coupon_code=#{field_values["#{length} #{plan.name} Coupon Code"]}" if field_values["#{length} #{plan.name} Coupon Code"].present?
    result
  end

  def pricing_table_cta_url(field_values, plan, duration)
    plan_params = plan_params(plan, field_values, duration.capitalize)
    custom_redirect = field_values["Custom Redirect"]
    if custom_redirect.present?
      "#{custom_redirect}#{plan_params}"
    else
      plan_name = (plan.name == "Professional" ? "Pro" : plan.name)
      custom_redirect_field_name = "#{duration.capitalize} #{plan_name} Custom URL"
      if field_values[custom_redirect_field_name].present?
        field_values[custom_redirect_field_name]
      else
        "/create-account#{plan_params}"
      end
    end
  end

  def remove_empty_fields(field_name, field_values)
    regex = (field_name.class == Regexp ? field_name : /(#{Regexp.quote(field_name)}) ([0-9])*/)
    values = field_values.select { |field, _value| field =~ regex }.values
    values.reject! do |value|
      ["", "(None)"].include?(value)
    end
    values
  end

  def s3_url(image)
    image.s3_url ? URI.encode("https://s3.amazonaws.com/cms-uploaded-images/#{image.s3_url}") : nil
  end

  def standard_margin_bottoms(partial_id, field_values)
    output_margin_bottoms("._#{partial_id}",

                          computer: field_values["Computer Margin Bottom"],
                          tablet: field_values["Tablet Margin Bottom"],
                          mobile: field_values["Mobile Margin Bottom"],)
  end

  def text_color_from_background(background_color)
    {
      "#FF6D2C" => "#ffffff", # Dark Orange
      "#FF7F45" => "#ffffff", # Orange
      "#FFE5DA" => "#5f5f5f", # Light Orange
      "#0F8D82" => "#ffffff", # Dark Green
      "#1EA69A" => "#ffffff", # Green
      "#D1EDEB" => "#5f5f5f", # Light Green
      "#E8F6F5" => "#5f5f5f", # Pale Green
      "#F4FAFA" => "#5f5f5f", # Background Green
      "#FFFFFF" => "#5f5f5f", # White
    }[background_color]
  end

  def text_color_from_background_color_name(background_color_name)
    case background_color_name
    when "Dark Orange", "Orange", "Dark Green", "Green", "Orange Gradient"
      "#FFFFFF"
    when "Deep Emerald"
      "#C8E3D8"
    when "Goat Milk"
      "#1B4942"
    when "White"
      "#222222"
    when "Celadon"
      "#1B4942"
    when "Teachable Green"
      "#222222"
    when "Slate with Green Text"
      "#C8E3D8"
    when "Slate with White Text"
      "#FFFFFF"
    else
      "#5F5F5F"
    end
  end

  def to_css_class(*strings)
    strings.map! { |str| str.split(" ") }
    strings.flatten!
    strings.map(&:downcase).join("-")
  end

  def webpack_bundle_tag(bundlename)
    manifest_path = Rails.root.join("public", "assets", "manifest-webpack.json")
    return unless File.exist? manifest_path
    @manifest ||= JSON.parse(File.read(manifest_path))
    src = @manifest[bundlename]
    return if src.nil?
    if bundlename.end_with? ".js"
      javascript_include_tag(src)
    elsif bundlename.end_with? ".css"
      stylesheet_link_tag(src)
    else
      raise "Invalid #{bundlename}"
    end
  end

  def webinar_cta_text(text)
    if @webinar.try!(:custom_cta_text).present?
      @webinar.custom_cta_text
    else
      text
    end
  end

  def webinar_cta_url(url)
    if @webinar.try!(:custom_cta_url).present?
      custom_base_url = @webinar.custom_cta_url.split("?")[0]
      params = url.split("?")[1]
      if params.present?
        "#{custom_base_url}?#{params}"
      else
        custom_base_url
      end
    else
      url
    end
  end

  def zero_is(output, input)
    input.gsub(/px|%/, "") == "0" ? output : input
  end

  def classname(*args)
    resolved_args = args.map do |arg|
      case arg
      when String then arg
      when Hash then arg.map { |key, val| key if val }.compact
      when Array then classname(*arg)
      end
    end

    resolved_args.uniq.join(' ')
  end

end
