module Metadatable
  extend ActiveSupport::Concern

  included do
    validates_inclusion_of :active, in: [true, false]
    validates_inclusion_of :index_page, in: [true, false]
    validates_presence_of :canonical_url, :title_text, :description, if: :active
    validate :valid_canonical_url
  end

  def valid_canonical_url
    # Regex Key:
      # (http)              # http
      # (s)?                # s (optional)
      # (:\/\/)             # ://
      # (teachable\.com)    # teachable.com
      # (\/[a-z\/])*?       # lowercase characters and forward slashes only
    valid_url_regex = /^(http)(s)?(:\/\/)(teachable\.com)(\/[a-z0-9\/])*?/
    unless !!(valid_url_regex.match(canonical_url))
      errors.add(:canonical_url, "is invalid (format: http(s)://teachable.com/URL_NAME)")
    end
  end
end
