class ContentCategory < ApplicationRecord
  attr_reader :index_page, :active

  include Metadatable

  validates_presence_of :name, :url, :title_text, :description, :header,
    :subheader, :content_download_1_event_name, :content_download_2_event_name

  validates_numericality_of :content_download_1_workflow_id, :content_download_2_workflow_id

  belongs_to :content_download_1, class_name: "ContentDownload"
  belongs_to :content_download_2, class_name: "ContentDownload"

  # TODO: Needs to be removed after blog re-categorization. This is currently no different from .all
  scope :general, -> { preload(:content_download_1, :content_download_2) }

  after_initialize :set_up_page_flags

  def self.js_menu_items
    all.map { |content| { displayText: content.name, value: content.id } }
  end

  def set_up_page_flags
    @index_page = true
    @active = true
  end

  def canonical_url
    "https://teachable.com" + self.url
  end

  def nav_item
    { name: self.name.upcase, url: self.url }
  end

  def name_and_value
    { name: self.name, value: self.id }
  end
end
