class Page < ApplicationRecord

  include StandardPageValidations
  include Metadatable

  validates :name, :creator_id, presence: true
  validates :name, uniqueness: true

  enum theme: { default: 0, rebrand: 1 }

  has_many :page_partials, -> { order(:order) }, dependent: :destroy
  has_many :scheduled_tasks, as: :schedulable, dependent: :destroy
  has_many :banner_pages, dependent: :destroy

  belongs_to(
    :og_image,
    class_name: "Upload",
    foreign_key: :og_image_id,
    primary_key: :id
  )

  belongs_to(
    :creator,
    class_name: "User",
    foreign_key: :creator_id,
    primary_key: :id
  )

  before_save do
    @active_page = changes.dig("active", 0)
    @previous_url = changes.dig("url", 0)
  end

  def publish
    update!(active: true)
    clear_cache!
  end

  def unpublish
    update!(active: false)
    clear_cache!
  end

  def self.all_cached_urls
    urls = Page.where(active: true).pluck(:url)
    home_page_index = urls.find_index('/')
    urls[home_page_index] = '/index' if home_page_index
    urls
  end

  def cached_urls
    if self.active || @active_page
      url_to_clear = @previous_url || url
      route = url_to_clear == '/' ? '/index' : url_to_clear
      return [route]
    end
    []
  end

  def clear_cache!
    cached_urls.each do |url|
      Rails.cache.delete("views/#{ENV['CACHE_HOST_WITH_PORT']}#{url}")
    end
  end
end
