class PagePartialField < ApplicationRecord

  validates :page_partial_id, :partial_field_id, presence: true
  validate :partial_field_set_to_mandatory
  validate :partial_field_is_number

  belongs_to :partial_field
  belongs_to :page_partial
  has_one :page, through: :page_partial
  has_one :image, class_name: "Upload", foreign_key: :id, primary_key: :value
  has_one :instance, class_name: "Objekt", foreign_key: :id, primary_key: :value
  has_one :nested_page_partial, -> { where.not page_partial_field_id: nil }, class_name: "PagePartial", foreign_key: :id, primary_key: :value, dependent: :destroy

  def self.create_page_partial_fields!(page_partial)
    page_partial.partial_fields.each do |partial_field|
      page_partial_field = PagePartialField.new(
        page_partial_id: page_partial.id,
        partial_field_id: partial_field.id,
        value: default_partial_field_value(partial_field)
      )
      page_partial_field.save!(validate: false) # validate is set to false to allow empty string values, in case the fields have been set to mandatory in the object template
    end
  end

  def self.default_partial_field_value(partial_field)
    field_type = partial_field.field_type
    default_field_values = {
      "number" => "0",
      "color" => "#FFFFFF",
      "image" => "",
      "file" => "",
      "page-partial" => "",
      "text" => "",
      "markdown" => "",
      "header" => ""
    }
    case field_type
      when "object"
        default_field_values["object"] = Template.find_by_name(partial_field.object_type).objects.first.id
      when "dropdown"
        default_field_values["dropdown"] = partial_field.menu_options.first.value
      when "event_series"
        default_field_values["event_series"] = EventSeries.first.try(:id) || ""
      when "content_download"
        default_field_values["content_download"] = ContentDownload.first.try(:id) || ""
      when "webinar"
        default_field_values["webinar"] = Webinar.first.try(:id) || ""
      when "transcription"
        default_field_values["transcription"] = Temi.get_transcripts.first["id"]
    end
    default_field_values[field_type]
  end

  def partial_field_set_to_mandatory
    errors.add(partial_field.name, "is mandatory") if partial_field.mandatory && value == ""
  end

  def partial_field_is_number
    if partial_field.field_type == "number"
      validates_numericality_of :value, message: "#{partial_field.name} is not a number"
    end
  end
end
