class Partial < ApplicationRecord

  validates :name, presence: true, uniqueness: true

  has_many :partial_fields, -> { order('partial_fields.order') }, dependent: :destroy
  has_many :page_partials

  # Has key for image_id (does not technically 'belong to' it)
  belongs_to :image, class_name: "Upload"

  def self.pages(id)
    pages_with_partial = PagePartial.where.not(page_id: nil).where(partial_id: id).includes(:page)
    .map { |page_partial| page_partial.page }
    pages_with_nested_partial = PagePartial.where.not(page_partial_field_id: nil).where(partial_id: id).includes(page_partial_field: [page_partial: [:page]])
    .map { |page_partial| page_partial.page_partial_field.page_partial.page }
    (pages_with_partial + pages_with_nested_partial).uniq
  end


  def cached_urls
    urls = []
    Partial.pages(id).each { |page| urls.concat(page.cached_urls) }
    urls
  end
end
