class Plan < ApplicationRecord

  validates :name, :monthly_price, :order, :app_monthly_plan_id, presence: true
  validates :name, uniqueness: true
  validates_numericality_of :monthly_price, greater_than_or_equal_to: 0, only_integer: true
  validates_numericality_of :annual_price, greater_than_or_equal_to: 0, only_integer: true, if: :annual_included
  validates_numericality_of :app_monthly_plan_id, greater_than_or_equal_to: 0, only_integer: true
  validates_numericality_of :app_annual_plan_id, greater_than_or_equal_to: 0, only_integer: true, if: :annual_included

  has_many :plan_features, -> { order(:order) }, dependent: :destroy
  has_many :features, through: :plan_features

  def self.get_annual_plan_ids(plans_array = nil)
    plans = plans_array || Plan.all
    plans.select { |plan| plan.app_annual_plan_id }.map { |plan| plan.app_annual_plan_id }
  end

  def self.get_paid_plans
    paid_plans = {}
    Plan.all.each do |plan|
      next if plan.monthly_price == 0
      monthly_plan_id = plan.app_monthly_plan_id
      annual_plan_id = plan.app_annual_plan_id
      if monthly_plan_id
        paid_plans[monthly_plan_id.to_s] = plan
      end
      if annual_plan_id
        paid_plans[annual_plan_id.to_s] = plan
      end
    end
  paid_plans
  end

  def self.get_plan_names_and_ids
    plans = {}
    Plan.all.each do |plan|
      plans[plan.name] = {
        'app_monthly_plan_id' => plan['app_monthly_plan_id'],
        'app_annual_plan_id' => plan['app_annual_plan_id']
      }
    end
    plans
  end

  def cached_urls
    Partial.find_by(name: 'Pricing Table').cached_urls
  end
end
