class Redirect < ApplicationRecord

  validates :url, :redirect, presence: true
  validates :url, uniqueness: true

  validate :valid_url
  validate :valid_redirect

  has_many :scheduled_tasks, as: :schedulable, dependent: :destroy

  has_many :pages, foreign_key: "url", primary_key: "url"
  has_many :blog_posts, foreign_key: "url", primary_key: "url"

  def ineffective?
    pages.first.try(:active?) || blog_posts.first.try(:active?)
  end

  def publish
    update(active: true)
    clear_cache!
  end

  def unpublish
    update(active: false)
    clear_cache!
  end

  def valid_url
    unless url[0] == "/"
      errors.add(:url, "is invalid")
    end
  end

  def valid_redirect
    unless redirect.starts_with?("/", "http://", "https://")
      errors.add(:redirect, "is invalid")
    end
  end

  def clear_cache!
    Page.where(url: url).find_each do |page|
      page.clear_cache!
    end
  end

end
