class ScheduledTask < ApplicationRecord

  enum status: { pending: 1, successful: 2, failed: 3 }
  enum action: { publish: 1, unpublish: 2 }

  validates :action, :status, :schedulable_id, :schedulable_type, presence: true
  validates_date :time, allow_blank: false
  validates_date :time, on_or_after: DateTime.now.in_time_zone("America/New_York"), if: Proc.new { |scheduled_task| scheduled_task.status == "pending" }
  validate :schedulable?

  scope :pending,    -> { where(status: :pending) }
  scope :executable, -> { pending.where("time <= ?", Time.now)}
  scope :successful, -> { where(status: :successful) }
  scope :failed,     -> { where(status: :failed) }

  belongs_to :schedulable, polymorphic: true

  def schedulable?
    if action == "publish" && schedulable_id
      schedulable.active = true
      schedulable.valid?
      schedulable.errors.messages.each do |key, value|
        value.each do |element|
          errors.add(key, element)
        end
      end
    end
  end

  def process!
    schedulable.public_send(action) # schedulables should have public unpublish/publish methods
    update(status: :successful)
  rescue => exception
    update(status: :failed)
    Tea::Client.new.emit_message({
      slack: [
        {
          channels: ["#marketing-tech-issues"],
          message: "Issue running scheduled task ##{id}. Action: `#{action}` " +
                   "Entity: `#{schedulable.to_global_id}` Message: `#{exception.message}`"
        }
      ]
    })
  end

end
