class Template < ApplicationRecord

  validates :name, presence: true, uniqueness: true

  belongs_to :image, class_name: "Upload"

  has_many(
    :fields,
    dependent: :destroy
  )

  has_many(
    :objects,
    class_name: "Objekt",
    foreign_key: :template_id,
    primary_key: :id,
    dependent: :destroy
  )

  has_many :blog_posts, through: :objects

  scope :by_permission, -> (permission_id) { permission_id > 1 ? Template.all : Template.all.where(devs_only: false) }

  def image_url
    image_id ? Upload.find(image_id).url : ActionController::Base.helpers.image_path("admin/default-user-image.jpg")
  end

  def cached_urls
    cached_urls = []
    if self.blog_posts.length > 0
      cached_urls = BlogPost.where(status: 3).pluck(:url).map { |url| "/blog#{url}" }
    elsif name != "Blog Settings"
      cached_urls = Page.all_cached_urls if cached_urls.length == 0
    end
    cached_urls.concat(['/blog', '/blog/resources']) unless cached_urls.include?('/blog')
    cached_urls
  end
end
