class TrackingLinkRedirectEvent < ApplicationRecord

  # This is an append only model
  attr_readonly :anonymous_id,
                :created_at,
                :redirect_url,
                :id,
                :redirected_at,
                :referrer,
                :request_url,
                :tracking_link_id,
                :visitor_ip,
                :visitor_user_agent

  belongs_to :cms_user, class_name: "User", foreign_key: "cms_user_id"
  belongs_to :tracking_link

  validates :anonymous_id, presence: true
  validates :redirect_url, presence: true, length: { maximum: 2083 }, format: { with: URI.regexp(%w[http https]) }
  validates :redirected_at, presence: true
  validates :referrer, length: { maximum: 2083 }, allow_nil: true
  validates :request_url, presence: true, length: { maximum: 2083 }, format: { with: URI.regexp(%w[http https]) }
  validates :visitor_ip, presence: true

end
