module Testing
  module ConfigurationGenerator

    def self.call(key)
      page = Page.find_by(url: key)
      return nil if page.nil?
      data = {
        name: page.name,
        url: page.url,
        canonical_url: page.canonical_url,
        description: page.description,
        title_text: page.title_text,
        partials: [],
      }
      page.page_partials.each do |page_partial|
        page_partial_data = get_page_partial_data(page_partial)
        data[:partials] << page_partial_data
      end
      file_path = Rails.root.join("config", "testing", "page_definitions", "#{data[:name].parameterize}.yml")
      File.write(file_path, JSON.parse(data.to_json).to_yaml)
    end

    def self.get_page_partial_data(page_partial)
      page_partial_data = {
        name: page_partial.partial.name,
        fields: [],
      }
      page_partial.page_partial_fields.sort_by { |field| field.partial_field.order }.each do |field|
        partial_field = field.partial_field
        field_data = {
          name:        partial_field.name,
          type:        partial_field.field_type,
          object_type: partial_field.object_type,
          mandatory:   partial_field.mandatory,
          char_limit:  partial_field.char_limit,
          header_type: HeaderType.find_by(id: field.header_type).try!(:name),
          options:     partial_field.menu_options.order(:order).map do |option|
                         option.value
                       end,
          value:       get_configuration_value(field),

        }
        page_partial_data[:fields] << field_data
      end
      page_partial_data
    end

    def self.get_configuration_value(field)
      partial_field = field.partial_field

      case partial_field.field_type
      when "object"
        template = Template.find_by name: partial_field.object_type
        {
          partial_field.object_type => {
            plural: template.plural,
            fields: get_object_fields(field),
          },
        }
      when "webinar"
        Webinar.find(field.value).attributes
      when "image"
        Upload.find_by(id: field.value).try!(:url)
      when "page-partial"
        page_partial = PagePartial.find(field.value)
        get_page_partial_data(page_partial)
      else
        field.value
      end
    end

    def self.get_object_fields(original_field)
      objekt = original_field.instance
      objekt.fields.sort_by { |object_field| object_field.field.order }.map do |object_field|
        field = object_field.field
        data = {
          name: field.name,
          type: field.field_type,
          mandatory: field.mandatory,
          char_limit: field.char_limit,
          object_type: field.object_type,
        }
        data[:value] =
          if data[:type] == "object"
            {
              field.object_type => {
                plural: field.template.plural,
                fields: get_object_fields(object_field),
              },
            }
          elsif data[:type] == "webinar"
            Webinar.find(object_field.value).attributes
          elsif data[:type] == "image"
            Upload.find_by(id: object_field.value).try!(:url)
          else
            object_field.value
          end
        data
      end
    end

  end
end
