##
# Generates the final url a user is redirects to
# given a
module TrackingLinks
  module RedirectUrl

    def self.build(
      additional_query_params:,
      destination_url:,
      event_id:,
      link_id:,
      forward_all_query_params:,
      forward_query_params_whitelist:,
      request_query_params:
      )
      uri = Addressable::URI.parse(destination_url)

      # Any query parameters we want to keep from the request url
      destination_query_params = uri.query_values.to_h
      # params to keep from the request url
      forwarded_query_params = forward_all_query_params ? request_query_params : request_query_params.slice(*forward_query_params_whitelist)

      tracking_params = {
        "link_id" => link_id,
        "event_id" => event_id
      }

      # Build up the redirect url query obj
      redirect_query = {}
      redirect_query.merge!(destination_query_params)
      redirect_query.merge!(forwarded_query_params)
      redirect_query.merge!(additional_query_params)
      redirect_query.merge!(tracking_params)

      # In order to curb situations where we receive a nested query parameter,
      # we want to call to_query on the hash. The reason for this is because Addressable::URI
      # will throw an undefined method error for calling `to_str` on a hash. And this is a way
      # to get around that without having to monkey patch `to_str` onto the Hash model
      # Example of a naughty URI: https://teachable.com/r/20206108105838?src&fbclid&h&__tn__&c%5B0%5D
      uri.query_values = redirect_query.map{|k,v| v.is_a?(Hash) ? [k, v.to_query] : [k,v] }
      uri.to_s
    end
  end
end
