# frozen_string_literal: true


module TrackingLinks
  module Tracker

    def self.track_redirect(
      id:,
      tracking_link_id:,
      request_url:,
      redirect_url:,
      referrer:,
      user_agent:,
      ip:,
      anonymous_id:,
      cms_user_id: nil,
      campaign: nil
    )
      eventable_args = {
        noun: "user",
        noun_id: anonymous_id,
        verb: "visited_tracking_link",
        metadata: {
          "event_id" => id,
          "tracking_link_id" => tracking_link_id,
          "request_url" => request_url,
          "redirect_url" => redirect_url,
          "referrer" => referrer,
          "visitor_ip" => ip,
          "visitor_user_agent" => user_agent,
          "cms_user_id" => cms_user_id,
          "campaign" => campaign,
        },
        ts: Time.now,
        timeout: 5,
      }
      SendEventableEvent.perform_async(eventable_args)
      TrackingLinkRedirectEvent.create!(
        id: id,
        tracking_link_id: tracking_link_id,
        request_url: request_url,
        redirect_url: redirect_url,
        referrer: referrer,
        visitor_ip: ip,
        anonymous_id: anonymous_id,
        visitor_user_agent: user_agent,
        cms_user_id: cms_user_id,
        redirected_at: Time.now,
      )
    end

  end
end
