class ExportTrackingLinks
  include Sidekiq::Worker
  sidekiq_options :retry => false

  HEADERS = [
    "ID",
    "Slug",
    "Destination Url",
    "Status",
    "Channel",
    "Subchannel",
    "Source",
    "Campaign",
    "Subcampaign",
    "Content",
    "Medium",
    "Audience",
    "Goal",
    "Placement",
    "Coupon Code",
    "Upgrade Source",
    "CX",
    "Plan ID"
  ]

  def perform(job_id)
    job = Job.find(job_id)
    filename = "tracking-links-export-#{Time.now.to_i}.csv"
    path = "tmp/#{filename}"
    CSV.open(path, "w+") do |csv|
      csv << HEADERS
      TrackingLink.all.order(:created_at).each do |tracking_link|
        csv << [
          tracking_link.id,
          tracking_link.slug,
          tracking_link.destination_url,
          tracking_link.status,
          tracking_link.metadata["channel"],
          tracking_link.metadata["subchannel"],
          tracking_link.metadata["source"],
          tracking_link.metadata["campaign"],
          tracking_link.metadata["subcampaign"],
          tracking_link.metadata["content"],
          tracking_link.metadata["medium"],
          tracking_link.metadata["audience"],
          tracking_link.metadata["goal"],
          tracking_link.metadata["placement"],
          tracking_link.additional_query_params["coupon_code"],
          tracking_link.additional_query_params["upgrade_src"],
          tracking_link.additional_query_params["cx"],
          tracking_link.additional_query_params["plan_id"]
        ]
      end
      job.update(current_value: job.current_value + 1)
    end
    job.update(text: "Uploading to AWS", show_progress: false)
    aws_client = Aws::S3::Client.new
    file = File.read(path)
    bucket = ENV.fetch("TRACKING_LINK_EXPORTS_BUCKET")
    aws_client.put_object({ bucket: bucket, key: filename, body: file, acl: "private", content_type: "text/csv" })
    aws_presigner = Aws::S3::Presigner.new(client: aws_client)
    signed_url = aws_presigner.presigned_url(:get_object, { bucket: bucket, key: filename, expires_in: 7.days.to_i })
    job.update(done: true, metadata: { file_url: signed_url })
  end

end
