class SendWebinarRegistrationToIterable
  include Sidekiq::Worker
  include WebinarBonuses

  def perform(webinar_id, registrant_id, registration_id, params_time_2, automated_time = nil)
    data = create_body(webinar_id, registration_id, params_time_2, automated_time)
    @registrant = Registrant.find(registrant_id)
    Iterable.send_iterable_event({
      instance_id: 1,
      event_name: 'Webinar - Registered for Webinar',
      email: @registrant.email,
      data: data,
      template_id: @registration.templateid,
      campaign_id: @registration.campaignid
    })
    Iterable.send_iterable_event({
      instance_id: 1,
      event_name: 'Marketing - Email Submitted',
      email: @registrant.email,
      data: data,
      template_id: @registration.templateid,
      campaign_id: @registration.campaignid
    })
  rescue => e
    Raven.capture_exception(e)
    if Rails.env.production?
      Tea::Client.new("https://tea.teachable.cloud").emit_message({
        slack: [
          {
            channels: ["CB57C35HC"],
            message: "Uh oh. Failed to send webinar registration to Iterable:"
          },
          {
            channels: ["CB57C35HC"],
            message: e
          }
        ]
      })
    end
  end

  def create_body(webinar_id, registration_id, params_time_2, automated_time)
    params_time_2 ||= ''
    @webinar = Webinar.find(webinar_id)
    proper_start_time = automated_time.presence && DateTime.parse(automated_time) || (params_time_2.empty? ? @webinar.proper_time1 : @webinar.time2)
    @registration = Registration.find(registration_id)
    automated_play_url = generate_play_url(@webinar, automated_time)
    automated_play_session_id = automated_time.present? && CGI.escape(@webinar.encrypt_start_time(automated_time)) || ""
    bonuses, total_bonus_value = create_bonuses_array(@webinar)
    {
      'webinar_id' => @webinar.id,
      'url_string' => @webinar.url,
      'automated_play_url' => automated_play_url,
      'automated_play_session_id' => automated_play_session_id,
      'title' => @webinar.title,
      'upgrade_src' => {
        'default' => @webinar.url.to_s,
        'play' => "#{@webinar.url}-play",
        'replay' => "#{@webinar.url}-replay"
      },
      'tracking' => {
        'utm' => {
          'campaign' => @registration.utm_campaign,
          'source' => @registration.utm_source,
          'medium' => @registration.utm_medium,
          'term' => @registration.utm_term,
          'content' => @registration.utm_content
        },
        'mt' => {
          'tier' => @registration.mt_tier,
          'interval' => @registration.mt_interval,
          'campaign_group' => @registration.mt_campaign_group
        },
        'mbsy' => @registration.mbsy
      },
      'total_bonus_value' => total_bonus_value,
      'bonuses' => bonuses,
      'datetime' => proper_start_time.strftime("%F %T %z")
    }
  end

  def generate_play_url(webinar, automated_time)
    return "" if automated_time.blank?
    domain = case APP_ENV
             when "production"
               "https://teachable.com"
             when "staging", /^review-(\d+)/
               "https://zeachable.com"
             else
               ""
            end
    "#{domain}/webinars/#{webinar.url}/play"
  end

end
