# frozen_string_literal: true

require File.expand_path("boot", __dir__)

require "rails/all"
require_relative "./../lib/rack/seo_redirect.rb"
require_relative "./../lib/rack/cloudflare_middleware.rb"

Bundler.require(*Rails.groups)

module TeachableMarketing
  class Application < Rails::Application

    config.enable_dependency_loading = true
    config.autoload_paths << Rails.root.join("lib")
    config.time_zone = "Eastern Time (US & Canada)"
    config.action_dispatch.default_headers = {
      "X-Frame-Options" => "DENY",
    }
    config.filepicker_rails.api_key = ENV.fetch("FILESTACK_API_KEY", nil) # <-- nil so Docker can build an image
    config.middleware.insert_before(0, Rack::SeoRedirect)
    config.middleware.insert_before ActionDispatch::RemoteIp, CloudflareMiddleware
    config.middleware.insert_after ActionDispatch::RemoteIp, TeachableLogger::Middleware::RequestLogTagger

    config.middleware.use AAA::Middleware, site_id: "cms"

  end
end
