const { resolve, basename, extname } = require('path');
const { promisify } = require('util');
const glob = require('glob');

const assets = resolve(__dirname, '../../', 'app', 'assets')
const app = resolve(assets, 'javascripts');

const stylesheets = resolve(assets, 'stylesheets');

//joining path of directory
const globPromise = promisify(glob);
const vendorLibs = [
  resolve(app, 'teachable', 'polyfill'),
  resolve(app, 'teachable', 'fetch'),
  'jquery',
  'jquery-ujs',
  'jquery-ui',
  'jquery-ui/ui/widgets/draggable',
  'jquery-ui/ui/widgets/droppable',
  'jquery-ui/ui/unique-id',
  'bootstrap-sass/assets/javascripts/bootstrap.min',
  resolve(app, 'entry'),
  resolve(app, 'teachable', 'chat'),
  resolve(app, 'teachable', 'tools'),
  'Public/lazyload',
  'Public/video',
  'Public/tracking',
  'Public/common',
  'Public/iterable',
  'Public/referrals',
  'Public/modals/_general',
  'Public/content-download/content-download-hero-form',
  'Public/banners/cookie-banner',
  'Public/sticky/sticky-banner',
  'Public/sticky/sticky-scroll',
  'Public/countdown-timer',
];

const staticEntrypoints = {
  admin: './frontend/entry.tsx',
  public: resolve(app, 'entry.js'),
  'admin-css': resolve(stylesheets, 'admin.scss'),
  'blog-css': resolve(stylesheets, 'blog.scss'),
  'content-download-css': resolve(stylesheets, 'content-download.scss'),
  'job-post-css': resolve(stylesheets, 'job-post.scss'),
  'webinars-css': resolve(stylesheets, 'webinars.scss'),
  'pages-css': resolve(stylesheets, 'pages.scss'),
  'rebrand-css': resolve(stylesheets, 'rebrand.scss'),
};

// dynamically get entry points for public javascript
module.exports = () => Promise.all([
  // javascript for partials
  globPromise(`${app}/teachable/public/**/*.js`),
  // javascript for libraries
  globPromise(`${app}/library/**/*.js`),
])
  .then(([entrypointFiles, libs]) => {
    // for our entrypoints, we want to have a new entry point for each file
    const dynamicEntrypoints = entrypointFiles.reduce((memo, file) => {
      memo[basename(file, '.js')] = file;
      return memo;
    }, {});

    const retVal = {
      ...staticEntrypoints,
      ...dynamicEntrypoints,
      // for our libs, we want to extend the existing library list with our own
      vendor: [...vendorLibs, ...libs],
    };

    return retVal;
  });
