const { loader: miniCssExtractPluginLoader } = require('mini-css-extract-plugin');

const typeScriptLoader =   {
  test: /\.tsx?$/,
  exclude: /node_modules/,
  use: [
    { loader: 'awesome-typescript-loader' }
  ]
}
const babelLoader =   {
  test: /\.jsx?$/,
  exclude: /node_modules/,
  use: [
    { loader: 'babel-loader' }
  ]
}

const fileLoader = {
  test: /\.(eot|woff|svg|png|ttf|jpg)/,
  use: [
    { loader: 'file-loader' },
  ]
}

const cssLoaders = [
  { loader: miniCssExtractPluginLoader },
  { loader: 'css-loader' },
]

const scssLoader = {
  test: /\.s[ac]ss/,
  use: [
    ...cssLoaders,
    { loader: 'sass-loader' }
  ]
}

const cssLoader = {
  test: /\.css/,
  use: cssLoaders
}

// expose library export to property on window
const expose = (lib, ...windowVars) => ({
  test: require.resolve(lib),
  use: windowVars.map(v => ({
    loader: 'expose-loader',
    options: v
  }))
});

module.exports = [
  expose('jquery', 'jQuery', '$'),
  // TODO: making these relative makes the structure of the files brittle
  expose('../../app/assets/javascripts/teachable/tools', 'Tools'),
  expose('../../app/assets/javascripts/teachable/public/tracking', 'Tracking'),
  expose('../../app/assets/javascripts/teachable/public/common', 'Common'),
  expose('../../app/assets/javascripts/teachable/public/iterable', 'Iterable'),
  expose('../../app/assets/javascripts/teachable/public/modals/webinar-registration', 'WebinarRegistrationModal'),
  expose('../../app/assets/javascripts/teachable/public/webinars/play', 'WebinarPlay'),
  // this and WebinarPlay have a circular dependency, refactor that to a controller module
  expose('../../app/assets/javascripts/teachable/chat', 'Chat'),
  expose('../../app/assets/javascripts/library/ouibounce.min', 'ouibounce'),
  // babel loader must come last because of a weird interaction between it and the expose loader
  typeScriptLoader,
  scssLoader,
  cssLoader,
  fileLoader,
  babelLoader,
];
