class CreateTrackingLinks < ActiveRecord::Migration[5.1]

  def change
    create_table :tracking_links do |t|
      t.text :slug, limit: 256, null: false, index: true, unique: true, comment: "the identifier in the short url"
      t.text :destination_url, limit: 1000, null: false, comment: "url to redirect to"
      t.jsonb :additional_query_params, null: false, default: {}, comment: "query params to append to destination url"
      t.jsonb :metadata, null: false, default: {}, comment: "query params to append to destination url"
      t.text :forward_query_params_whitelist, array: true, default: [], comment: "whitelist of query params to forward from the destination_url"
      t.boolean :forward_all_query_params, null: false, default: false
      t.string :status, null: false, index: true, default: "draft", comment: "draft/published/hidden, should be an enum"
      t.datetime :published_at
      t.datetime :hidden_at
      t.text :description, comment: "Optional description of the link"
      t.references :creator, null: false, foreign_key: { to_table: :users }
      t.references :publisher, null: true, foreign_key: { to_table: :users }
      t.references :hider, null: true, foreign_key: { to_table: :users }
      t.timestamps
    end

    add_index  :tracking_links, :metadata, using: :gin
    add_index  :tracking_links, :additional_query_params, using: :gin
  end

end
