class CreateTrackingLinkRedirectEvents < ActiveRecord::Migration[5.1]

  def change
    enable_extension "uuid-ossp" unless extensions.include?("uuid-ossp")
    enable_extension "pgcrypto" unless extensions.include?("pgcrypto")

    create_table :tracking_link_redirect_events, id: :uuid do |t|
      t.references :tracking_link, null: false, index: true, foreign_key: true
      t.datetime :redirected_at, null: false, index: true
      t.text :referrer, limit: 2083
      t.inet :visitor_ip, null: false
      t.text :visitor_user_agent
      t.uuid :anonymous_id, null: false, index: true, comment: "stored in cookie if not set, used for tracking unique visitors"
      t.text :request_url, null: false, limit: 2083, comment: "Full url including query string of the tracking link clicked"
      t.text :redirect_url, null: false, limit: 2083, comment: "url the link redirects to"
      t.references :cms_user, {
        null: true,
        foreign_key: { to_table: :users },
        comment: "References the user table - cms_user to distinguish from teachable accounts and fedora users",
      }
      t.timestamps
    end

    add_index :tracking_link_redirect_events, :redirected_at, order: { redirected_at: :desc }, name: "index_link_redirect_events_on_redirected_at_desc"
  end

end
