@single_page_checkout @javascript
Feature: Single page checkout V2 - Without an existing Teachable Account

  Scenario: Creating an account with an invalid plan id in the params
    Given I visit the create-account-v2 page with params of "plan_id=123231231"
    Then I should see "Get Started with Teachable"
    Then I should not see "Order summary"

  Scenario: Creating an account with the plan id for a free plan in the params
    Given I visit the create-account-v2 page with params of "plan_id=4"
    Then I should see "Get Started with Teachable"
    Then I should not see "Order summary"

  Scenario: The user sees the correct information for the plan
    Given I visit the create-account-v2 page with params of "plan_id=153"
    Then I should see the correct plan information

  Scenario: Successfully signing up a US user
    Given I visit the create-account-v2 page with params of "plan_id=185"
    When I enter the following information:
      | field            | value                |
      | email            | madeline7771@fake.com |
    And I sign up with the following information:
      | field            | value             |
      | account name     | madeline          |
      | password         | password          |
      | confirm password | password          |
      | school name      | cooking           |
      | card number      | 5555555555554444  |
      | card expiration  | 10 / 21           |
      | cvc code         | 321               |
      | country code     | United States     |
      | region code      | Indiana           |
      | postal code      | 46765             |
    And I should be on a page with the title "This school isn't published right now."

  Scenario: Successfully signing up a Canadian user
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I sign up with the following information:
      | field            | value               |
      | email            | madeline241@fake.com |
    And I sign up with the following information:
      | field            | value             |
      | account name     | madeline          |
      | password         | password          |
      | confirm password | password          |
      | school name      | cooking           |
      | card number      | 4000001240000000  |
      | card expiration  | 10 / 21           |
      | cvc code         | 321               |
      | country code     | Canada            |
      | postal code      | M4C1N3            |
    And I should be on a page with the title "This school isn't published right now."

  Scenario: Api errors show and clear
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I sign up with the following information:
      | field            | value                |
      | email            | petery@teachable.com  |
    And I sign up with the following information:
      | field            | value                |
      | account name     | Peter                |
      | password         | password             |
      | confirm password | password             |
      | school name      | test                 |
      | card number      | 5555555555554444     |
      | card expiration  | 10 / 21              |
      | cvc code         | 321                  |
      | country code     | United States        |
      | region code      | Indiana              |
      | postal code      | 46765                |
    Then I should see "School name is invalid"

    When I enter the following information:
      | field       | value   |
      | school name | cooking |
    Then I should not see "School name is invalid"

  Scenario: Field errors show and clear
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I press the purchase button
    Then I should see "Please enter a valid email."

    And I should see "Please enter a valid card number."
    And I should see "Please enter a valid expiration date."
    And I should see "Please enter a valid cvc code."
    And I should see "Please select a country."
    And I should see "Please enter a valid zip code."

    And I should not see "Passwords do not match."
    And I should not see "Please select a state"

    When I enter the following information:
      | field        | value                  |
      | email        | madeline.test@test.com |
      | card number  | 4242424242424242       |
    Then I should not see "Please enter a valid email."
    And I should not see "Please enter a valid card number."

  Scenario: Confirm password and state errors show
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I enter the following information:
      | field        | value            |
      | email        | testtt@testt.com |
    And I sign up with the following information:
      | field        | value         |
      | password     | password      |
      | country code | United States |
    Then I should see "Passwords do not match."
    And I should see "Please select a state."
    And I should not see "Please enter a password which is at least six characters."

  Scenario: Errors show for invalid stripe fields
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I sign up with the following information:
      | field           | value   |
      | card number     | 424     |
      | card expiration | 10 / 11 |
      | cvc code        | 3       |
    Then I should see "Please enter a valid card number."
    And I should see "Please enter a valid expiration date."
    And I should see "Please enter a valid cvc code."

  Scenario: Country and billing code error shows
    Given I visit the create-account-v2 page with params of "plan_id=153"
    When I sign up with the following information:
      | field            | value               |
      | email            | madeline161@fake.com |
    And I sign up with the following information:
      | field            | value             |
      | account name     | madeline          |
      | password         | password          |
      | confirm password | password          |
      | school name      | cooking           |
      | card number      | 4000001240000000  |
      | card expiration  | 10 / 21           |
      | cvc code         | 321               |
      | country code     | Algeria           |
      | postal code      | M4C1N3            |
    Then I should see "Your credit card and billing country do not match. Please reselect your billing country."
