class RedirectContext < CmsContext
  def given_redirect(url, redirect)
    Redirect.create!({url: url, redirect: redirect})
  end

  def given_on_show_page(url, _)
    @redirect_id = Redirect.find_by(url:  url).id
    visit "admin/redirects/#{@redirect_id}#{get_params}"
  end

  def create_redirects
    get_setup_reidrects.each do |redirect_object|
      Redirect.create!(redirect_object)
    end
  end

  def create_redirect(name)
    @redirect_id, @field_values = Redirect.all.length + 1, get_redirects[name]
    within('.admin-modal') do
      @field_values.each do |field, value|
        fill_in_by_data_field(field, value)
      end
      click_and_wait('Add Redirect')
    end
    @field_values = @field_values.merge({ 'id' => @redirect_id, 'status_code' => '301', 'active' => true })
  end

  def change_field(field, value)
    set_original_field_values(field)
    @field_values[field] = value
    fill_in_by_data_field(field, value)
    click_and_wait('Save')
  end

  def expect_all
    index_values = get_index_values
    Redirect.all.each { |redirect| expect_in_index(redirect, index_values) }
  end

  def expect_in_index(redirect, index_values = get_index_values)
    db_redirect_values = get_values_array(redirect)
    expect(field_values_in_index?(db_redirect_values, index_values)).to be true
  end

  def expect_alphabetized
    expect_sorted_text(get_all_urls)
  end

  def expect_matching_database_values
    redirect = Redirect.find(@redirect_id)
    new_database_values = Redirect.column_names.map { |name|[name, redirect[name]] }.to_h
    new_database_values.delete('created_at')
    new_database_values.delete('updated_at')
    @field_values.delete('created_at')
    @field_values.delete('updated_at')
    expect(new_database_values).to eq(@field_values)
  end

  def expect_not_in_database(url)
    expect(Redirect.find_by(url: url)).to be_falsey
  end

  private

  def get_all_urls
    row_values = []
    within('tbody') do
      all_rows = all('tr')
      all_rows.each do |row|
        if row['data-id']
          within(row) do
            row_values << all('td').first.text
          end
        end
      end
    end
    row_values
  end

  def get_setup_reidrects
    [
      {
        'url' => '/test',
        'redirect' => '/example'
      },
      get_redirects['Home']
    ]
  end

  def get_redirects
    {
      'Home' => {
        'url' => '/home',
        'redirect' => 'https://www.google.com'
      }
    }
  end

  def get_values_array(redirect)
    columns = Redirect.column_names
    values = []
    columns.each do |column|
      next if column == 'id' || column == 'status_code' || column == 'updated_at' || column == 'created_at'
      value = redirect[column]
      values << convert_to_frontend_format(value)
    end
    values
  end

  def get_index_values
    within('.admin-table') do
      all('tr').map do |row|
        within(row) do
          all('td').map do |column|
            column.text
          end
        end
      end
    end
  end

  def set_original_field_values(field)
    redirect = Redirect.find(@redirect_id)
    @field_values = Redirect.column_names.map {|name|[name, redirect[name]]}.to_h
  end

  def convert_to_frontend_format(value)
    if value == true
      return 'Yes'
    elsif value == false
      return 'No'
    end
    return value
  end
end
