class TooltipAndFeatureContext < CmsContext

  def initialize
    super
    create_plans
    create_plan_features
    create_pricing_page
  end

  def given_a_tooltip_or_feature(text, item)
    @id = make_class_name(item).find_by(text: text).id
    @text = text
  end

  def create_tooltip_or_feature(text, item)
    @text ||= text
    within("##{item}s-index") { click_a_tag("Add #{item.capitalize}") }
    within("#new-entity") do
      find('input').set(text, { clear: :backspace })
      click_and_wait("Add #{item.capitalize}")
    end
  end

  def change_tooltip_or_feature(item, new_text)
    @new_text = new_text
    find('input').set(new_text)
    click_and_wait('Save')
  end

  def delete(item)
    click_a_tag('Delete')
    click_a_tag('Yes')
  end

  def expect_all(item)
    item = item[0...-1]
    make_class_name(item).all.each do |item_instance|
      expect_in_index('should', item_instance.text, item)
    end
  end

  def expect_alphabetized(item)
    within("div##{item}-index table") do
      text = all("a").map { |instance| instance["innerHTML"] }
      expect_sorted_text(text)
    end
  end

  def expect_existance(item, should_or_not)
    if should_or_not == 'should'
      expect_in_database(item)
      expect_in_index('should', @text, item)
    else
      expect_not_in_database(item)
      expect_in_index('should_not', @text, item)
    end
  end

  def expect_one_instance(type)
    number_of_instances = make_class_name(type).where(text: @text).length
    expect(number_of_instances).to eq(1)
  end

  def expect_correct_information(item)
    expect_new_text_in_database(item)
    expect_in_index('should', @new_text, item)
    expect_in_index('should not', @text, item)
  end

  private

  def expect_in_index(should_or_not, text, item)
    within("div##{item}s-index .component") do
      items = all('a').select do |element|
        element['innerHTML'] == text
      end

      if should_or_not == 'should'
        expect(items.length).to eq(1)
      else
        expect(items.length).to eq(0)
      end
    end
  end

  def expect_in_database(item)
    class_name = make_class_name(item)
    expect(class_name.find_by(text: @text)).to be_truthy
  end

  def expect_not_in_database(item)
    class_name = make_class_name(item)
    expect(class_name.find_by(text: @text)).to be_falsey
  end

  def expect_new_text_in_database(item)
    class_name = make_class_name(item)
    expect(class_name.find(@id).text).to eq(@new_text)
  end
end
