import ChangeCase from 'change-case'

export function fetchEntities(input) {
  let directory;
  let url;
  if (typeof input === 'string') {
    directory = input;
    url = `/admin/api/${directory}`;
  } else {
    directory = input.directory;
    url = input.url || `/admin/api/${directory}`;
  }
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'FETCH_ENTITIES' });
        dispatch(obj);
      }
    );
  }
}

export function fetchDataForNew(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: `/admin/api/${args.directory}/new`
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'NEW_ENTITY_DATA' });
        dispatch(obj);
      }
    );
  }
}

export function fetchDataForCopy(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: `/admin/api/${args.directory}/copy?id=${args.id}`
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'COPY_ENTITY_DATA' });
        dispatch(obj);
      }
    );
  }
}

export function createEntity(args) {
  let data = { [ChangeCase.snakeCase(args.entityName)]: Tools.convertObjectKeysToUnderscore(args.entity) };
  if (args.additionalData) {
    data = Object.assign(data, Tools.convertObjectKeysToUnderscore(args.additionalData));
  }
  return (dispatch) => {
    return $.ajax({
      method: 'POST',
      url: `/admin/api/${args.directory}`,
      data
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'CREATE_ENTITY' });
        dispatch(obj);
      },
      (response) => dispatch({
        type: 'ERRORS',
        errors: response.responseJSON
      })
    );
  }
}

export function fetchEntity(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: `/admin/api/${args.directory}/${args.id}`
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'FETCH_ENTITY' });
        dispatch(obj);
      }
    );
  }
}

export function updateEntity(args) {
  let data = { [ChangeCase.snakeCase(args.entityName)]: Tools.convertObjectKeysToUnderscore(args.entity) };
  if (args.additionalData) {
    data = Object.assign(data, Tools.convertObjectKeysToUnderscore(args.additionalData));
  }
  return (dispatch) => {
    return $.ajax({
      method: 'PATCH',
      url: `/admin/api/${args.directory}/${args.id}`,
      data
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'UPDATE_ENTITY' });
        dispatch(obj);
      },
      (response) => {
        let responseJSON = response.responseJSON;
        if (Array.isArray(responseJSON)) {
          dispatch({
            type: 'ERRORS',
            errors: response.responseJSON
          });
        } else {
          let obj = Object.assign(responseJSON, { type: 'ERRORS' });
          dispatch(obj);
        }
      }
    );
  }
}

export function deleteEntity(args) {
  const { directory, id, callback } = args;
  let { data } = args;
  if (data) {
    data = Tools.convertObjectKeysToUnderscore(data);
  }
  return (dispatch) => {
    return $.ajax({
      method: 'DELETE',
      url: `/admin/api/${directory}/${id}`,
      data
    }).then(
      (response) => {
        if (callback) {
          callback.call({}, response);
        } else {
          let directories = window.location.pathname.split('/');
          directories.pop()
          window.location.pathname = directories.join('/');
        }
      }
    );
  }
}

export function rearrangeEntities(args) {
  let data = { new_order: args.newOrder };
  if (args.additionalData) {
    data = Object.assign(data, Tools.convertObjectKeysToUnderscore(args.additionalData));
  }
  return (dispatch) => {
    return $.ajax({
      method: 'POST',
      url: `/admin/api/${args.directory}/rearrange`,
      data
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'REARRANGE_ENTITIES' });
        dispatch(obj);
      }
    );
  }
}

export function uploadReplay(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'POST',
      url: '/admin/api/webinars/upload_replay',
      data: {
        webinar_id: args.webinarId,
        youtube_id: args.youTubeID
      }
    });
  }
}

export function checkDelete(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: `/admin/api/check_delete/${directory}/${id}`
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'CHECK_DELETE' });
        dispatch(obj);
      }
    );
  }
}

export function copyWebinar(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'POST',
      url: `/admin/api/webinars/${args.oldWebinarId}`,
      data: {
        webinar: {
          name: args.newWebinar.name,
          url: args.newWebinar.url
        }
      }
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'COPY_WEBINAR' });
        dispatch(obj);
      },
      (response) => {
        dispatch({
          type: 'ERRORS',
          errors: response.responseJSON
        });
      }
    );
  }
}

export function fetchBlogSettings() {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: '/admin/api/blog_settings'
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'FETCH_BLOG_SETTINGS' });
        dispatch(obj);
      }
    );
  }
}

export function updateBlogSettings(data) {
  return (dispatch) => {
    return $.ajax({
      method: 'POST',
      url: '/admin/api/blog_settings',
      data
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'UPDATE_BLOG_SETTINGS' });
        dispatch(obj);
      },
      (response) => {
        dispatch({
          type: 'ERRORS',
          errors: response.responseJSON
        });
      }
    );
  }
}

export function exportTrackingLinks() {
  return (dispatch) => {
    return $.ajax({
      method: 'GET',
      url: '/admin/api/tracking_links/export'
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'EXPORT_TRACKING_LINKS' });
        dispatch(obj);
      }
    );
  }
}

export function updateUserPassword(args) {
  return (dispatch) => {
    return $.ajax({
      method: 'PATCH',
      url: `/admin/api/users/${args.id}/update_password`,
      data: {
        password: args.password
      }
    }).then(
      (response) => {
        let obj = Object.assign(response, { type: 'UPDATE_ENTITY' });
        dispatch(obj);
      },
      (response) => {
        let responseJSON = response.responseJSON;
        if (Array.isArray(responseJSON)) {
          dispatch({
            type: 'ERRORS',
            errors: response.responseJSON
          });
        } else {
          let obj = Object.assign(responseJSON, { type: 'ERRORS' });
          dispatch(obj);
        }
      }
    );
  }
}
