import React from 'react';
import _DeleteButton from './_delete_button';

interface _BlockEntityProps {
  block: any
  blockProps: any,
  contentState: any
}

interface _BlockEntityState extends ComponentState {
  height?: number,
  isMounted?: boolean,
  isResizing?: boolean,
  isSelected: boolean,
  previewHeight?: number,
  previewWidth?: number,
  width?: number
}

export default class _BlockEntity extends React.Component<_BlockEntityProps, _BlockEntityState> {
  entityKey: any
  entityType: any
  initialEntityDataValues: any
  selfSelector: any

  constructor(props) {
    super(props);
    this.initialEntityDataValues = this.getInitialEntityDataValues();

    this.state = { ...this.defaultState() };

    this.entityType = this._getOwnEntityType();
    this.entityKey = this.props.block.getEntityAt(0);
    this.selfSelector = `.custom-block-entity-${this.entityKey}`;

    this.handleMouseDown = this.handleMouseDown.bind(this);
  }

  // SETUP -----

  defaultState() {
    return {
      isSelected: false,
    };
  }

  getInitialEntityDataValues() {}

  // LIFECYCLE & CLICK HANDLING -----

  shouldComponentUpdate(nextProps, nextState) {
    const wrapper = this.refs.wrapper as HTMLElement;
    const wrapperRect = wrapper && wrapper.getBoundingClientRect();
    if (wrapperRect.top > 2000 || wrapperRect.bottom < 0) { return false; }

    const editor = $('#text-editor-admin')[0];
    const editorRect = editor && editor.getBoundingClientRect();

    const aboveView = wrapperRect.bottom < editorRect.top;
    const belowView = wrapperRect.top > editorRect.bottom;
    return !aboveView && !belowView;
  }

  componentDidMount() {
    $(':root').on('mousedown', this.handleMouseDown);
    this._validate();
    this._setOwnDefaultBlockStyles();
  }

  componentWillUnmount() {
    $(':root').off('mousedown', this.handleMouseDown);
  }

  handleMouseDown(e) {
    const wrapper = this.refs.wrapper as HTMLElement;
    const clickIsInside = wrapper.contains(e.target);
    if (clickIsInside) {
      this.selectOwnBlock();
      this.setState({ isSelected: true });
      this.handleClickInside();
    } else {
      this.setState({ isSelected: false });
      this.handleClickOutside();
    }
  }

  // PUBLIC (HANDLING) -----

  handleClickInside() {
    // When you click IN/ON the component
  }

  handleClickOutside() {
    // When you click outside of the component
  }

  // PUBLIC (DRAFT INTERACTION) -----

  selectOwnBlock() {
    const { blockProps, block } = this.props;
    const blockKey = block.getKey();
    blockProps.selectAtomicBlock(blockKey);
  }

  removeOwnBlock() {
    const { blockProps, block } = this.props;
    const blockKey = block.getKey();
    blockProps.removeAtomicBlock(blockKey);
  }

  getOwnBlockData() {
    const { block } = this.props;
    return block.getData();
  }

  mergeOwnBlockData(data) {
    const { block, blockProps } = this.props;
    const blockKey = block.getKey();
    blockProps.mergeBlockDataByKey(blockKey, data);
  }

  toggleOwnMetaStyle(styleCode) {
    const { blockProps } = this.props;
    blockProps.toggleBlockMetaStyle(styleCode);
  }

  getOwnEntityData() {
    const { block, contentState } = this.props;
    const entityKey = block.getEntityAt(0);
    const entity = contentState.getEntity(entityKey);
    return entity.getData();
  }

  mergeOwnEntityData(data) {
    const { block, blockProps } = this.props;
    const entityKey = block.getEntityAt(0);
    blockProps.mergeEntityData(entityKey, data);
  }

  // PUBLIC (RENDERING) -----

  renderDeleteButton() {
    return (
      <_DeleteButton
        ref="deleteButton"
        isCollapsed={ !this.state.isSelected }
        deleteFn={ this.removeOwnBlock.bind(this) }
      />
    );
  }

  renderLabel() {
    return <div className="custom-block-label">{ this.entityType }</div>
  }

  // PRIVATE -----

  _validate() {
    const defaultStateKeys = Object.keys(this.defaultState());
    const currentStateKeys = Object.keys(this.state);
    const errorMessage = `ERROR: ${this.constructor.name}'s state does not contain its default state.
      You likely over-wrote this.state in a child constructor. Be sure to use
      "this.specificState()" instead.`

    defaultStateKeys.forEach(defaultKey => {
      if (!currentStateKeys.includes(defaultKey)) throw(errorMessage);
    });
  }

  _clickIsInside(e) {
    const { wrapper } = this.refs;
    if (!wrapper) { throw(`ERROR: Component ${this.constructor.name} did not receive a "wrapper" ref`)}
  }

  _getOwnEntityType() {
    const { block, contentState } = this.props;
    const entityKey = block.getEntityAt(0);
    const entity = contentState.getEntity(entityKey);
    return entity.getType();
  }

  _setOwnDefaultBlockStyles() {
    const { defaultBlockStyles } = this.props.blockProps;
    if (!defaultBlockStyles) { return; }

    const blockData = this.getOwnBlockData().toJS();
    defaultBlockStyles.forEach((blockStyle) => {
      const [blockDataKey, blockDataValue] = blockStyle.split('-');
      if (!blockData[blockDataKey]) blockData[blockDataKey] = blockDataValue;
    });
    Object.keys(blockData).forEach(k => this.mergeOwnBlockData({ [k]: blockData[k] }));
  }
}
