import React from 'react';
import { getStyleIcon } from '../../../util/icon-util';

interface DeleteButtonProps {
  deleteFn: any
  isCollapsed: boolean
}

interface DeleteButtonState extends ComponentState {
  isCollapsed: boolean
  isHovered: boolean
}

export default class _DeleteButton extends React.Component<DeleteButtonProps, DeleteButtonState> {
  constructor(props) {
    super(props);
    this.state = {
      isHovered: false,
      isCollapsed: true,
    }
  }

  componentWillReceiveProps(nextProps) {
    if (!nextProps.isCollapsed && this.props.isCollapsed) {
      setTimeout(() => this.setState({ isCollapsed: false }), 400);
    } else if (nextProps.isCollapsed && !this.state.isCollapsed) {
      this.setState({ isCollapsed: true });
    }
  }

  handleMouseEnter() {
    this.setState({ isHovered: true });
  }

  handleMouseLeave() {
    this.setState({ isHovered: false });
  }

  render() {
    const { isCollapsed, isHovered } = this.state;
    const props = {
      className: (
        'delete-button custom-component-delete-button'
        + (isCollapsed ? ' collapsed' : '')
        + (isHovered ? ' hovered' : '')
      ),
      onClick: this.props.deleteFn,
      onMouseEnter: this.handleMouseEnter.bind(this),
      onMouseLeave: this.handleMouseLeave.bind(this),
    };
    const Icon = getStyleIcon(isHovered ? 'media-delete-hover' : 'media-delete', false);
    return <div { ...props }>{ Icon }</div>;
  }
}
