import React from 'react'

interface _EditableHTMLProps {
  className: string
  handleChange: any
  isSelected: boolean
  value: string
}

interface _EditableHTMLState extends ComponentState {
  value: string
}

export default class _EditableHTML extends React.Component<_EditableHTMLProps, _EditableHTMLState> {
  constructor(props) {
    super(props);
    this.state = {
      value: this.props.value || '',
    }
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevProps.isSelected && this.props.isSelected) {
      this._resizeTextArea();
    }
  }

  handleTextChange(e) {
    e.preventDefault();
    const value = e.target.value;
    this.setState({ value }, () => {
      this.props.handleChange(value);
      this._resizeTextArea();
    });
  }

  render() {
    const { isSelected, className } = this.props;
    return (
      <div className={ className }>
        { isSelected ? this.renderEditableHTML() : this.renderHTML() }
      </div>
    );
  }

  renderEditableHTML() {
    return (
      <textarea
        style={ { overflowY: 'hidden' } }
        ref="textarea"
        className={ 'editable-html' }
        onChange={ this.handleTextChange.bind(this) }
        value={ this.state.value }
      />
    );
  }

  renderHTML() {
    const html = { __html: this.state.value }
    return (
      <div>
        <div className="html-preview" dangerouslySetInnerHTML={ html } />
        { this._hasScript() ? <p className="script-indicator">!Script Inserted!</p> : null }
      </div>
    );

  }

  _hasScript() {
    return !!this.state.value.match(/<script.*<\/script>/);
  }

  _resizeTextArea() {
    const { textarea } = this.refs;
    textarea["style"].height = 'auto';
    textarea["style"].height = (textarea["scrollHeight"]) + 'px';
  }
}
