import React from 'react'

interface _EditableLinkButtonProps {
  buttonTextValue: string
  className: string
  handleButtonHrefChange: any
  handleButtonTextChange: any
  isSelected: boolean
  linkHrefValue: string
}

interface _EditableLinkButtonState extends ComponentState {
  buttonTextPlaceholder: string
  buttonTextValue: string
  linkHrefPlaceholder: string
  linkHrefValue: string
}

export default class _EditableLinkButton extends React.Component<_EditableLinkButtonProps, _EditableLinkButtonState> {
  constructor(props) {
    super(props);
    this.state = {
      buttonTextPlaceholder: 'This is the button text',
      buttonTextValue: this.props.buttonTextValue || '',

      linkHrefValue: this.props.linkHrefValue || '',
      linkHrefPlaceholder: 'This is the link URL'
    }
  }

  // HANDLERS -----

  handleButtonTextChange(e) {
    e.preventDefault();
    const buttonTextValue = e.target.value;
    this.setState({ buttonTextValue }, () => {
      this.props.handleButtonTextChange(buttonTextValue);
    });
  }

  handleLinkHrefChange(e) {
    e.preventDefault();
    const linkHrefValue = e.target.value;
    this.setState({ linkHrefValue }, () => {
      this.props.handleButtonHrefChange(linkHrefValue);
    });
  }

  // RENDERING -----

  render() {
    const { isSelected, className } = this.props;
    return (
      <div className={ className }>
        { isSelected ? this.renderSelected() : this.renderUnselected() }
      </div>
    );
  }

  renderSelected() {
    return (
      <div className="cta-button-container">
        <div className="button orange-button">
          <input
            placeholder={ this.state.buttonTextPlaceholder }
            onFocus={ () => this.setState({ buttonTextPlaceholder: '' }) }
            onBlur={ () => this.setState({ buttonTextPlaceholder: 'This is the button text' }) }
            ref="buttonTextInput"
            className="button-text-input"
            value={ this.state.buttonTextValue }
            onChange={ this.handleButtonTextChange.bind(this) }
            size={ 30 }
          />
        </div>
        <div className="connector-line"></div>
        <input
          placeholder={ this.state.linkHrefPlaceholder }
          onFocus={ () => this.setState({ linkHrefPlaceholder: '' }) }
          onBlur={ () => this.setState({ linkHrefPlaceholder: 'This is the link URL' }) }
          ref="linkHrefInput"
          className="link-href-input"
          value={ this.state.linkHrefValue }
          onChange={ this.handleLinkHrefChange.bind(this) }
          size={ 50 }
        />
      </div>
    );
  }

  renderUnselected() {
    const { buttonTextValue, linkHrefValue } = this.state;
    return (
      <div className="cta-button-container">
        <a className="button orange-button" href={ linkHrefValue }>
          { buttonTextValue || 'This is the button text' }
        </a>
      </div>
    );
  }
}
