import React from 'react'

interface _EditableTextProps {
  className: string
  handleChange: any
  isSelected: boolean
  value: string
}

interface _EditableTextState extends ComponentState {
  value: string
}

export default class _EditableText extends React.Component<_EditableTextProps, _EditableTextState> {
  constructor(props) {
    super(props);
    this.state = {
      value: this.props.value || '',
    }
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevProps.isSelected && this.props.isSelected) {
      this._resizeTextArea();
    }
  }

  handleTextChange(e) {
    e.preventDefault();
    const value = e.target.value;
    this.setState({ value }, () => {
      this.props.handleChange(value);
      this._resizeTextArea();
    });
  }

  render() {
    const { isSelected, className } = this.props;
    return (
      <div className={ className }>
        { isSelected ? this.renderTextarea() : this.renderDiv() }
      </div>
    );
  }

  renderTextarea() {
    return (
      <textarea
        style={ { overflow: 'hidden' } }
        ref="textarea"
        className="editable-text"
        onChange={ this.handleTextChange.bind(this) }
        value={ this.state.value }
      />
    );
  }

  renderDiv() {
    return <div>{ this.state.value }</div>
  }

  _resizeTextArea() {
    const { textarea } = this.refs;
    textarea["style"]["height"] = 'auto';
    textarea["style"]["height"] = (textarea["scrollHeight"]) + 'px';
  }
}
