import React from 'react';
import _BlockEntity from '../_common/_block_entity';
import _DeleteButton from '../_common/_delete_button';
import _EditableText from './_editable-text';
import _EditableLinkButton from './_editable-link-button';
import _EditableLink from './_editable-link';
import _EditableHTML from './_editable-html';

export default class _EditableContent extends _BlockEntity {
  constructor(props) {
    super(props);
  }

  // SETUP -----

  getInitialEntityDataValues() {
    const entityData = this.getOwnEntityData();
    const fieldData = this.fieldData();

    const initialValues = {};
    Object.keys(fieldData).forEach((fieldName) => {
      const fieldValue = entityData[fieldName];
      const coercedFieldValue = (fieldValue || fieldValue === '') ? fieldValue : fieldData[fieldName];
      initialValues[fieldName] = coercedFieldValue;
    });

    return initialValues;
  }

  // PUBLIC -----

  fieldData() { return null; } // Overwritten per-component

  handleFieldChange(fieldName, value) {
    this.mergeOwnEntityData({ [fieldName]: value });
  }

  // RENDERING HELPERS -----

  wrapContent(content) {
    const props = {
      ref: 'wrapper',
      className: 'container editable-content-container' + (this.state.isSelected ? ' selected' : ''),
      'data-remain-read-only': 'true',
    };
    return (
      <div { ...props } >
        <div className="row">
          { content }
        </div>
      </div>
    );
  }

  renderEditableText(fieldName, className) {
    return (
      <_EditableText
        value={ this.initialEntityDataValues[fieldName] }
        handleChange={ (value) => this.handleFieldChange(fieldName, value) }
        isSelected={ this.state.isSelected }
        className={ className }
      />
    );
  }

  renderEditableLinkButton(buttonTextFieldName, buttonHrefFieldName, className) {
    return (
      <_EditableLinkButton
        buttonTextValue={ this.initialEntityDataValues[buttonTextFieldName] }
        linkHrefValue={ this.initialEntityDataValues[buttonHrefFieldName] }
        handleButtonTextChange={ (value) => this.handleFieldChange(buttonTextFieldName, value) }
        handleButtonHrefChange={ (value) => this.handleFieldChange(buttonHrefFieldName, value) }
        isSelected={ this.state.isSelected }
        className={ className }
      />
    );
  }

  renderEditableLink(linkHrefFieldName, className) {
    return (
      <_EditableLink
        linkHrefValue={ this.initialEntityDataValues[linkHrefFieldName] }
        handleChange={ (value) => this.handleFieldChange(linkHrefFieldName, value) }
        isSelected={ this.state.isSelected }
        className={ className }
      />
    );
  }

  renderEditableHTML(fieldName, className) {
    return (
      <_EditableHTML
        value={ this.initialEntityDataValues[fieldName] }
        handleChange={ (value) => this.handleFieldChange(fieldName, value) }
        isSelected={ this.state.isSelected }
        className={ className }
      />
    );
  }

  // PRIVATE -----

  _validate() {
    super._validate();
    if (!this.fieldData()) {
      throw(`
        WARNING: ${this.constructor.name} received no fieldData() function. Define
        a fieldData() function on the component that returns:
          { [fieldName]: "placeholder text" }
      `);
    }
  }
}
