import React from 'react'

interface _MediaResizeCornerProps {
  direction: string,
  height: number,
  isSelected: boolean,
  previewResize: any,
  resize: any,
  toolbarHeight: number,
  width: number
}

interface _MediaResizeCornerState extends ComponentState {
  dragEndX: number,
  dragEndY: number,
  dragStartX: number,
  dragStartY: number,
  resizeFactor: number
}

export default class _MediaResizeCorner extends React.Component<_MediaResizeCornerProps, _MediaResizeCornerState> {
  constructor(props) {
    super(props);

    this.state = {
      resizeFactor: 1,

      dragStartX: null,
      dragStartY: null,

      dragEndX: null,
      dragEndY: null,
    }
  }

  // HANDLERS -----

  handleResize() {
    const { dragStartX, dragStartY, dragEndX, dragEndY } = this.state;
    const { direction, width, height, previewResize } = this.props;
    let diffs = {
      x: (dragEndX - dragStartX),
      y: (dragEndY - dragStartY),
    };

    let action
    switch (direction) {
      case 'nw':
        action = (diffs.x > 0 && diffs.y > 0) ? 'shrink' : 'grow';
        break;
      case 'ne':
        action = (diffs.x < 0 && diffs.y > 0) ? 'shrink' : 'grow';
        break;
      case 'sw':
        action = (diffs.x > 0 && diffs.y < 0) ? 'shrink' : 'grow';
        break;
      case 'se':
        action = (diffs.x < 0 && diffs.y < 0) ? 'shrink' : 'grow';
        break;
      default:
        break;
    }

    Object.keys(diffs).forEach(axis => diffs[axis] = Math.abs(diffs[axis]));
    let resizeFactor;
    if (action === 'shrink') {
      if (diffs.x > diffs.y) {
        resizeFactor = 1 - (diffs.x / width);
      } else {
        resizeFactor = 1 - (diffs.y / height);
      }
    } else {
      if (diffs.x > diffs.y) {
        resizeFactor = 1 + (diffs.x / width);
      } else {
        resizeFactor = 1 + (diffs.y / height);
      }
    }

    this.setState({ resizeFactor },
      () => previewResize(width * resizeFactor, height * resizeFactor)
    );
  }

  handleDragStart(e) {
    const { x, y } = this._determineDragStart({ x: e.screenX, y: e.screenY });
    this.setState({
      dragStartX: x,
      dragStartY: y,
    });
  }

  handleDrag(e) {
    this.setState({
      dragEndX: e.screenX || this.state.dragEndX,
      dragEndY: e.screenY || this.state.dragEndY,
    }, () => this.handleResize());
  }

  handleDragEnd(e) {
    this.props.resize(this.state.resizeFactor);
  }

  // RENDERING -----

  render() {
    const { direction, isSelected } = this.props;

    return (
      <div
        ref="self"
        className={ `resize-corner ${direction}` + (isSelected ? '' : ' collapsed') }
        draggable
        onDrag={ this.handleDrag.bind(this) }
        onDragStart={ this.handleDragStart.bind(this) }
        onDragEnd={ this.handleDragEnd.bind(this) }
      >
      </div>
    );
  }

  // PRIVATE -----

  _getToolbarHeight() {
    return (window.outerHeight - window.innerHeight);
  }

  _determineDragStart(position) {
    const boundingRect = (this.refs.self as HTMLElement).getBoundingClientRect();
    const { top, bottom, left, right } = boundingRect;
    return {
      x: (left + right) / 2,
      y: ((top + bottom) / 2) + this.props.toolbarHeight,
    }
  }
}
