import { CompositeDecorator } from 'draft-js'
import LinkEntity from './inline/link'
import { linkStrategy } from './strategies'
import decorateComponentWithProps from 'decorate-component-with-props'
import EditorStateUtil from '../util/editor-state-util'

const createDecorators = (getEditorState, setEditorState, getEntityDataByKey, mergeEntityData, setReadOnly) => {

  // Entity functions that apply to the specific component they're attached to
  const getOwnEntityData = (props) => {
    const { entityKey } = props;
    if (!entityKey) return null;

    const editorState = getEditorState();
    return getEntityDataByKey(editorState, entityKey);
  };

  const mergeOwnEntityData = (props, data) => {
    mergeEntityData(props.entityKey, data);
  };

  const applyOwnEntityToText = (props, entityInfo) => {
    const { children, decoratedText } = props;

    const blockKey = children[0].props.block.getKey();
    const textRangeStart = children[0].props.start;
    const textRangeEnd = textRangeStart + decoratedText.length;

    const selectionInfo = {
      blockKey,
      textRangeStart,
      textRangeEnd,
    }

    const editorState = getEditorState();
    const newEditorState = EditorStateUtil.applyEntityToText(editorState, entityInfo, selectionInfo);
    setEditorState(newEditorState);
  }

  return new CompositeDecorator([
    {
      strategy: linkStrategy,
      component: decorateComponentWithProps(LinkEntity, {
        getOwnEntityData,
        mergeOwnEntityData,
        applyOwnEntityToText,
        setReadOnly,
      }),
    },
  ]);
}

export default createDecorators;
