function wrapCustomBlockContents(customBlockHTML, classNames) {
  return ([
    `<figure class="${classNames}">`,
      '<div class="container">',
        '<div class="row">',
          customBlockHTML,
        '</div>',
      '</div>',
    '</figure>'
  ].join(''));
}

const ImageRenderer = (entityData, classNames) => {
  const { src, alt, width, href, id } = entityData;

  const aTag = href ? `<a href="${href}">` : '<a>';
  const idAttr = id ? `id="${id}"` : '';
  const ImageHTML = [
    `<figure class="${classNames}">`,
      aTag,
        `<img ${idAttr} src="${src}" alt="${alt}" width="${width}">`,
      '</a>',
    '</figure>'
  ].join('');

  return ImageHTML;
};

const TipRenderer = (entityData, classNames) => {
  const TipHTML = [
    '<div class="tip tip-label-container">',
      '+',
      '<div class="tip triangle-left"></div>',
      '<div class="tip triangle-right"></div>',
    '</div>',

    '<div class="col-xs-12 tip tip-header">',
      '<p>',
        entityData.header,
      '</p>',
    '</div>',

    '<div class="col-xs-12 tip tip-body">',
      '<p>',
        entityData.body,
      '</p>',
    '</div>',
  ].join('');

  return wrapCustomBlockContents(TipHTML, classNames);
}

const TestimonialRenderer = (entityData, classNames) => {
  const TestimonialHTML = [
    '<div class="col-xs-12 testimonial testimonial-text header-two">',
      '<p>',
        entityData.testimonial,
      '</p>',
    '</div>',

    '<div class="col-xs-6 testimonial author unstyled">',
      entityData.author,
    '</div>',

    '<div class="col-xs-6 col-xs-offset-6 testimonial school unstyled">',
      `<a href=${entityData.schoolHref} target="_blank">`,
        entityData.school,
      '</a>',
    '</div>',
  ].join('');

  return wrapCustomBlockContents(TestimonialHTML, classNames);
}

const CTARenderer = (entityData, classNames) => {
  const target = (entityData.buttonHref && entityData.buttonHref[0] == "#") ? "_self" : "_blank";
  const ctaHTML = [
    '<div class="col-xs-12 cta-header header-three">',
      '<p>',
        entityData.header,
      '</p>',
    '</div>',

    '<div class="col-xs-12 cta-subheader unstyled">',
      entityData.subHeader,
    '</div>',

    '<div class="col-xs-12 cta-button-container">',
      `<a class="button orange-button" href="${entityData.buttonHref}" target="${target}">`,
        entityData.buttonText,
      '</a>',
    '</div>',
  ].join('');

  return wrapCustomBlockContents(ctaHTML, classNames);
};

const CustomHTMLRenderer = (entityData, classNames) => {
  return [
    `<figure class="${classNames}">`,
      `${entityData.html}`,
    '</figure>'
  ].join('');
}

export const CUSTOM_COMPONENT_RENDERERS = {
  IMAGE: ImageRenderer,
  CTA: CTARenderer,
  HTML: CustomHTMLRenderer,
  TESTIMONIAL: TestimonialRenderer,
  TIP: TipRenderer,
};
