import React from 'react';
import ReactSimpleColorPicker from 'react-simple-colorpicker';

interface ColorPickerProps {
  currentColor: any
  isCollapsed: boolean
  onMouseDown: any
  toggleColorStyle: any
}

interface ColorPickerState extends ComponentState {
  alpha: number
  color: string
  hexValue: string
}

export default class ColorPicker extends React.Component<ColorPickerProps, ColorPickerState> {
  constructor(props) {
    super(props)
    this.state = {
      color: this.props.currentColor,
      hexValue: this._parseHexValue(this.props.currentColor),
      alpha: 1
    };
  }

  // HANDLERS -----

  handleColorChange(rgbStr) {
    if (rgbStr === this.state.color) { return; }
    this.setState({
      color: rgbStr,
      hexValue: this._parseHexValue(rgbStr),
      alpha: this._parseAlphaValue(rgbStr)
    }, () => this.props.toggleColorStyle(rgbStr));
  }

  handleTextChange(e) {
    e.preventDefault();
    const hexValue = this._coerceHexText(e.currentTarget.value);
    const isValidHex = this._isValidHexValue(hexValue);

    this.setState({
      hexValue,
      color: isValidHex ? hexValue : this.state.color,
    });

    if (isValidHex) { this.props.toggleColorStyle(hexValue); }
  }

  // RENDERING -----

  render() {
    const { isCollapsed } = this.props;
    return(
      <div className={ 'color-picker-container' + (isCollapsed ? ' collapsed' : '') }>
        <div className={ 'options-triangle' + (isCollapsed ? ' collapsed' : '') }></div>
        <ReactSimpleColorPicker
          onChange={ this.handleColorChange.bind(this) }
          color={ this.state.color }
          opacitySlider
        />
        { !isCollapsed ? this.renderHexInput() : null }
      </div>
    );
  }

  renderHexInput() {
    return (
      <div className="hex-input-container">
        <span>HEX:</span>
        <input
          type="text"
          className="hex-input"
          value={ this.state.hexValue || '' }
          onChange={ this.handleTextChange.bind(this) }
          maxLength={ 7 }
        />
      </div>
    );
  }

  // PRIVATE -----

  _coerceHexText(hexStr) {
    return (hexStr !== '' && hexStr[0] !== '#') ? `#${hexStr}` : hexStr;
  }

  _isValidHexValue(hexStr) {
    return /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(hexStr);
  }

  _parseHexValue(rgbStr) {
    if (rgbStr[0] === '#') return rgbStr;

    const toHex = (n) => {
      const hex = n.toString(16);
      return hex.length == 1 ? '0' + hex : hex;
    };
    const re = /([0-9]*,[0-9]*,[0-9]*)/;
    const [r, g, b] = rgbStr.match(re)[0].split(',').map(s => parseInt(s));
    return '#' + toHex(r) + toHex(g) + toHex(b);
  }

  _parseAlphaValue(rgbStr) {
    const re = /(?:,)([0-1]\.?[0-9]*)(?:\))/;
    return rgbStr.match(re)[1];
  }
}
