import React from 'react'

interface CounterProps {
  editorState: string
}

interface CounterState extends ComponentState {
  blockCount: number
  wordCount: number
}

export default class Counter extends React.Component<CounterProps, CounterState> {
  constructor(props) {
    super(props);

    this.state = {
      wordCount: 0,
      blockCount: 0,
    }
  }

  componentWillReceiveProps(nextProps) {
    const plainTextStr = nextProps.editorState.getCurrentContent().getPlainText('');

    this.setState({
      wordCount: this.getWordCount(plainTextStr),
      blockCount: this.getBlockCount(nextProps.editorState),
    });
  }

  getBlockCount(editorState) {
    const blockArray = editorState.getCurrentContent().getBlocksAsArray();
    return blockArray ? blockArray.length : 0;
  }

  getWordCount(plainTextStr) {
    const re = /\S+/g;
    const match = plainTextStr.match(re);
    return !!match ? match.length : 0;
  }

  render() {
    const { wordCount, blockCount } = this.state;
    return (
      <div className="editor-counter">
        <p className="word-count-label">Words: </p><div className="word-count">{ wordCount }</div>
        <p className="block-count-label">Blocks: </p><div className="block-count">{ blockCount }</div>
      </div>
    );
  }
}
