import React from 'react'
import { getStyleIcon } from '../util/icon-util'

interface optionsType {
  isCollapsed?: boolean
  optionalProps?: any
}

interface StyleButtonProps {
  isPressed?: boolean
  onMouseDown?: any
  options: optionsType
  styleCode: string
}

interface StyleButtonState extends ComponentState {}

export default class StyleButton extends React.Component<StyleButtonProps, StyleButtonState> {
  onMouseDown: any
  
  constructor(props) {
    super(props)

    this.onMouseDown = (e) => {
      e.preventDefault();
      this.props.onMouseDown(this.props.styleCode);
    }
  }

  shouldComponentUpdate(nextProps, nextState) {
    const changedCollapsed = nextProps.options.isCollapsed !== this.props.options.isCollapsed;
    const changedPressed = nextProps.isPressed !== this.props.isPressed;
    return changedCollapsed || changedPressed;
  }

  render() {
    const { styleCode, isPressed, options } = this.props;
    const { isCollapsed, optionalProps } = options;
    const Icon = getStyleIcon(styleCode, isPressed);

    const props = {
      className: (
        'style-button'
        + (isPressed ? ' pressed' : '')
        + (isCollapsed ? ' collapsed' : '')
        + (Icon.type === 'img' ? ' image-icon' : ' text-icon')
      ),
      onMouseDown: this.onMouseDown,
      ...optionalProps
    };
    return <div { ...props }>{ Icon }</div>;
  }
}
