import React from 'react'
import _DropdownMenu from './_dropdown-menu'
import { getOptionMenuIcon } from '../util/icon-util'

export default class StyleOptionsMenu extends _DropdownMenu {
  constructor(props) {
    super(props)
  }

  specificState() {
    return { menuStyles: {} };
  }

  // LIFECYCLE -----

  componentDidMount() {
    super.componentDidMount();
    const { menuDirection } = this.props;
    this.setState({
      menuStyles: menuDirection === 'horizontal' ? this._horizontalStyles() : {},
    });
  }

  // RENDERING -----

  render() {
    const { currentOption, defaultOption, alwaysPressed } = this.props;
    const isPressed = !!currentOption || alwaysPressed;
    const Icon = getOptionMenuIcon(currentOption || defaultOption, isPressed);
    const props = {
      ref: 'button',
      id: `${this._classPrefix()}-selector-button`,
      className: (
        'style-button style-options-menu-button'
        + (this.state.isExpanded ? ' expanded' : '')
        + (isPressed ? ' pressed' : '')
        + (Icon.type === 'img' ? ' image-icon' : ' text-icon')
      ),
      onMouseDown: this.handleMouseDown,
    };
    return (
      <div className="style-button-options-menu-container">
        <div { ...props }>{ Icon }</div>
        { this.renderMenu() }
      </div>
    );
  }

  renderMenu() {
    const { options, styleType, renderStyleButton, menuDirection } = this.props;
    const isCollapsed = !this.state.isExpanded;

    const styleButtonOptions = {
      isCollapsed,
      optionalProps: {
        onClick: () => this.setState({ isExpanded: false }),
      },
    };

    const containerClassName = (
      'style-options-menu '
      + this._classPrefix()
      + (isCollapsed ? ' collapsed ': ' ')
      + menuDirection
    );

    return (
      <div className={ containerClassName } style={ this.state.menuStyles } ref="menu">
        <div className={ 'options-triangle ' + (isCollapsed ? 'collapsed' : '') }></div>
        { options.map(styleCode => renderStyleButton(styleCode, styleType, styleButtonOptions)) }
      </div>
    );
  }

  // PRIVATE HELPERS -----

  _classPrefix() {
    const { styleType, styleCategory } = this.props;
    return `${styleType}-${styleCategory}`;
  }

  _horizontalStyles() {
    const numOptions = this.props.options.length;
    const width = numOptions * 50;
    const left = (((1 - numOptions) / 2.0) * 50) - 3;
    return { width, left };
  }
}
