import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import Admin from './../admin'
import ConfirmDelete from './confirm-delete'
import { ConfirmDeleteModalStyles } from './helpers/modal-styles'
import { fetchEntity, createEntity, updateEntity, deleteEntity } from '../actions/index'
import { withExitPrompt } from './util/with-exit-prompt'
import { Field } from './field'
import { Filepicker } from './filepicker'

interface ContentDownloadDetailsProps {
  contentCategories: any[];
  contentDownload: any;
  contentTypes: any[];
  deleteEntity: any;
  errors: string[];
  fetchEntity: any;
  funnelStages: any[];
  updateEntity: any;
  createEntity: (config: any) => Promise<any>
}

interface ContentDownloadDetailsState extends ComponentState {
  category: string;
  contentCategories: any[];
  contentDownload: any;
  contentDownloadSaved: any;
  contentTypes: any[];
  deleteModalOpen: boolean;
  errors: string[];
  funnelStages: any[];
}

class ContentDownloadDetails extends React.Component<ContentDownloadDetailsProps, ContentDownloadDetailsState> {

  constructor(props) {
    super(props);
    this.state = {
      category: 'content_downloads',
      contentCategories: [],
      contentDownload: {},
      contentDownloadSaved: {},
      contentTypes: [],
      errors: [],
      fetching: true,
      funnelStages: [],
      deleteModalOpen: false
    };
  }

  componentDidMount() {
    let pathDirectories = window.location.pathname.split('/');
    this.props.fetchEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "content_downloads",
      entityName: "contentDownload"
    }).then(() => {
      this.setState({
        fetching: false,
        contentCategories: this.props.contentCategories,
        contentDownload: this.props.contentDownload,
        contentDownloadSaved: cloneDeep(this.props.contentDownload),
        contentTypes: this.props.contentTypes,
        funnelStages: this.props.funnelStages
      }, () => {
        Admin.setUpNiceSelect({ selector: 'select', func: Admin.changeField.bind(this, this.changeFieldArgs()) });
      });
    });
  }

  checkForChanges() {
    return !isEqual(this.state.contentDownload, this.state.contentDownloadSaved);
  }

  changeFieldArgs() {
    return {
      thing: "contentDownload",
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this)
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }

  clickSave() {
    this.setState({
      fetching: true
    });
    let pathDirectories = window.location.pathname.split('/');
    this.props.updateEntity({
      id: pathDirectories[pathDirectories.length - 1],
      directory: "content_downloads",
      entityName: "contentDownload",
      entity: this.state.contentDownload
    }).then(() => {
      this.setState({
        fetching: false,
        contentDownload: this.props.contentDownload,
        contentDownloadSaved: cloneDeep(this.props.contentDownload),
        changesToSave: false
      });
    }, () => {
      this.setState({
        fetching: false,
        errors: this.props.errors
      });
    });
  }

  render() {
    return (
      <div id="content-download-details" className="component details-component">
        <h1 className="blog-post-details-header">Content Download details</h1>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 9, entity: "contentDownload", property: "name" }) }
            { Admin.renderDropDown.bind(this)({
              columnWidth: 3,
              entity: "contentDownload",
              property: "contentCategoryId",
              columnHeader: "Category",
              optionLabel: "name"
            }) }
          </div>
          <div className="row">
            { Admin.renderDropDown.bind(this)({
              columnWidth: 2,
              entity: "contentDownload",
              property: "contentType",
              optionValue: "value",
              optionLabel: "value"
            }) }
            { this.renderContentTypeField(4) }
            { Admin.renderField.bind(this)({ columnWidth: 3, entity: "contentDownload", property: "landingPageEventName", columnHeader: "Event Name" }) }
            { Admin.renderDropDown.bind(this)({
              columnWidth: 2,
              entity: "contentDownload",
              property: "contentFunnelStageId",
              columnHeader: "Funnel Stage"
            }) }
            { Admin.renderField.bind(this)({ columnWidth: 2, entity: "contentDownload", property: "contentScore", columnHeader: "Score" }) }
          </div>
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: "contentDownload", property: "customRedirect" }) }
          </div>
          <hr />
          <h3>Landing and Thank You Pages</h3>
          <div className="row">
            { Admin.renderBooleanDropDown.bind(this)({
              entity: "contentDownload",
              property: "active"
            }) }
            <div className="col-xs-3">
              <h2>URL</h2>
              <div className="prefix">/get/</div><input id="landingPageSlug" className={ "with-prefix admin-form-text-field " + Admin.errorClass(this.state.errors, Admin.errors.landingPageSlug) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.contentDownload.landingPageSlug || "" } data-field="landingPageSlug" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.landingPageSlug) }
            </div>
            { Admin.renderField.bind(this)({ columnWidth: 7, entity: "contentDownload", property: "titleText" }) }
          </div>
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: "contentDownload", property: "description", columnHeader: "Meta Description" }) }
          </div>
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: "contentDownload", property: "landingPageHeader", columnHeader: "Header" }) }
          </div>
          <div className="row">
            { Admin.renderImageField.bind(this)({
              columnWidth: 4,
              entity: "contentDownload",
              property: "landingPageBackgroundImageId",
              imageUrlProperty: "landingPageBackgroundImageUrl",
              columnHeader: "Background Image",
              createImageRecord: true })
            }
            { Admin.renderImageField.bind(this)({
              columnWidth: 4,
              entity: "contentDownload",
              property: "landingPageLargeImageId",
              imageUrlProperty: "landingPageLargeImageUrl",
              columnHeader: "Large Image",
              createImageRecord: true })
            }
            { Admin.renderField.bind(this)({ columnWidth: 4, entity: "contentDownload", property: "landingPageButtonText", columnHeader: "Button Text" }) }
          </div>
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 12, entity: "contentDownload", property: "landingPageBulletHeader", columnHeader: "Bullet Header" }) }
          </div>
          <div className="row">
            { this.renderBulletPointsField(12) }
          </div>
          { this.renderPreviewButton(`/admin/content_downloads/${this.state.contentDownload.id}/preview/landing_page`, 'Preview Landing Page') }
          <hr />
          <h3 className="with-field-info">Hero Form</h3>
          <p className="field-info under-h3">Choose how the content download will look if selected for the hero on the <a href="/blog" target="_blank">blog homepage</a> or <a href="/resources" target="_blank">resources page</a></p>
          <div className="row">
            { Admin.renderField.bind(this)({ columnWidth: 6, entity: "contentDownload", property: "heroFormHeader" }) }
            { Admin.renderImageField.bind(this)({
              columnWidth: 3,
              entity: "contentDownload",
              property: "heroFormBackgroundImageId",
              imageUrlProperty: "heroFormBackgroundImageUrl",
              columnHeader: "Hero Form Background Image",
              createImageRecord: true })
            }
            { Admin.renderField.bind(this)({ columnWidth: 3, entity: "contentDownload", property: "heroFormButtonText" }) }
          </div>
          { this.renderPreviewButton(`/admin/content_downloads/${this.state.contentDownload.id}/preview/hero_form`, 'Preview Hero Form') }
          <hr />
          <h3 className="with-field-info">Preview Card</h3>
          <p className="field-info under-h3">Choose how the content download will look if selected to be highlighted on the <a href="/resources" target="_blank">resources page</a></p>
          <div className="row">
            { Admin.renderImageField.bind(this)({
              columnWidth: 6,
              entity: "contentDownload",
              property: "previewCardImageId",
              imageUrlProperty: "previewCardImageUrl",
              columnHeader: "Preview Card Image",
              createImageRecord: true })
            }
          </div>
          { this.renderPreviewButton(`/admin/content_downloads/${this.state.contentDownload.id}/preview/preview_card`, 'Preview Card') }
          <hr />
          { this.renderBottomButtons() }
        </div>
        { this.renderModal() }
      </div>
    );
  }

  onFileChange = ({ key, url }) => {
    if (url) {
      // upload our new file
      this.props.createEntity({
        directory: 'uploads',
        entityName: 'upload',
        entity: {
          uploadType: 'file',
          category: '(none)',
          url,
          s3_url: key,
          uploadedBy: Admin.currentUser.id
        }
      }).then(() => {
        const newContentDownload = {
          ...this.state.contentDownload,
          // @ts-ignore -- state type is a mess
          uploadId: this.props.upload.id,
          uploadUrl: url
        }
        this.setState({
          contentDownload: newContentDownload
        }, () => {
          this.setState({
            changesToSave: this.checkForChanges()
          })
        })
      })
    }
  }

  renderContentTypeField(size) {
    switch (this.state.contentDownload.contentType) {
      case 'file':
        return (
          <Field columnWidth={size} className="file-field" columnHeader="File">
            <Filepicker uploadUrl={this.state.contentDownload.uploadUrl} onChange={this.onFileChange} />
          </Field>
        )
        /* return Admin.renderFileField.call(this, { columnWidth: size, entity: "contentDownload", property: "uploadId", fileUrlProperty: "uploadUrl", columnHeader: "File", createImageRecord: true }); */
      case 'external_link':
        return Admin.renderField.call(this, { columnWidth: size, entity: "contentDownload", property: "externalLink" });
      default:
        return null;
    }
  }

  renderBulletPointsField(size) {
    const props = {
      rows: 5,
      id: 'landingPageBulletPoints',
      className: this.fieldClassName('markdown', 'landingPageBulletPoints'),
      onChange: Admin.changeField.bind(this, this.changeFieldArgs()),
      value: this.state.contentDownload.landingPageBulletPoints || '',
      'data-field': 'landingPageBulletPoints'
    }
    return (
      <div className={ `col-xs-${size}` }>
        <h2>Bullet Points</h2>
        <textarea { ...props } ></textarea>
        { Admin.renderFieldError(this.state.errors, Admin.errors['landingPageBulletPoints'] || []) }
      </div>
    );
  }

  renderPreviewButton(redirect, buttonText = 'Preview') {
    const props = {
      className: 'preview-button light-orange-button ' + Admin.renderDisabledButtonClass(this.state.fetching),
      href: redirect,
      target: '_blank'
    };
    return (
      <div className="row">
        <div className="col-xs-12">
          <a { ...props }>{ buttonText }</a>
        </div>
      </div>
    );
  }

  renderDeleteButton() {
    return (
      <a className={ "delete-button" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
        Delete
      </a>
    );
  }

  renderModal() {
    return (
      <Modal isOpen={ this.state.deleteModalOpen } onRequestClose={ Admin.closeDeleteModal.bind(this) } contentLabel="Modal" style={ ConfirmDeleteModalStyles }>
        <ConfirmDelete entityName="content download" entityId={ this.state.contentDownload.id } entityDirectory="content_downloads" closeModal={ Admin.closeDeleteModal.bind(this) } closeAndFetch={ Admin.closeDeleteModalAndFetch.bind(this) } delete={ Admin.deleteEntity.bind(this) } />
      </Modal>
    );
  }

  renderBottomButtons() {
    if (this.state.changesToSave) {
      var buttonText = "Save";
    } else {
      var buttonText = this.state.justSaved ? "Saved" : "No Changes";
    }
    return(
      <div>
        <a id="update-button" className={ "orange-button standard-button-width btn" + Admin.renderDisabledButtonClass(this.state.fetching || !this.state.changesToSave) } onClick={ this.clickSave.bind(this) }>
          { buttonText }
        </a>
        <a id="delete" className={ "delete-button " + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "deleteModalOpen", true) }>
          Delete
        </a>
        <div className="page-links-container">
          <a className={ "light-orange-button page-links-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) }>
            Page links
          </a>
          <ul className="page-links-inner-menu">
            <div></div>
            <li><a href={ `/get/${this.state.contentDownload.landingPageSlug}` } target="_blank" rel="noopener noreferrer">Landing</a></li>
            <li><a href={ `/get/${this.state.contentDownload.landingPageSlug}/thank-you` } target="_blank" rel="noopener noreferrer">Thank You</a></li>
          </ul>
        </div>
      </div>
    );
  }

  fieldClassName(type, property) {
    type = (type === 'boolean-dropdown') ? 'dropdown' : type;
    const errorClass = Admin.errorClass(this.state.errors, Admin.errors[property] || []);
    return `admin-form-${type}-field ${errorClass}`;
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity, createEntity, updateEntity, deleteEntity }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(withExitPrompt(ContentDownloadDetails));
