import React from 'react'
import Admin from './../admin'
import isEqual from 'lodash/isEqual'
import cloneDeep from 'lodash/cloneDeep'
import { withExitPrompt } from './util/with-exit-prompt';

interface FieldIndexItemProps {
  clickTrash: any
  errors?: any
  fetching?: boolean
  field: any
  index: number
  notifySaveButton: any
  parentTemplates: any
  rearrangeFields: any
  renderHandle: any
}

interface FieldIndexItemState extends ComponentState {
  field: any
  fieldSaved: any
}

class FieldIndexItem extends React.Component<FieldIndexItemProps, FieldIndexItemState> {

  constructor(props) {
    super(props);
    this.state = {
      field: {
        id: this.props.field.id || "",
        frontEndId: this.props.field.frontEndId || "",
        name: this.props.field.name || "",
        mandatory: this.props.field.mandatory || "no",
        fieldType: this.props.field.fieldType || "text",
        charLimit: this.props.field.charLimit || "0",
        menuOptions: this.props.field.menuOptions || [],
        objectType: this.props.field.objectType || "",
        description: this.props.field.description || ""
      },
      fieldSaved: {
        id: this.props.field.id || "",
        frontEndId: this.props.field.frontEndId || "",
        name: this.props.field.name || "",
        mandatory: this.props.field.mandatory || "no",
        fieldType: this.props.field.fieldType || "text",
        charLimit: this.props.field.charLimit || "0",
        menuOptions: this.props.field.menuOptions || [],
        objectType: this.props.field.objectType || "",
        description: this.props.field.description || ""
      },
      errors: []
    }
  }

  clickTrash() {
    this.props.clickTrash(this.props.index);
  }

  componentDidMount() {
    $('.field').draggable({
      cursor: '-webkit-grabbing',
      handle: '.handle',
      helper: function() { return '<div></div>'; },
      stop: this.dragEndHandler
    });
    $('.drop-zone').droppable({
      accept: Admin.canIDrop,
      tolerance: 'pointer',
      over: this.dragOverHandler.bind(this),
      out: this.dragOutHandler.bind(this),
      drop: this.dropHandler.bind(this)
    });
    Admin.setUpNiceSelect({
      selector: '#field-' + this.props.index + ' select',
      func: Admin.changeField.bind(this, this.changeFieldArgs())
    });
  }

  componentWillReceiveProps(nextProps) {
    let state: any = {
      field: nextProps.field,
      errors: nextProps.errors
    };
    if (!nextProps.fetching && nextProps.errors.length === 0) {
      // the field was only saved if the server response came back with no errors
      state.fieldSaved = cloneDeep(nextProps.field);
    }
    this.setState(state);
    Admin.resetNiceSelect({
      selector: '#field-' + this.props.index + ' select',
      func: Admin.changeField.bind(this, this.changeFieldArgs())
    });
  }

  mouseDownHandler(e) {
    $('.handle, input, .trash-can, .nice-select').addClass('grabbing');
    const field = e.target.parentElement.parentElement.parentElement;
    field.classList.add('highlight');
    var children = field.parentElement.children;
    var bottomDropZone = children[children.length - 1];
    bottomDropZone.classList.add('smaller');
    if (children.length == 3) {
      var topDropZone = children[0];
    } else {
      var fieldIndex = +field.getAttribute('id').split('-')[1];
      var fields = field.parentElement.parentElement.children;
      var topField = fields[fieldIndex];
      var topDropZone = topField.children[topField.children.length - 1];
    }
    topDropZone.classList.add('smaller');
  }

  mouseUpHandler(e) {
    $('.handle, input, .trash-can, .nice-select').removeClass('grabbing');
    e.target.parentElement.parentElement.parentElement.classList.remove('highlight');
    $('.smaller').removeClass('smaller');
  }

  dragOverHandler(e) {
    e.target.classList.add('highlight');
  }

  dragOutHandler(e) {
    e.target.classList.remove('highlight');
  }

  dragEndHandler(e) {
    $('.handle, input, .trash-can, .nice-select').removeClass('grabbing');
    $('.smaller').removeClass('smaller');
    $('.field.highlight').removeClass('highlight');
  }

  dropHandler(e, ui) {
    var draggedIndex = ui.draggable.attr('id').split('-')[1];
    var dropZoneIndex = e.target.dataset.index;
    $('.highlight').removeClass('highlight');
    this.props.rearrangeFields(draggedIndex, dropZoneIndex);
  }

  clickAddOption(e) {
    var field = this.state.field;
    var menuOptions = field.menuOptions.slice(0);
    var inputField = e.target.parentElement.children[1];
    var value = inputField.value;
    if (inputField.value.trim() !== '') {
      menuOptions.push({
        value: value,
        order: menuOptions.length
      });
      field.menuOptions = menuOptions;
      var fakeEvent = {
        target: {
          dataset: {
            field: field
          }
        }
      };
      Admin.changeField.call(this, this.changeFieldArgs(), fakeEvent);
      inputField.value = '';
    }
  }

  clickDeleteOption(e) {
    var row = e.target.dataset.index;
    var newOptions = [];
    for (var i = 0; i < this.state.field.menuOptions.length; i++) {
      if (i != row) {
        this.state.field.menuOptions[i].order = newOptions.length;
        newOptions.push(this.state.field.menuOptions[i]);
      }
    }
    this.state.field.menuOptions = newOptions;
    var fakeEvent = {
      target: {
        dataset: {
          field: this.state.field
        }
      }
    };
    Admin.changeField.call(this, this.changeFieldArgs(), fakeEvent);
  }

  checkForChanges() {
    return !isEqual(this.state.field, this.state.fieldSaved);
  }

  changeFieldArgs() {
    return {
      thing: 'field',
      errorsArray: this.state.errors,
      changesFunction: this.checkForChanges.bind(this),
      callback: this.notifySaveButton.bind(this)
    }
  }

  notifySaveButton() {
    this.props.notifySaveButton(this.props.index, this.state.changesToSave);
  }

  reportFullStateToParent() {
    return this.state.field;
  }

  render() {
    return(
      <div>
        { this.renderTopDropZone() }
        <div id={ "field-" + this.props.index } className="field" data-index={ this.props.index }>
          <div className="row first-field-row">
            <div className="col-xs-3">
              <h2>Field name</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.name) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.field.name || "" } data-field="name" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.name) }
            </div>
            <div className="col-xs-2">
              <h2>Mandatory&#63;</h2>
              <select onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } data-field="mandatory" value={ this.state.field.mandatory }>
                <option value="no">No</option>
                <option value="yes">Yes</option>
              </select>
            </div>
            <div className="col-xs-2 select-scroll-6">
              <h2>Type</h2>
              <select onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } data-field="fieldType" value={ this.state.field.fieldType }>
                <option value="text">Text</option>
                <option value="markdown">Markdown</option>
                { this.renderHeaderOption() }
                <option value="image">Image</option>
                <option value="number">Number</option>
                <option value="dropdown">Drop Down</option>
                <option value="color">Color</option>
                <option value="object">Object</option>
                <option value="file">File</option>
                <option value="content_download">Content Download</option>
                <option value="event_series">Event Series</option>
                <option value="transcription">Transcript</option>
                { this.renderPagePartialOption() }
                { this.renderWebinarOption() }
              </select>
            </div>
            { this.renderCustomColumn() }
            <div className="col-xs-2">
              { this.renderHandle() }
              <img className="trash-can" src={ window.Images.trashCan } onClick={ this.clickTrash.bind(this) } />
            </div>
          </div>
          <div className="row">
            <div className="col-xs-7">
              <h2>Description</h2>
              <input className={ Admin.errorClass(this.state.errors, Admin.errors.description) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.field.description || "" } data-field="description" />
              { Admin.renderFieldError(this.state.errors, Admin.errors.description) }
            </div>
          </div>
        </div>
        <div className="row drop-zone" data-index={ this.props.index }>
        </div>
      </div>
    );
  }

  renderHeaderOption() {
    if (window.location.href.indexOf('/partials/') > -1) {
      return(
        <option value="header">Header</option>
      );
    } else {
      return null;
    }
  }

  renderPagePartialOption() {
    if (window.location.href.indexOf('/partials/') > -1) {
      return(
        <option value="page-partial">Page Partial</option>
      );
    } else {
      return null;
    }
  }

  renderWebinarOption() {
    if (window.location.href.indexOf('/partials/') > -1) {
      return(
        <option value="webinar">Webinar</option>
      );
    } else {
      return null;
    }
  }

  renderTopDropZone() {
    if (this.props.index == 0) {
      return(
        <div className="row drop-zone" data-index="-1">
        </div>
      );
    } else {
      return null;
    }
  }

  renderHandle() {
    if (this.props.renderHandle) {
      return(
        <img className="handle" src={ window.Images.handle } onMouseDown={ this.mouseDownHandler } onMouseUp={ this.mouseUpHandler } data-index={ this.props.index } />
      );
    } else {
      return null;
    }
  }

  renderCustomColumn() {
    if (['text', 'header', 'markdown'].indexOf(this.state.field.fieldType) > -1) {
      return(
        <div className="col-xs-3">
          <h2>Character limit</h2>
          <input className={ Admin.errorClass(this.state.errors, ["Limit can't be zero"]) } onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } value={ this.state.field.charLimit || "" } data-field="charLimit" />
          { Admin.renderFieldError(this.state.errors, ["Limit can't be zero"]) }
        </div>
      );
    } else if (this.state.field.fieldType == "dropdown") {
      return(
        <div className="col-xs-3">
          <h2>Options</h2>
          <div className="options-list">
            { this.state.field.menuOptions.map((menuOption, index) => {
              return(
                <div key={ index } className="menu-option">
                  { menuOption.value }
                  <img src={ window.Images.smallX } onClick={ this.clickDeleteOption.bind(this) } data-index={ index } data-field={ this.props.field.id } />
                </div>
              );
            }) }
            <div className="clearfix">
              <a className="add-option-button" onClick={ this.clickAddOption.bind(this) }>Add option</a>
              <input className="new-text" type="text" />
            </div>
          </div>
        </div>
      );
    } else if (this.state.field.fieldType == "object") {
      return(
        <div className="col-xs-3 select-scroll-6">
          <h2>Object Type</h2>
          <select className="object-dropdown" onChange={ Admin.changeField.bind(this, this.changeFieldArgs()) } data-field="objectType" value={ this.state.field.objectType }>
            { this.props.parentTemplates().map((template, index) => {
              return(
                <option key={ index } value={ template.name }>{ template.name }</option>
              );
            }) }
          </select>
        </div>
      );
    } else {
      return(
        <div className="col-xs-3">
        </div>
      );
    }
  }

  componentDidUpdate() {
    if ($('#field-' + this.props.index + ' select.object-dropdown').length > 0 && $('#field-' + this.props.index + ' div.nice-select.object-dropdown').length === 0) {
      Admin.setUpNiceSelect({
        selector: '#field-' + this.props.index + ' select.object-dropdown',
        func: Admin.changeField.bind(this, this.changeFieldArgs())
      });
    } else if ($('#field-' + this.props.index + ' select.object-dropdown').length === 0 && $('#field-' + this.props.index + ' div.nice-select.object-dropdown').length > 0) {
      $('div.object-dropdown').remove();
    }
    $('.drop-zone').droppable({
      accept: Admin.canIDrop,
      tolerance: 'pointer',
      over: this.dragOverHandler.bind(this),
      out: this.dragOutHandler.bind(this),
      drop: this.dropHandler.bind(this)
    });
  }
}
export default withExitPrompt(FieldIndexItem)
