import React from 'react';
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal';
import NewEntity from './new-entity';
import Admin from './../admin'
import { fetchEntities } from '../actions/index'
import sortBy from 'lodash/sortBy';
import truncate from 'lodash/truncate';

const NewPageModalStyles = {
  overlay: {
    background: 'rgba(0, 0, 0, 0.50)'
  },
  content: {
    background: 'white',
    padding: 0,
    margin: 'auto',
    maxWidth: 1000,
    height: 359
  }
};

interface PagesIndexProps {
  cloneId?: number;
  fetchEntities: any;
  pages: any[];
}

interface PagesIndexState {
  fetching: boolean;
  modalOpen: boolean;
  pages: any[];
  searchProperty: string;
  searchText: string;
  tab: string;
}

class PagesIndex extends React.Component<PagesIndexProps, PagesIndexState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      pages: [],
      modalOpen: false,
      searchText: "",
      searchProperty: "name",
      tab: 'General'
    };
  }

  componentDidMount() {
    this.props.fetchEntities("pages", "pages").then(() => {
      this.setState({
        fetching: false,
        pages: this.props.pages
      });
    });
  }

  clickNew() {
    this.setState({ modalOpen: true });
  }

  closeModal() {
    this.setState({ modalOpen: false });
  }

  render() {
    var filteredPages = Admin.filterSearchText(this.state.pages.filter((page) => {
      switch (this.state.tab) {
        case 'General':
          return page.active && !page.url.startsWith('/welcome/')
        case 'Affiliate Pages':
          return page.active && page.url.startsWith('/welcome/')
        case 'Inactive':
          return page.active === false
        default:
          return false;
      }
    }), this.state.searchText, this.state.searchProperty);
    return(
      <div id="pages-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Pages</h1>
          { Admin.renderTopTabs.call(this, "pages", ["General", "Affiliate Pages", "Inactive"]) }
          <a className={ "orange-button float-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>Add Page</a>
          <input className="search-box margin" onChange={ Admin.changeSearchText.bind(this) } value={ this.state.searchText } />
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <table className='admin-table contains-links'>
            <thead>
              <tr>
                <th><div className={ Admin.sortClass.bind(this)("name") } onClick={ Admin.clickHeader.bind(this, "name") }>Name</div></th>
                <th><div className={ Admin.sortClass.bind(this)("url") } onClick={ Admin.clickHeader.bind(this, "url") }>Url</div></th>
                <th><div className={ Admin.sortClass.bind(this)("createdBy") } onClick={ Admin.clickHeader.bind(this, "createdBy") }>Created By</div></th>
              </tr>
            </thead>
            <tbody>
              <tr><td></td><td></td><td></td></tr>
              { sortBy(filteredPages, [Admin.commonSort.bind(this)]).map((page, index) => {
                return(
                  <tr key={ index }>
                    <td className="name-column">
                      <a href={ `/admin/pages/${page.id}` }>
                        { truncate(page.name, { length: 53 }) }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/pages/${page.id}` }>
                        { truncate(page.url, { length: 43 }) }
                      </a>
                    </td>
                    <td>
                      <a href={ `/admin/pages/${page.id}` }>
                        { page.createdBy }
                      </a>
                    </td>
                  </tr>
                );
              }) }
            </tbody>
          </table>
        </div>
        <Modal isOpen={ this.state.modalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ NewPageModalStyles }>
          <NewEntity entityName="page" initialEntity={ { name: "", url: "/", titleText: "", creatorId: Admin.currentUser.id } } redirect={ true } />
        </Modal>
      </div>
    );
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return {
    pages: reducers.standardReducer.pages
  };
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(PagesIndex);
