import React from 'react';
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal';
import Admin from './../admin'
import sortBy from 'lodash/sortBy';
import truncate from "lodash/truncate";
import NewEntity from './new-entity';
import { fetchEntities } from '../actions/index'

const NewRedirectModalStyles = {
  overlay: {
    background: 'rgba(0, 0, 0, 0.50)'
  },
  content: {
    background: 'white',
    padding: 0,
    margin: 'auto',
    maxWidth: 1000,
    height: 240
  }
};

const warningTooltip = "This redirect isn't doing anything, because a page or blog post with this url is active.";

interface RedirectsIndexProps {
  fetchEntities: any;
  redirects: any[];
}

interface RedirectsIndexState extends ComponentState {
  modalOpen: boolean;
  redirects: any[];
}

class RedirectsIndex extends React.Component<RedirectsIndexProps, RedirectsIndexState> {

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      redirects: [],
      modalOpen: false,
      searchText: "",
      searchProperty: "url"
    };
  }

  componentDidMount() {
    this.props.fetchEntities("redirects").then(() => {
      this.setState({
        fetching: false,
        redirects: this.props.redirects
      });
    });
  }

  updateIndex(entities) {
    this.setState({
      modalOpen: false,
      redirects: entities
    });
  }

  redirect(e) {
    var id = e.target.parentElement.dataset.id;
    window.location.pathname = "admin/redirects/" + id;
  }

  clickNew() {
    this.setState({ modalOpen: true });
  }

  closeModal() {
    this.setState({ modalOpen: false });
  }

  render() {
    var filteredRedirects = Admin.filterSearchText(this.state.redirects, this.state.searchText, this.state.searchProperty);
    return(
      <div id="redirects-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Redirects</h1>
          <a className={ "orange-button float-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>Add Redirect</a>
          <input className="search-box margin" onChange={ Admin.changeSearchText.bind(this) } value={ this.state.searchText } />
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <table className='admin-table'>
            <thead>
              <tr>
                <th></th>
                <th><div className={ Admin.sortClass.bind(this)("url") } onClick={ Admin.clickHeader.bind(this, "url") }>Redirect From</div></th>
                <th><div className={ Admin.sortClass.bind(this)("redirect") } onClick={ Admin.clickHeader.bind(this, "redirect") }>Redirect To</div></th>
                <th><div className={ Admin.sortClass.bind(this)("active") } onClick={ Admin.clickHeader.bind(this, "active") }>Active</div></th>
              </tr>
            </thead>
            <tbody>
              <tr><td></td><td></td><td></td><td></td></tr>
              { sortBy(filteredRedirects, [Admin.commonSort.bind(this)]).map((redirect, index) => {
                return(
                  <tr key={ index } onClick={ this.redirect.bind(this) } data-id={ redirect.id }>
                    <td>
                      { this.renderImage(redirect) }
                    </td>
                    <td>
                      { truncate(redirect.url, { length: 53 }) }
                    </td>
                    <td>
                      { truncate(redirect.redirect, { length: 53 }) }
                    </td>
                    <td>
                      { redirect.active ? "Yes" : "No" }
                    </td>
                  </tr>
                );
              }) }
            </tbody>
          </table>
        </div>
        <Modal isOpen={ this.state.modalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ NewRedirectModalStyles }>
          <NewEntity entityName="redirect" initialEntity={ { url: "/", redirect: "" } } callback={ this.updateIndex.bind(this) } />
        </Modal>
      </div>
    );
  }

  renderImage(redirect) {
    if (redirect.active && redirect.ineffective) {
      return <img data-toggle="tooltip" title={ warningTooltip } src={ window.Images.warning } />;
    } else {
      return null;
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
    $('#redirects-index [data-toggle="tooltip"]').tooltip();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(RedirectsIndex);
