import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Admin from './../admin'
import { fetchEntities } from '../actions/index'

interface ReusableImagesProps {
  closeModal: any;
  entityName?: string;
  fetchEntities: any;
  fieldId?: number;
  instanceId?: number;
  receiveReusableImageFunction: any;
  uploads: any[];
}

interface ReusableImagesState extends ComponentState {
  fieldId: number;
  images: any[];
  instanceId: number;
}

class ReusableImages extends React.Component<ReusableImagesProps, ReusableImagesState> {

  constructor(props) {
    super(props);
    this.state = {
      images: [],
      instanceId: this.props.instanceId,
      fieldId: this.props.fieldId,
      fetching: true
    };
  }

  componentDidMount() {
    this.props.fetchEntities({
      directory: 'uploads/reusable_index'
    }).then(() => {
      this.setState({
        fetching: false,
        images: this.props.uploads
      });
    });
  }

  setImage(index) {
    if (this.props.receiveReusableImageFunction) {
      this.props.receiveReusableImageFunction({
        uploadId: this.state.images[index].id,
        url: this.state.images[index].url,
        entityName: this.props.entityName
      });
    }
    this.props.closeModal(this.state.images[index].id, this.state.images[index].url);
  }

  render() {
    return (
      <div className={ "admin-modal reusable-images-container" + (this.state.fetching ? " reusable-images-spinner" : "") }>
        <div className="component">
          <div className="white-box">
            { Admin.renderSpinner(this.state.fetching) }
            { Admin.renderGrayedOut(this.state.fetching) }
            { this.state.images.map((imageHash, index) => {
              return(
                <img onClick={ this.setImage.bind(this, index) } key={ index } src={ imageHash.url } />
              );
            }) }
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(ReusableImages);
