import React from "react"
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from "react-modal"
import Admin from "./../admin"
import { fetchEntities } from '../actions/index'
import ScheduledTaskNew from "./scheduled-task-new"
import orderBy from "lodash/orderBy";
import upperFirst from "lodash/upperFirst";
import truncate from "lodash/truncate";

const NewTaskModalStyles = {
  overlay: {
    background: "rgba(0, 0, 0, 0.50)"
  },
  content: {
    background: "white",
    padding: 0,
    margin: "auto",
    maxWidth: 1000,
    height: 260
  }
};

interface ScheduledTasksIndexProps {
  fetchEntities: any;
  scheduledTasks: any[];
}

interface ScheduledTasksIndexState extends ComponentState {
  tab: string;
  modalOpen: boolean;
  scheduledTasks: any[];
  pages: any[];
  redirects: any[];
  blogPosts: any[];
  banners: any[];
}

class ScheduledTasksIndex extends React.Component<ScheduledTasksIndexProps, ScheduledTasksIndexState> {
  scheduledTasksListener: any;

  constructor(props) {
    super(props);
    this.state = {
      fetching: true,
      tab: "Pending",
      scheduledTasks: [],
      pages: [],
      redirects: [],
      blogPosts: [],
      banners: [],
      modalOpen: false,
      searchText: "",
      searchProperty: "timeUnix"
    };
  }

  componentDidMount() {
    this.props.fetchEntities("scheduled_tasks").then(() => {
      this.setState({
        fetching: false,
        scheduledTasks: this.props.scheduledTasks
      });
    });
  }

  clickRow(e) {
    if (this.state.tab === "Pending") {
      let id = e.target.parentElement.dataset.id;
      window.location.pathname = `admin/scheduled_tasks/${id}`;
    }
  }

  closeModal() {
    this.setState({ modalOpen: false });
  }

  updateIndex(scheduledTasks) {
    this.setState({
      scheduledTasks,
      modalOpen: false
    })
  }

  render() {
    let filteredScheduledTasks = Admin.filterSearchText(this.state.scheduledTasks.filter((task) => {
      switch (this.state.tab) {
        case "Pending":
          return task.status === "pending";
        case "Successful":
          return task.status === "successful";
        case "Failed":
          return task.status === "failed";
        default:
          return false;
      }
    }), this.state.searchText, this.state.searchProperty);
    return(
      <div id="scheduledTasks-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Scheduled Tasks</h1>
          { Admin.renderTopTabs.call(this, "scheduledTasks", ["Pending", "Successful", "Failed"]) }
          <a className={ "orange-button float-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ Admin.changeState.bind(this, "modalOpen", true) }>Add Scheduled Task</a>
          <input className="search-box margin" onChange={ Admin.changeSearchText.bind(this) } value={ this.state.searchText } />
        </div>
        <div className="white-box">
          { Admin.renderSpinner(this.state.fetching) }
          { Admin.renderGrayedOut(this.state.fetching) }
          <table className={ `admin-table${this.state.tab === "Pending" ? "" : " no-hover"}` }>
            <thead>
              <tr>
                <th><div className={ Admin.sortClass.bind(this)("timeUnix") } onClick={ Admin.clickHeader.bind(this, "timeUnix") }>Time</div></th>
                <th><div className={ Admin.sortClass.bind(this)("schedulableType") } onClick={ Admin.clickHeader.bind(this, "schedulableType") }>Type</div></th>
                <th><div className={ Admin.sortClass.bind(this)("title") } onClick={ Admin.clickHeader.bind(this, "title") }>Name</div></th>
                <th><div className={ Admin.sortClass.bind(this)("action") } onClick={ Admin.clickHeader.bind(this, "action") }>Action</div></th>
              </tr>
            </thead>
            <tbody>
              <tr><td></td><td></td><td></td><td></td></tr>
              { orderBy(filteredScheduledTasks, [Admin.commonSort.bind(this)], [this.state.tab == "Pending" ? "asc" : "desc"]).map(function(scheduledTask, index) {
                return(
                  <tr key={ index } onClick={ this.clickRow.bind(this) } data-id={ scheduledTask.id }>
                    <td>
                      { scheduledTask.timeParsed }
                    </td>
                    <td>
                      { scheduledTask.schedulableType }
                    </td>
                    <td>
                      { truncate(scheduledTask.title, { length: 68 }) }
                    </td>
                    <td>
                      { upperFirst(scheduledTask.action) }
                    </td>
                  </tr>
                );
              }.bind(this)) }
            </tbody>
          </table>
        </div>
        <Modal isOpen={ this.state.modalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ NewTaskModalStyles }>
          <ScheduledTaskNew callback={ this.updateIndex.bind(this) } />
        </Modal>
      </div>
    );
  }

  componentDidUpdate() {
    $(".match-height-layout").matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(ScheduledTasksIndex);
