import React from 'react'
import { connect } from 'react-redux'
import { bindActionCreators } from 'redux'
import Modal from 'react-modal'
import Admin from './../admin'
import { fetchEntities } from '../actions/index'
import NewEntity from './new-entity'

const NewTemplateModalStyles = {
  overlay: {
    background: 'rgba(0, 0, 0, 0.50)'
  },
  content: {
    background: 'white',
    padding: 0,
    margin: 'auto',
    maxWidth: 600,
    height: 240
  }
};

interface TemplateType {
  id: number;
  imageUrl: string;
  name: string;
  plural: string;
}

interface TemplatesIndexProps {
  fetchEntities: any
  templates: TemplateType[]
}

interface TemplatesIndexState extends ComponentState {
  modalOpen: boolean
  templates: TemplateType[]
}

class TemplatesIndex extends React.Component<TemplatesIndexProps, TemplatesIndexState> {

  constructor(props) {
    super(props);
    this.state = {
      templates: [],
      modalOpen: false
    };
  }

  componentDidMount() {
    this.props.fetchEntities("templates").then(() => {
      this.setState({
        fetching: false,
        templates: this.props.templates
      });
    });
  }

  clickTemplate(id) {
    if (window.location.pathname == "/admin/templates") {
      window.location.pathname = "admin/templates/" + id;
    } else if (window.location.pathname == "/admin/objects") {
      window.location.pathname = "admin/templates/" + id + "/all";
    }
  }

  clickNew() {
    this.setState({ modalOpen: true });
  }

  closeModal() {
    this.setState({ modalOpen: false });
  }

  render() {
    return(
      <div id="templates-index" className="component">
        { this.renderHeader() }
        { this.state.templates.map((template, index) => {
          return(
            <div key={ index } className="template-square" onClick={ this.clickTemplate.bind(this, template.id) } >
              <img src={ template.imageUrl } /><br />
              { window.location.pathname == "/admin/templates" ? template.name : template.plural }
            </div>
          );
        }) }
        <Modal isOpen={ this.state.modalOpen } onRequestClose={ this.closeModal.bind(this) } contentLabel="Modal" style={ NewTemplateModalStyles }>
          <NewEntity entityName="template" initialEntity={ { name: "" } } redirect={ true } />
        </Modal>
      </div>
    );
  }

  renderHeader() {
    if (window.location.pathname == "/admin/templates") {
      return (
        <div className="clearfix">
          <h1 className="templates-header">Object Templates</h1>
          <a className={ "orange-button float-button btn" + Admin.renderDisabledButtonClass(this.state.fetching) } onClick={ this.clickNew.bind(this) }>Add Template</a>
        </div>
      );
    } else if (window.location.pathname == "/admin/objects") {
      return (
        <h1>Objects</h1>
      );
    } else {
      return null;
    }
  }

  componentDidUpdate() {
    $('.match-height-layout').matchHeight();
  }
}

const mapStateToProps = (reducers, props) => {
  return reducers.standardReducer;
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(mapStateToProps, mapDispatchToProps)(TemplatesIndex);
