import React from "react";
import { connect } from "react-redux";
import { bindActionCreators } from "redux";

import Admin from "../../admin";
import { fetchEntities } from "../../actions/index";
import { TrackingLink, TrackingLinkView } from "./tracking-link-types";
import TrackingLinkLink from "./tracking-link-link";
import { TrackingLinkBulkUpload } from './tracking-link-bulk-upload';
interface TrackingLinksIndexState {
  fetching: boolean;
  modalOpen: boolean;
  trackingLinks: TrackingLink[];
}

interface TrackingLinksIndexProps {
  fetchEntities: any;
  trackingLinks: TrackingLink[];
  goToView: (view: TrackingLinkView, id?: string) => void;
}

class TrackingLinksIndex extends React.Component<
  TrackingLinksIndexProps,
  TrackingLinksIndexState
> {
  public state: TrackingLinksIndexState = {
    fetching: false,
    trackingLinks: [],
    modalOpen: false
  };

  constructor(props) {
    super(props);
  }

  filteredTrackingLinks() {
    return this.state.trackingLinks;
  }

  componentDidMount() {
    this.setState({
      fetching: true
    });
    this.props.fetchEntities("tracking_links", "trackingLinks").then(() => {
      this.setState({
        fetching: false,
        trackingLinks: this.props.trackingLinks
      });
    });
  }

  render() {
    return (
      <div id="tracking-links-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Tracking Links</h1>
          <TrackingLinkLink
                        goToView={this.props.goToView}
                        view="new"
                      >New</TrackingLinkLink>
          <TrackingLinkBulkUpload />
        </div>
        <div className="white-box">
          {Admin.renderSpinner(this.state.fetching)}
          {Admin.renderGrayedOut(this.state.fetching)}
          <table className="admin-table contains-links">
            <thead>
              <tr>
                <th>Slug</th>
                <th>Destination</th>
                <th>Active</th>
              </tr>
            </thead>
            <tbody>
              {this.state.trackingLinks.map(link => {
                return (
                  <tr key={link.id}>
                    <td>
                      <TrackingLinkLink
                        goToView={this.props.goToView}
                        id={link.id}
                        view="show"
                      >
                        {link.slug}
                      </TrackingLinkLink>
                    </td>
                    <td>{link.destinationUrl}</td>
                    <td>{link.status}</td>
                  </tr>
                );
              })}
            </tbody>
          </table>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (reducers, props) => {
  return {
    trackingLinks: reducers.standardReducer.trackingLinks as TrackingLink[]
  };
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntities }, dispatch);
}

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(TrackingLinksIndex);
