import React from "react";

import TrackingLinkLink from "./tracking-link-link";
import TrackingLinkForm from "./tracking-link-form";

import { TrackingLinkView } from "./tracking-link-types";

interface TrackingLinksNewProps {
  goToView: (view: TrackingLinkView, id?: string) => void;
}

// Should probably be replaced with redux
const createTrackingLink = (data) => {

  const csrfToken = (document.getElementsByName('csrf-token')[0] as HTMLMetaElement).content
  const convertedData = (window as any).Tools.convertObjectKeysToUnderscore(data)
  return fetch("/api/tracking_links", {
      method: 'POST',
      mode: 'cors',
      cache: 'no-cache',
      credentials: 'same-origin',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken
      },
      body: JSON.stringify({tracking_link: convertedData}),
  })
  .then(response => {
    if(response.ok) {
      return response.json()
    }
    console.error(response.json())
    throw new Error(`${response.status} err`)

  })

}
export default class TrackingLinksNew extends React.Component<
  TrackingLinksNewProps,
  any
> {
  public state = {
    fetching: false,
    trackingLink: {
      campaign: {
        channel: "podcast",
      },
      destinationUrl: "https://example.com"
    }
  };

  constructor(props) {
    super(props);
    this.onSubmit = this.onSubmit.bind(this)
  }

  async onSubmit(data) {
    console.log(data)

    this.setState({fetching: true})
    const {trackingLink} = await createTrackingLink(data.formData)

    this.setState({fetching: false})
    this.props.goToView("show", trackingLink.id.toString())

  }

  render() {
    return (
      <div id="tracking-links-index" className="component">
        <div className="white-box">
          {this.state.fetching ? "Loading.." : ""}
          <TrackingLinkLink goToView={this.props.goToView} view="index">
            Home
          </TrackingLinkLink>
          <TrackingLinkForm trackingLink={this.state.trackingLink} onSubmit={this.onSubmit}/>
        </div>
      </div>
    );
  }
}
