import React from "react";
import { connect } from "react-redux";
import { bindActionCreators } from "redux";
import { TrackingLink, TrackingLinkView } from "./tracking-link-types";
import TrackingLinkLink from "./tracking-link-link";

import { fetchEntity } from "../../actions/index";

interface TrackingLinksShowProps {
  id: string;
  goToView: (view: TrackingLinkView, id?: string) => void;
  fetchEntity: any;
  trackingLink?: TrackingLink;
}

interface TrackingLinksShowState {
  fetching: boolean;
  trackingLink?: TrackingLink;
}

class TrackingLinksShow extends React.Component<
  TrackingLinksShowProps,
  TrackingLinksShowState
> {
  public state: TrackingLinksShowState = {
    fetching: false
  };

  constructor(props) {
    super(props);
  }

  componentDidMount() {
    this.setState({
      fetching: true
    });
    this.props
      .fetchEntity({ directory: "tracking_links", id: this.props.id })
      .then(() => {
        this.setState({
          fetching: false,
          trackingLink: this.props.trackingLink
        });
      });
  }

  render() {
    return (
      <div id="tracking-links-index" className="component">
        <div className="clearfix">
          <h1 className="search-box-header">Link</h1>
        </div>
        <div className="white-box">
          <TrackingLinkLink goToView={this.props.goToView} view="index">
            Home
          </TrackingLinkLink>
          Tracking Link {this.props.id}
        </div>
      </div>
    );
  }
}

const mapStateToProps = (reducers, props) => {
  return {
    trackingLink: reducers.standardReducer.trackingLink as TrackingLink
  };
};

function mapDispatchToProps(dispatch) {
  return bindActionCreators({ fetchEntity }, dispatch);
}

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(TrackingLinksShow);
