import React from 'react'

import TrackingLinksIndex from "./tracking-link/tracking-links-index";
import TrackingLinksShow from "./tracking-link/tracking-links-show";
import TrackingLinksNew from "./tracking-link/tracking-links-new";

import { TrackingLinkView } from "./tracking-link/tracking-link-types";

import { getQueryParams, toQueryString }  from "./tracking-link/utils"

interface TrackingLinksRouterProps {
}



interface TrackingLinksRouterState {
  id?: string;
  view: TrackingLinkView
}
const defaultView: TrackingLinkView = "index"

const getStateFromQueryString = (): TrackingLinksRouterState => {
  const queryObj = getQueryParams()
  const view = (queryObj.view || defaultView) as TrackingLinkView
  const { id } = queryObj
  return { id, view }
}

// Entrypoint for all tracking link related pages so we can switch
// pages without a full page reload
export default class TrackingLinksRouter extends React.Component<TrackingLinksRouterProps, TrackingLinksRouterState> {

  constructor(props) {
    super(props)

    this.state = getStateFromQueryString()
    this.goToView = this.goToView.bind(this);
    this.setStateFromHistory = this.setStateFromHistory.bind(this);
  }

  componentDidMount() {
    const queryState = getStateFromQueryString()
    window.addEventListener("popstate", this.setStateFromHistory);

    console.log(queryState)
    this.setState(
      queryState
     )
  }

  setStateFromHistory(e: PopStateEvent) {

    this.setState(e.state.data)
  }
  // Navigate between tracking links 'views' by setting the query string
  // and pushing it into the state
  goToView(view: TrackingLinkView, id?: string) {
    const data = {view, id}
    const newQs = toQueryString(data)
    history.pushState({data}, 'Link page', `?${newQs}`)

    this.setState({
      view,
      id
    })
  }

  render() {

    switch(this.state.view) {
      case "index":
      return <TrackingLinksIndex goToView={this.goToView} />;
       break;

      case "show":
      return <TrackingLinksShow id={this.state.id} goToView={this.goToView} />;
      case "new":
      return <TrackingLinksNew goToView={this.goToView} />;
      default:
        return <h1>Page not found</h1>;
    }
  }
}
