import * as React from 'react'

const { useEffect, useCallback, useRef, useImperativeHandle } = React;

type BoolThunk = boolean | (() => boolean)
export const useExitPrompt = (shouldPrompt: BoolThunk) => {
  const onBeforeUnmount = useCallback((e: BeforeUnloadEvent) => {
    const booleanShouldPrompt = typeof shouldPrompt === 'boolean' && shouldPrompt

    const functionShouldPrompt = typeof shouldPrompt === 'function' && shouldPrompt()
    if (booleanShouldPrompt || functionShouldPrompt) {
      e.preventDefault()
      e.returnValue = ''
    }
  }, [shouldPrompt])
  useEffect(() => {
    window.addEventListener('beforeunload', onBeforeUnmount)

    return () => window.removeEventListener('beforeunload', onBeforeUnmount)
  }, [onBeforeUnmount])
}

export const withExitPrompt = <P extends {}, S extends { changesToSave?: boolean }, T extends React.Component<P, S>>
  (Element: React.ComponentClass<P, S>) => {

  const wrappedComponent = (props: P, ref: React.RefObject<T>)  => {
    const componentRef = useRef<T>()

    useImperativeHandle(ref, () => componentRef.current, [componentRef.current])

    const checkShouldPrompt = useCallback(
      () => componentRef.current && componentRef.current.state.changesToSave,
      [componentRef.current]
    )

    useExitPrompt(checkShouldPrompt);

    return <Element {...props} ref={componentRef} />
  }

  return React.forwardRef<T, P>(wrappedComponent);
}
