module Iterable
  extend WebinarBonuses
  @@api_url = 'https://api.iterable.com:443/api/'

  def self.determine_iterable_instance(instance_id)
    instance_id = instance_id ? instance_id.to_i : 0
    instances = [{ instance_id: 0, name: 'staging', api_key: ENV['ITERABLE_STAGING_API_KEY'] },
                 { instance_id: 1, name: 'teachable', api_key: ENV['ITERABLE_TEACHABLE_API_KEY'] },
                 { instance_id: 2, name: 'enterprise', api_key: ENV['ITERABLE_ENTERPRISE_API_KEY'] },
                 { instance_id: 3, name: 'affiliate', api_key: ENV['ITERABLE_AFFILIATE_API_KEY'] },
                 { instance_id: 4, name: 'vizia', api_key: ENV['ITERABLE_VIZIA_API_KEY'] }]
    instances.detect { |h| h[:instance_id] == instance_id }[:api_key]
  end

  def self.send_iterable_event(obj)
    defaults = { data: {} }
    obj = defaults.merge(obj)
    api_key = Iterable.determine_iterable_instance(obj[:instance_id])
    endpoint = @@api_url + 'events/track?api_key=' + api_key
    body = {
      'email' => obj[:email],
      'eventName' => obj[:event_name],
      'dataFields' => Iterable.parse_json(obj[:data]),
      'templateId' => obj[:template_id].to_i,
      'campaignId' => obj[:campaign_id].to_i
    }
    response = HTTParty.post(endpoint, headers: { 'Content-Type' => 'application/json' }, body: body.to_json)
    p response
    if response.code != 200
      Rails.logger.info(response)
    end
  end

  def self.parse_json(str)
    JSON.parse(str.gsub("=>", ":")) rescue str
  end

  def self.trigger_workflow(instance_id, email, workflow_id, data_fields)
    api_key = Iterable.determine_iterable_instance(instance_id)
    endpoint = @@api_url + 'workflows/triggerWorkflow?api_key=' + api_key
    HTTParty.post(endpoint,
                  headers: { 'Content-Type' => 'application/json' },
                  body: {
                    email: email,
                    workflowId: workflow_id.to_i,
                    dataFields: data_fields
                  }.to_json)
  end

  def self.get_user_by_email(instance_id, email)
    api_key = Iterable.determine_iterable_instance(instance_id)
    endpoint = @@api_url + 'users/' + URI.encode(email) + '?api_key=' + api_key
    HTTParty.get(endpoint)
  end

  def self.update_subscription_preferences(instance_id, email, unsubscribed_message_type_ids, unsubscribed_channel_ids)
    api_key = Iterable.determine_iterable_instance(instance_id)
    endpoint = @@api_url + 'users/updateSubscriptions?api_key=' + api_key
    HTTParty.post(endpoint,
                  headers: { 'Content-Type' => 'application/json' },
                  body: {
                    email: email,
                    unsubscribedMessageTypeIds: unsubscribed_message_type_ids,
                    unsubscribedChannelIds: unsubscribed_channel_ids
                  }.to_json)
  end

  def self.update_webinar(webinar_id)
    webinar = Webinar.find(webinar_id)
    hosts = webinar.hosts.map do |host|
      field_values = host.field_values_hash(extend_image_urls: true)
      {
        name: field_values["Name"],
        title: field_values["Title"],
        image: field_values["Image"]
      }
    end
    bonuses, total_bonus_value = create_bonuses_array(webinar)
    api_key = Iterable.determine_iterable_instance(1)
    endpoint = @@api_url + "metadata/webinarsbyid/#{webinar_id}?api_key=#{api_key}"
    HTTParty.put(endpoint,
                  headers: { 'Content-Type' => 'application/json' },
                  body: {
                    value: {
                      url: webinar.url,
                      title: webinar.title,
                      hosts: hosts,
                      upgrade_src: {
                        default: webinar.url,
                        play: "#{webinar.url}-play",
                        replay: "#{webinar.url}-replay"
                      },
                      total_bonus_value: total_bonus_value,
                      bonuses: bonuses,
                      datetime: webinar.webinar_type == "automated" ? "" : webinar.proper_time1.strftime("%F %T %z"),
                      datetime2: webinar.time2 ? webinar.time2.strftime('%F %T %z') : ''
                    }
                  }.to_json)
  end

  def self.register_token(instance_id, token, email)
    api_key = Iterable.determine_iterable_instance(instance_id)
    endpoint = @@api_url + 'users/registerBrowserToken?api_key=' + api_key
    HTTParty.post(endpoint,
                  headers: { 'Content-Type' => 'application/json' },
                  body: {
                    email: email,
                    browserToken: token
                  }.to_json)
  end

end
