require "uri"

module Pendo

  INTEGRATION_KEY ||= ENV.fetch("PENDO_INTEGRATION_KEY", "")
  HOST ||= "app.pendo.io"
  VERSION ||= "v1"
  TRACK_EVENT_URL ||= URI::HTTPS.build(
    host: HOST,
    path: "/" + ["api", VERSION, "track"].join("/")
  ).to_s

  def self.track_event(event, visitor_id, account_id, timestamp, properties)
    return if Rails.env.test?
    # https://help.pendo.io/resources/support-library/integrations/track-events.html
    # https://help.pendo.io/resources/support-library/integrations/track-event-configuration.html
    payload = {
      type: "track",
      event: event,
      visitorId: visitor_id,
      accountId: account_id,
      timestamp: timestamp,
      properties: properties,
    }

    HTTParty.post(
      TRACK_EVENT_URL,
      headers: {
        "Content-Type" => "application/json",
        'x-pendo-integration-key': INTEGRATION_KEY,
      },
      body: payload.to_json
    )
  end

end
