module Temi

  def self.get_transcripts
    results = HTTParty.get("https://api.temi.com/v1/jobs", headers: { "Authorization" => "Bearer #{ENV["TEMI_AUTH_TOKEN"]}" })
    if results.code == 200
      results.to_a
    else
      raise "Issue getting transcripts: #{results.body}"
    end
  end

  def self.get_transcript(id)
    results = HTTParty.get(
      "https://api.temi.com/v1/jobs/#{id}/transcript?version=latest",
      headers: {
        "Authorization" => "Bearer #{ENV["TEMI_AUTH_TOKEN"]}",
        "Accept" => "application/json",
      }
    )
    if results.code == 200
      raw_data = JSON.parse(results)
      format_transcript(raw_data)
    else
      raise "Issue getting transcripts: #{results.body}"
    end
  end

  def self.format_transcript(raw_data)
    speakers = raw_data.dig("speakers")
    results = raw_data.dig("monologues").map do |monologue|
      content = monologue["elements"].reject { |item| item["end_ts"].nil? }
      [monologue["speaker"], content]
    end

    results.map do |result|
      [
        speakers.find { |speaker| speaker["id"] == result[0] }["name"],
        result[1],
      ]
    end
  end

end
